using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualBasic.FileIO;
using System.IO;
using System.Windows.Forms;
using System.Diagnostics;
using Converters;
using System.Drawing;

namespace CodeCommander
{
    class DictExecute : IDictProcess
    {
        #region Public Event
        public event EventHandler Close;
        #endregion

        #region Private Fields
        private int preferredHeight;
        private System.Windows.Forms.WebBrowser web;
        private o2Mate.Dictionnaire inputDict, outputDict;
        private string fileNameFinal;
        private string fileNameSrc;
        #endregion

        #region Default Constructor
        public DictExecute(System.Windows.Forms.WebBrowser web, o2Mate.Dictionnaire inputDict, o2Mate.Dictionnaire outputDict, string fileName)
        {
            this.web = web;
            this.inputDict = inputDict;
            this.outputDict = outputDict;
            this.fileNameSrc = fileName;
            this.fileNameFinal = Path.Combine(Documents.TempDirectory, "execute.htm");
        }
        #endregion

        #region IDictProcess Membres

        public void Load()
        {
            o2Mate.Dictionnaire IHMdict = new o2Mate.Dictionnaire();
            string fileNameDict = Path.Combine(SpecialDirectories.Temp, FileSystem.GetTempFileName());
            IHMdict.Save(fileNameDict);
            o2Mate.Compilateur comp = new o2Mate.Compilateur();
            comp.LoadTemplates(Documents.GeneratedDictionariesTemplatesDirectory);
            comp.Compilation(Documents.ExecutePage, fileNameDict, this.fileNameFinal, null);
            this.preferredHeight = 300;
        }

        public string ExecuteAndRead()
        {
            string result = String.Empty;
            string fileNameDict = Path.Combine(SpecialDirectories.Temp, FileSystem.GetTempFileName());
            this.outputDict.Save(fileNameDict);
            o2Mate.Compilateur comp = new o2Mate.Compilateur();
            comp.LoadTemplates(Documents.TemplatesDirectory);
            comp.LoadSkeletons(Documents.SkeletonsDirectory);
            FileInfo fi = new FileInfo(this.fileNameSrc);
            string outputFinalFile = Documents.BuildDirectory + Path.GetFileNameWithoutExtension(fi.Name) + ".txt";
            try
            {
                comp.Compilation(this.fileNameSrc, fileNameDict, outputFinalFile, null);
                StreamReader sr = new StreamReader(outputFinalFile);
                result = sr.ReadToEnd();
                sr.Close();
                sr.Dispose();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            if (this.Close != null)
                this.Close(this, new EventArgs());
            return result;
        }

        public void Navigate()
        {
            this.web.DocumentCompleted += new System.Windows.Forms.WebBrowserDocumentCompletedEventHandler(web_DocumentCompleted);
            this.web.Navigate(this.fileNameFinal);
        }

        void web_DocumentCompleted(object sender, System.Windows.Forms.WebBrowserDocumentCompletedEventArgs e)
        {
            if (this.preferredHeight < this.web.Parent.MaximumSize.Height)
                this.web.Parent.Size = new Size(this.web.Parent.Size.Width, this.preferredHeight);
            else
                this.web.Parent.Size = new Size(this.web.Parent.Size.Width, this.web.Parent.MaximumSize.Height);
            this.web.Document.GetElementById("exec").Click += new System.Windows.Forms.HtmlElementEventHandler(DictExecute_Exec);
            this.web.Document.GetElementById("vbscript").Click += new HtmlElementEventHandler(VBScript_Click);
            this.web.Document.GetElementById("powershell").Click += new HtmlElementEventHandler(PowerShell_Click);
            this.web.DocumentCompleted -= new System.Windows.Forms.WebBrowserDocumentCompletedEventHandler(web_DocumentCompleted);
        }

        void PowerShell_Click(object sender, HtmlElementEventArgs e)
        {
            string fileNameDict = Path.Combine(SpecialDirectories.Temp, FileSystem.GetTempFileName());
            this.outputDict.Save(fileNameDict);
            o2Mate.Compilateur comp = new o2Mate.Compilateur();
            FileInfo fi = new FileInfo(this.fileNameSrc);
            string outputFinalFile = Documents.BuildDirectory + Path.GetFileNameWithoutExtension(fi.Name) + ".ps1";
            try
            {
                comp.UnderConversion = true;
                comp.ConvertedLanguage = new Converters.PowerShellConverter();
                comp.LoadTemplates(Documents.TemplatesDirectory);
                comp.LoadSkeletons(Documents.SkeletonsDirectory);
                comp.Convert(comp.ConvertedLanguage, this.fileNameSrc, fileNameDict, outputFinalFile);
                Process proc = new Process();
                if (global::CodeCommander.Properties.Settings.Default.AcceptExecutingScript && global::CodeCommander.Properties.Settings.Default.PowerShellInstalled)
                {
                    proc.StartInfo.Verb = global::CodeCommander.Properties.Settings.Default.ExecPowerShellVerb;
                }
                else
                {
                    proc.StartInfo.Verb = "Open";
                }
                proc.StartInfo.FileName = outputFinalFile;
                proc.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            if (this.Close != null)
                this.Close(this, new EventArgs());
        }

        void VBScript_Click(object sender, HtmlElementEventArgs e)
        {
            string fileNameDict = Path.Combine(SpecialDirectories.Temp, FileSystem.GetTempFileName());
            this.outputDict.Save(fileNameDict);
            o2Mate.Compilateur comp = new o2Mate.Compilateur();
            FileInfo fi = new FileInfo(this.fileNameSrc);
            string outputFinalFile = Documents.BuildDirectory + Path.GetFileNameWithoutExtension(fi.Name) + ".vbs";
            try
            {
                comp.UnderConversion = true;
                comp.ConvertedLanguage = new Converters.VBScriptConverter();
                comp.LoadTemplates(Documents.TemplatesDirectory);
                comp.LoadSkeletons(Documents.SkeletonsDirectory);
                comp.Convert(comp.ConvertedLanguage, this.fileNameSrc, fileNameDict, outputFinalFile);
                Process proc = new Process();
                if (global::CodeCommander.Properties.Settings.Default.AcceptExecutingScript)
                {
                    proc.StartInfo.Verb = "Open";
                }
                else
                {
                    proc.StartInfo.Verb = "Edit";
                }
                proc.StartInfo.FileName = outputFinalFile;
                proc.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            if (this.Close != null)
                this.Close(this, new EventArgs());
        }

        void DictExecute_Exec(object sender, System.Windows.Forms.HtmlElementEventArgs e)
        {
            string fileNameDict = Path.Combine(SpecialDirectories.Temp, FileSystem.GetTempFileName());
            this.outputDict.Save(fileNameDict);
            o2Mate.Compilateur comp = new o2Mate.Compilateur();
            comp.LoadTemplates(Documents.TemplatesDirectory);
            comp.LoadSkeletons(Documents.SkeletonsDirectory);
            FileInfo fi = new FileInfo(this.fileNameSrc);
            string outputFinalFile = Documents.BuildDirectory + Path.GetFileNameWithoutExtension(fi.Name) + ".txt";
            try
            {
                comp.Compilation(this.fileNameSrc, fileNameDict, outputFinalFile, null);
                Process proc = new Process();
                proc.StartInfo.FileName = outputFinalFile;
                proc.Start();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
            if (this.Close != null)
                this.Close(this, new EventArgs());
        }

        public IDictProcess PreviousPage
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        public IDictProcess NextPage
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        public IDictProcess TablePage
        {
            get
            {
                throw new Exception("The method or operation is not implemented.");
            }
            set
            {
                throw new Exception("The method or operation is not implemented.");
            }
        }

        #endregion
    }
}
