using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NotifyProgress;

namespace CodeCommander
{
    public partial class PopupProgress : Form, INotifyProgress
    {
        private bool isFinished;
        private delegate void SetCountDelegate(int count);
        private delegate void CloseDelegate();
        private delegate void SetProgressDelegate(int position);

        public bool IsFinished
        {
            get { return this.isFinished; }
        }

        private void ThreadSafeClose()
        {
            this.Close();
        }

        private void ThreadSafeSetCount(int count)
        {
            if (this.progressBar1.Maximum != count)
                this.progressBar1.Value = 0;
            this.progressBar1.Maximum = count;
        }

        private void ThreadSafeSetProgress(int position)
        {
            this.progressBar1.Step = position;
            this.progressBar1.PerformStep();
        }

        public PopupProgress()
        {
            InitializeComponent();
            this.progressBar1.Value = 0;
        }

        #region INotifyProgress Membres

        public void Start()
        {
            this.isFinished = false;
        }

        public void SetCount(int count)
        {
            if (this.IsHandleCreated)
                this.Invoke(new SetCountDelegate(ThreadSafeSetCount), count);
        }

        public void SetProgress(int position)
        {
            if (this.IsHandleCreated)
                this.Invoke(new SetProgressDelegate(ThreadSafeSetProgress), position);
        }

        public void Terminate(bool noError)
        {
            if (this.IsHandleCreated)
                this.Invoke(new CloseDelegate(ThreadSafeClose));
            this.isFinished = true;
        }

        public void GiveException(Exception ex)
        {
            MessageBox.Show(ex.ToString(), "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            if (this.IsHandleCreated)
                this.Invoke(new CloseDelegate(ThreadSafeClose));
        }

        #endregion
    }
}