using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace Converters
{
    public class Function : IFunction
    {
        #region Private Constants
        private readonly string IndentString = "    ";
        #endregion

        #region Private Fields
        private string name;
        private string source;
        private string cacheSource;
        private string additionalSource;
        private List<IParameter> parameters;
        private List<string> locals;
        private List<string> used;
        private int indentSize;
        #endregion

        #region Default Constructor
        public Function()
        {
            this.parameters = new List<IParameter>();
            this.locals = new List<string>();
            this.used = new List<string>();
            this.source = "";
            this.cacheSource = "";
            this.additionalSource = "";
        }
        #endregion

        #region Public Properties
        public int IndentSize
        {
            get { return this.indentSize; }
            set { this.indentSize = value; }
        }
        #endregion

        #region Private Methods
        private string Repeat(string input, int round)
        {
            string output = "";
            for (int counter = 0; counter < round; ++counter)
            {
                output += input;
            }
            return output;
        }

        private string IndentSource(int indent, string source)
        {
            string output = "";
            Regex reg = new Regex("((.*" + Environment.NewLine + ")|.*$)");
            MatchCollection matches = reg.Matches(source);
            foreach (Match m in matches)
            {
                if (m.Success)
                {
                    if (!String.IsNullOrEmpty(m.Groups[1].Value))
                        output += this.Repeat(this.IndentString, this.IndentSize) + m.Groups[1].Value;
                }
            }
            return output;
        }
        #endregion

        #region IFunction Membres

        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                this.name = value;
            }
        }

        public List<IParameter> Parameters
        {
            get { return this.parameters; }
        }

        public List<string> LocalVariables
        {
            get { return this.locals; }
        }

        public List<string> UsedVariables
        {
            get { return this.used; }
        }

        public string Source
        {
            get
            {
                return this.source;
            }
            set
            {
                this.source = value;
            }
        }

        public string CacheSource
        {
            get
            {
                return this.cacheSource;
            }
            set
            {
                this.cacheSource = value;
            }
        }

        public string AdditionalSource
        {
            get
            {
                return this.additionalSource;
            }
            set
            {
                this.additionalSource = value;
            }
        }

        public void AddToSource(string source)
        {
            this.Source += this.IndentSource(this.IndentSize, source);
        }
        #endregion
    }
}
