using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace Converters
{
    public class PowerShellConverter : ICodeConverter
    {
        #region Private Constants
        private readonly string IndentString = "    ";
        #endregion

        #region Private Fields
        private string functionsFileName;
        private string principalFileName;
        private List<IFunction> implementedFunctions;
        private List<IFunction> callingFunctions;
        private IFunction main;
        private IFunction currentFunction;
        private Stack<IFunction> stack;
        #endregion

        #region Default Constructor
        public PowerShellConverter()
        {
            this.callingFunctions = new List<IFunction>();
            this.implementedFunctions = new List<IFunction>();
            this.stack = new Stack<IFunction>();
            this.main = new Function();
            this.currentFunction = this.main;
        }
        #endregion

        #region ICodeConverter Membres

        public string LanguageName
        {
            get { return "PowerShell"; }
        }

        public string FunctionsFileName
        {
            get { return this.functionsFileName; }
            set { this.functionsFileName = value; }
        }

        public string PrincipalFileName
        {
            get { return this.principalFileName; }
            set { this.principalFileName = value; }
        }

        public List<IFunction> ImplementedFunctions
        {
            get { return this.implementedFunctions; }
        }

        public List<IFunction> CallingFunctions
        {
            get { return this.callingFunctions; }
        }

        public IFunction Main
        {
            get { return this.main; }
        }

        public IFunction CurrentFunction
        {
            get
            {
                return this.currentFunction;
            }
            set
            {
                this.currentFunction = value;
            }
        }

        public void Convert(ILanguageConverter obj)
        {
            obj.WriteInPowerShell(this);
        }

        public void PushFunction(IFunction function)
        {
            this.stack.Push(function);
        }

        public IFunction PopFunction()
        {
            return this.stack.Pop();
        }

        #endregion

        #region Private Methods
        private string IndentSource(string source)
        {
            string output = "";
            Regex reg = new Regex("(.*)" + Environment.NewLine);
            MatchCollection matches = reg.Matches(source);
            foreach (Match m in matches)
            {
                if (m.Success)
                {
                    output += this.IndentString + m.Groups[1].Value + Environment.NewLine;
                }
            }
            return output;
        }
        #endregion

        #region Public Methods
        public void WriteToFile(string fileDict, string fileName)
        {
            StreamWriter sw = new StreamWriter(Path.GetDirectoryName(fileName) + "\\" + this.functionsFileName, false, Encoding.GetEncoding(1252));
            sw.WriteLine(@"

$trueValue = 1
$falseValue = 0
$SpaceChar = " + "\"" + " " + "\"" + @"
$PlusChar = " + "\"" + "+" + "\"" + @"
$MinusChar = " + "\"" + "-" + "\"" + @"
$StarChar = " + "\"" + "*" + "\"" + @"
$SlashChar = " + "\"" + "/" + "\"" + @"
$ColonChar = " + "\"" + ":" + "\"" + @"
$PointChar = " + "\"" + "." + "\"" + @"
$ExChar = " + "\"" + "!" + "\"" + @"
$IntChar = " + "\"" + "?" + "\"" + @"
$EmptyChar = " + "\"" + "" + "\"" + @"
$InfChar = " + "\"" + "<" + "\"" + @"
$SupChar = " + "\"" + ">" + "\"" + @"
$crlf = " + "\"" + Environment.NewLine + "\"" + @"

$theDict = New-Object -ComObject o2Mate.Dictionnaire
$theDict.Load(" + "\"" + fileDict + "\"" + @")


function CreateDirectoryIfNotExists($dir)
{
    if (! (Test-Path $dir))
    {
        New-Item $dir -type directory
    }
}

function EraseFile($fileName)
{
    if (Test-Path $fileName)
    {
        New-Item $fileName -type file -force
    }
}

function Indent($writer)
{
    $writer[1]++;
}

function Unindent($writer)
{
    if ($writer[1] -gt 0)
    {
        $writer[1]--;
    }
}

function WriteToFile($writer, $text)
{
    $m = $text | select-string (" + "\"((.*\" + $crlf + \")|.*$)\") -allmatches" + @"
    foreach($k in $m.matches)
    {
        if ($writer[3])
        {
            if ($k.Value)
            {
                for($i=0;$i -lt $writer[1];++$i) { $writer[2] += " + "\"    \"" + @" }
                if ($k.Value.Length -gt 1) {
                    if ($k.Value.Substring($k.Value.Length-2,2) -ne $crlf)
                    {
                        $writer[2] += $k.Value
                        $writer[3] = $false
                    }
                    else
                    {
                        $writer[2] += $k.Value.Substring(0, $k.Value.Length-2)
                        $writer[2] | Out-File $writer[0] -Append
                        $writer[2] = " + "\"\"" + @"
                    }
                }
                else
                {
                    $writer[2] += $k.Value
                    $writer[3] = $false
                }
            }
        }
        else
        {
            if ($k.Value)
            {
                if ($k.Value.Length -gt 1 -and $k.Value.Substring($k.Value.Length-2,2) -eq $crlf)
                {
                    $writer[2] += $k.Value.Substring(0, $k.Value.Length-2)
                    $writer[2] | Out-File $writer[0] -Append
                    $writer[2] = " + "\"\"" + @"
                    $writer[3] = $true
                }
                else
                {
                    $writer[2] += $k.Value
                }
            }
        }
    }
}

function getValue($val)
{
   if ($val.GetType().Name.EndsWith(" + "\"" + @"PSReference" + "\"" + @"))
   {
       return $val.value;
   }
   else
   {
       return $val;
   }
}

");
            foreach (IFunction f in this.implementedFunctions)
            {
                sw.Write("function " + f.Name + "(");
                bool first = true;
                foreach (IParameter p in f.Parameters)
                {
                    if (!first) { sw.Write(","); } else { first = false; }
                    if (p.IsMutableParameter) { sw.Write("[ref] "); }
                    sw.Write("$");
                    sw.Write(p.FormalParameter);
                }
                sw.WriteLine(")");
                sw.WriteLine("{");
                sw.WriteLine(this.IndentSource(f.Source));
                sw.WriteLine("}");
            }
            sw.WriteLine();
            sw.Close();
            sw.Dispose();
            sw = new StreamWriter(this.principalFileName, false, Encoding.GetEncoding(1252));
            sw.WriteLine(@"
function Get-ScriptDirectory() {
  $invocation = (Get-Variable MyInvocation -Scope 1).Value
  Split-Path $invocation.MyCommand.Path
}

");
            sw.WriteLine("$functions = Join-Path (Get-ScriptDirectory) \"" + this.functionsFileName + "\"" + Environment.NewLine);
            sw.WriteLine(". $functions" + Environment.NewLine);
            sw.WriteLine(@"

$defaultWriter = @(" + "\"" + Path.GetDirectoryName(fileName) + Path.DirectorySeparatorChar + Path.GetFileNameWithoutExtension(fileName) + ".txt" + "\"" + @", 0, " + "\"\"" + @", $true)
EraseFile(" + "\"" + Path.GetDirectoryName(fileName) + Path.DirectorySeparatorChar + Path.GetFileNameWithoutExtension(fileName) + ".txt" + "\"" + @")

");
            sw.WriteLine(this.main.Source);
            sw.Close();
            sw.Dispose();
        }
        #endregion
    }
}
