using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace Converters
{
    public class VBScriptConverter : ICodeConverter
    {
        #region Private Constants
        private readonly string IndentString = "    ";
        #endregion

        #region Private Fields
        private string functionsFileName;
        private string principalFileName;
        private List<IFunction> implementedFunctions;
        private List<IFunction> callingFunctions;
        private IFunction main;
        private IFunction currentFunction;
        private Stack<IFunction> stack;
        #endregion

        #region Default Constructor
        public VBScriptConverter()
        {
            this.callingFunctions = new List<IFunction>();
            this.implementedFunctions = new List<IFunction>();
            this.stack = new Stack<IFunction>();
            this.main = new Function();
            this.currentFunction = this.main;
        }
        #endregion

        #region ICodeConverter Membres

        public string LanguageName
        {
            get { return "VBScript"; }
        }

        public string FunctionsFileName
        {
            get { return this.functionsFileName; }
            set { this.functionsFileName = value; }
        }

        public string PrincipalFileName
        {
            get { return this.principalFileName; }
            set { this.principalFileName = value; }
        }

        public List<IFunction> ImplementedFunctions
        {
            get { return this.implementedFunctions; }
        }

        public List<IFunction> CallingFunctions
        {
            get { return this.callingFunctions; }
        }

        public IFunction Main
        {
            get { return this.main; }
        }

        public IFunction CurrentFunction
        {
            get
            {
                return this.currentFunction;
            }
            set
            {
                this.currentFunction = value;
            }
        }

        public void Convert(ILanguageConverter obj)
        {
            obj.WriteInVBScript(this);
        }

        public void PushFunction(IFunction function)
        {
            this.stack.Push(function);
        }

        public IFunction PopFunction()
        {
            return this.stack.Pop();
        }

        #endregion

        #region Private Methods
        private string IndentSource(string source)
        {
            string output = "";
            Regex reg = new Regex("(.*)" + Environment.NewLine);
            MatchCollection matches = reg.Matches(source);
            foreach (Match m in matches)
            {
                if (m.Success)
                {
                    output += this.IndentString + m.Groups[1].Value + Environment.NewLine;
                }
            }
            return output;
        }
        #endregion

        #region Public Methods
        public void WriteToFile(string fileDict, string fileName)
        {
            StreamWriter sw = new StreamWriter(fileName);
            sw.WriteLine(@"
Const ForAppending = 8
Const ForReading = 1
Const ForWriting = 2
Const Unicode = -2
Const SpaceChar = " + "\"" + " " + "\"" + @"
Const PlusChar = " + "\"" + "+" + "\"" + @"
Const MinusChar = " + "\"" + "-" + "\"" + @"
Const StarChar = " + "\"" + "*" + "\"" + @"
Const SlashChar = " + "\"" + "/" + "\"" + @"
Const ColonChar = " + "\"" + ":" + "\"" + @"
Const CommaChar = " + "\"" + "," + "\"" + @"
Const DotChar = " + "\"" + "." + "\"" + @"
Const ExChar = " + "\"" + "!" + "\"" + @"
Const IntChar = " + "\"" + "?" + "\"" + @"
Const EmptyChar = " + "\"" + "" + "\"" + @"
Const InfChar = " + "\"" + "<" + "\"" + @"
Const SupChar = " + "\"" + ">" + "\"" + @"
CrLf = " + "vbCrLf" + @"

Dim theDict
Set theDict = CreateObject(" + "\"" + "o2Mate.Dictionnaire" +  "\")" + @"
theDict.Load " + "\"" + fileDict + "\"" + @"

Sub CreateDirectoryIfNotExists(dir)
    Dim fso
    Dim textStream
    Set fso = CreateObject(" + "\"Scripting.FileSystemObject\"" + @")
    If Not fso.FolderExists(dir) Then
	    fso.CreateFolder dir
	End If
    Set fso = Nothing
End Sub

Sub EraseFile(file)
    Dim fso
    Dim textStream
    Set fso = CreateObject(" + "\"Scripting.FileSystemObject\"" + @")
    If fso.FileExists(file) Then
        fso.DeleteFile file
    End If
    Set fso = Nothing
End Sub

Function WriteIndent(writer, text)
    Dim output
    Dim reg
    Dim matches
    Set reg = New RegExp
    output = vbNullString
    reg.Pattern = " + "\"(((.*\" + crlf + \")+).*$|.*$)\"" + @"
    reg.MultiLine = true
    Set matches = reg.Execute(text)
    For Each match in matches
        If writer(2) Then
            If Len(match.Value) > 0 Then
                Dim n
                For n = 1 To writer(1)
                    output = output & " + "\"    \"" + @"
                Next
                If InStr(match.Value, crlf) = -1 Then
                    output = output & match.Value
                    writer(2) = false
                Else
                    output = output & match.Value
                End If
            End If
        Else
            If Len(match.Value) > 0 Then
                If InStr(match.Value, crlf) = -1 Then
                    output = output & match.Value
                    writer(2) = true
                Else
                    output = output & match.Value
                End If
            End If
        End If
    Next
    WriteIndent = output
End Function

Sub Indent(writer)
    writer(1) = writer(1) + 1
End Sub

Sub Unindent(writer)
    If writer(1) > 0 Then
        writer(1) = writer(1) - 1
    End If
End Sub

Sub WriteToFile(writer, text)
    Dim fso
    Dim textStream
    Set fso = CreateObject(" + "\"Scripting.FileSystemObject\"" + @")
    If fso.FileExists(writer(0)) Then
        Set textStream = fso.OpenTextFile(writer(0), ForAppending, False, Unicode)
    Else
        Set textStream = fso.OpenTextFile(writer(0), ForWriting, True, Unicode)
    End If
    textStream.Write WriteIndent(writer, text)
    textStream.Close
    Set textStream = Nothing
    Set fso = Nothing
End Sub

trueValue = 1
falseValue = 0

defaultWriter = Array(" + "\"" + Path.GetDirectoryName(fileName) + Path.DirectorySeparatorChar + Path.GetFileNameWithoutExtension(fileName) + ".txt" + "\"" + @", 0, true)
EraseFile " + "\"" + Path.GetDirectoryName(fileName) + Path.DirectorySeparatorChar + Path.GetFileNameWithoutExtension(fileName) + ".txt" + "\"" + @"



");
            foreach (IFunction f in this.implementedFunctions)
            {
                sw.Write("Sub " + f.Name + "(");
                bool first = true;
                foreach (IParameter p in f.Parameters)
                {
                    if (!first) { sw.Write(","); } else { first = false; }
                    sw.Write(p.FormalParameter);
                }
                sw.WriteLine(")");
                sw.Write(this.IndentSource(f.Source));
                sw.WriteLine("End Sub");
            }
            sw.WriteLine();
            sw.WriteLine(this.main.Source);
            sw.Close();
            sw.Dispose();
        }
        #endregion
    }
}
