using System;
using System.Collections.Generic;
using System.Text;
using Converters;

namespace o2Mate
{
    internal class Affectation : Compiler, ILanguageConverter
    {
        #region Private Fields
        private string varName;
        private string expression;
        private int indent;
        private ICompilateur cachedComp;
        #endregion

        #region Public Properties
        public string VariableName
        {
            get { return this.varName; }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Affectation"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
            proc.CurrentScope.Add(this.varName, "", proc.Name);
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayAffectation", new object[] { this.varName, this.expression, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayAffectation", new object[] { this.varName, this.expression, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            this.varName = this.GetElementByName(elem, "varName").InnerText;
            this.expression = this.GetElementByName(elem, "expression").InnerText;
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("affectation");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("variable");
            writer.WriteString(this.varName);
            writer.WriteEndElement();
            writer.WriteStartElement("expression");
            writer.WriteString(this.expression);
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.varName = node.SelectSingleNode("variable").InnerText;
            this.expression = node.SelectSingleNode("expression").InnerText;
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.varName = node.SelectSingleNode("variable").InnerText;
            this.expression = node.SelectSingleNode("expression").InnerText;
            // on en a besoin pour plus tard dans la conversion
            this.cachedComp = comp;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            IData result = expr.Evaluate(this.expression, proc.CurrentScope);
            if (result != null)
            {
                if (proc.CurrentScope.Exists(this.varName))
                {
                    proc.CurrentScope.GetVariable(this.varName).Value = result.ValueString;
                }
                else
                {
                    proc.CurrentScope.Add(this.varName, result.ValueString, proc.Name);
                    proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "affectation", this.varName));
                }
            }
            else
            {
                if (proc.CurrentScope.Exists(this.varName))
                {
                    proc.CurrentScope.GetVariable(this.varName).Value = "0";
                }
                else
                {
                    proc.CurrentScope.Add(this.varName, "0", proc.Name);
                }
            }
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            expr.Convert(converter, this.expression, proc.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, proc, used, false);
            }
            if (!proc.CurrentScope.Exists(this.varName))
            {
                proc.CurrentScope.Add(this.varName, "0", proc.Name);
            }
            this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, proc, this.varName, true);
            converter.Convert(this);
        }

        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource(converter.CurrentFunction.AdditionalSource);
            converter.CurrentFunction.AddToSource(this.varName + " = " + converter.CurrentFunction.CacheSource + Environment.NewLine);
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource(converter.CurrentFunction.AdditionalSource);
            string local = converter.CurrentFunction.LocalVariables.Find(new Predicate<string>(delegate (string s) { return s == this.varName; }));
            if (local != null)
            {
                converter.CurrentFunction.AddToSource("$" + this.varName + " = " + converter.CurrentFunction.CacheSource + Environment.NewLine);
            }
            else
            {
                converter.CurrentFunction.AddToSource("$" + this.varName + ".value = " + converter.CurrentFunction.CacheSource + Environment.NewLine);
            }
        }

        #endregion
    }
}
