using System;
using System.Collections.Generic;
using System.Text;
using Converters;

namespace o2Mate
{
    internal class AffectationChaine : Compiler, ILanguageConverter
    {
        #region Private Fields
        private string varName;
        private string stringName;
        private int indent;
        private ICompilateur cachedComp;
        #endregion

        #region Public Properties
        public string VariableName
        {
            get { return this.varName; }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "AffectationChaine"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
            if (!proc.CurrentDictionnaire.Exists(this.stringName))
            {
                proc.CurrentDictionnaire.AddString(this.stringName, "");
            }
            proc.CurrentScope.Add(this.varName, "", proc.Name);
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.varName = node.SelectSingleNode("variable").InnerText;
            this.stringName = node.SelectSingleNode("chaine").InnerText;
            // on en a besoin pour plus tard  la conversion
            this.cachedComp = comp;
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayAffectationChaine", new object[] { this.varName, this.stringName, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayAffectationChaine", new object[] { this.varName, this.stringName, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.varName = this.GetElementByName(elem, "varName").InnerText;
            this.stringName = this.GetElementByName(elem, "stringName").InnerText;

        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("affectationchaine");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("variable");
            writer.WriteString(this.varName);
            writer.WriteEndElement();
            writer.WriteStartElement("chaine");
            writer.WriteString(this.stringName);
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.varName = node.SelectSingleNode("variable").InnerText;
            this.stringName = node.SelectSingleNode("chaine").InnerText;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            if (proc.CurrentDictionnaire.IsString(this.stringName))
            {
                if (proc.CurrentScope.Exists(this.varName))
                {
                    proc.CurrentScope.GetVariable(this.varName).Value = proc.CurrentDictionnaire.GetString(this.stringName);
                }
                else
                {
                    proc.CurrentScope.Add(this.varName, proc.CurrentDictionnaire.GetString(this.stringName), proc.Name);
                    proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "dictionary string", this.varName, this.stringName));
                }
            }
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
            converter.Convert(this);
            if (!proc.CurrentScope.Exists(this.varName))
            {
                proc.CurrentScope.Add(this.varName, "0", proc.Name);
            }
            this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, proc, this.varName, true);
        }

        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource(this.varName + " = theDict.GetString(\"" + this.stringName + "\")" + Environment.NewLine);
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource("$" + this.varName + " = $theDict.GetString(\"" + this.stringName + "\")" + Environment.NewLine);
        }

        #endregion
    }
}
