using System;
using System.Collections.Generic;
using System.Text;
using Converters;

namespace o2Mate
{
    internal class AffectationChamp : Compiler, ILanguageConverter
    {
        #region Private Fields
        private string varName;
        private string tabName;
        private string expression;
        private string fieldName;
        private int indent;
        private ICompilateur cachedComp;
        #endregion

        #region Public Properties
        public string VariableName
        {
            get { return this.varName; }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "AffectationChamp"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
            if (proc.CurrentDictionnaire.Exists(this.tabName))
            {
                if (proc.CurrentDictionnaire.IsArray(this.tabName))
                {
                    Array arr = proc.CurrentDictionnaire.GetArray(this.tabName) as Array;
                    Fields f = arr.Item(1) as Fields;
                    if (!f.Exists(this.fieldName))
                    {
                        f.AddString(this.fieldName, "");
                    }
                }
            }
            else
            {
                Array arr = new Array();
                Fields f = new Fields();
                f.AddString(this.fieldName, "");
                arr.Add(f);
                proc.CurrentDictionnaire.AddArray(this.tabName, arr);
            }
            proc.CurrentScope.Add(this.varName, "", proc.Name);
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayAffectationChamp", new object[] { this.varName, this.tabName, this.expression, this.fieldName, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayAffectationChamp", new object[] { this.varName, this.tabName, this.expression, this.fieldName, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.varName = this.GetElementByName(elem, "varName").InnerText;
            this.tabName = this.GetElementByName(elem, "tableau").InnerText;
            this.expression = this.GetElementByName(elem, "expression").InnerText;
            this.fieldName = this.GetElementByName(elem, "champ").InnerText;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("affectationchamp");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("variable");
            writer.WriteString(this.varName);
            writer.WriteEndElement();
            writer.WriteStartElement("champ");
            writer.WriteStartElement("tableau");
            writer.WriteString(this.tabName);
            writer.WriteEndElement();
            writer.WriteStartElement("expression");
            writer.WriteString(this.expression);
            writer.WriteEndElement();
            writer.WriteStartElement("variable");
            writer.WriteString(this.fieldName);
            writer.WriteEndElement();
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.varName = node.SelectSingleNode("variable").InnerText;
            this.tabName = node.SelectSingleNode("champ/tableau").InnerText;
            this.expression = node.SelectSingleNode("champ/expression").InnerText;
            this.fieldName = node.SelectSingleNode("champ/variable").InnerText;
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.varName = node.SelectSingleNode("variable").InnerText;
            this.tabName = node.SelectSingleNode("champ/tableau").InnerText;
            this.expression = node.SelectSingleNode("champ/expression").InnerText;
            this.fieldName = node.SelectSingleNode("champ/variable").InnerText;
            // on en a besoin pour plus tard dans la conversion
            this.cachedComp = comp;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            if (proc.CurrentDictionnaire.IsArray(this.tabName))
            {
                Array arr = proc.CurrentDictionnaire.GetArray(this.tabName) as Array;
                o2Mate.Expression expr = new o2Mate.Expression();
                IData result = expr.Evaluate(this.expression, proc.CurrentScope);
                Fields f = null;
                if (result != null)
                {
                    if (result.ValueInt <= arr.Count)
                    {
                        f = arr.Item(result.ValueInt) as Fields;
                    }
                }
                else
                {
                    if (arr.Count > 0)
                    {
                        f = arr.Item(1) as Fields;
                    }
                }
                if (f != null && f.Exists(this.fieldName))
                {
                    if (proc.CurrentScope.Exists(this.varName))
                    {
                        proc.CurrentScope.GetVariable(this.varName).Value = f.GetString(this.fieldName);
                    }
                    else
                    {
                        proc.CurrentScope.Add(this.varName, f.GetString(this.fieldName), proc.Name);
                        proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "dictionary field", this.varName, this.tabName, this.fieldName));
                    }
                }
                else
                {
                    if (proc.CurrentScope.Exists(this.varName))
                    {
                        proc.CurrentScope.GetVariable(this.varName).Value = "0";
                    }
                    else
                    {
                        proc.CurrentScope.Add(this.varName, "0", proc.Name);
                    }
                }
            }
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
            // pas de test sur l'existence du tableau dans le dictionnaire
            o2Mate.Expression expr = new o2Mate.Expression();
            expr.Convert(converter, this.expression, proc.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, proc, used, false);
            }
            converter.Convert(this);
            if (!proc.CurrentScope.Exists(this.varName))
            {
                proc.CurrentScope.Add(this.varName, "0", proc.Name);
            }
            this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, proc, this.varName, true);
        }
        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource(converter.CurrentFunction.AdditionalSource);
            converter.CurrentFunction.AddToSource(this.varName + " = theDict.GetArray(\"" + this.tabName + "\").Item(" + converter.CurrentFunction.CacheSource + ").GetString(\"" + this.fieldName + "\")" + Environment.NewLine);
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource(converter.CurrentFunction.AdditionalSource);
            converter.CurrentFunction.AddToSource("$" + this.varName + " = $theDict.GetArray(\"" + this.tabName + "\").Item(" + converter.CurrentFunction.CacheSource + ").GetString(\"" + this.fieldName + "\")" + Environment.NewLine);
        }

        #endregion
    }
}
