using System;
using System.Collections.Generic;
using System.Text;
using Converters;

namespace o2Mate
{
    internal class Call : Compiler, ILanguageConverter
    {
        #region Private Fields
        private string processName;
        private int indent;
        private ICompilateur cachedComp;
        private IProcess cachedProc;
        #endregion

        #region Default Constructor
        public Call()
        {
            this.processName = "";
        }
        #endregion

        #region Public Properties
        public string ProcessName
        {
            get
            {
                return this.processName;
            }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Call"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayCall", new object[] { this.processName, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayCall", new object[] { this.processName, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.processName = this.GetElementByName(elem, "process").InnerText;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("call");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteString(this.processName);
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.processName = node.InnerText;
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.processName = node.InnerText;
            this.cachedComp = comp;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
            this.cachedProc = proc;
            if (converter.ImplementedFunctions.Exists(new Predicate<IFunction>(delegate(IFunction f) { return f.Name == "func_" + this.processName; })))
            {
                converter.Convert(this);
            }
            else
            {
                throw new Exception("Pour convertir le programme, les processus doivent tre dclars auparavant; le processus '" + this.processName + "' n'a pas t dclar auparavant");
            }
        }

        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
            IFunction f = converter.ImplementedFunctions.Find(new Predicate<IFunction>(delegate(IFunction func) { return func.Name == "func_" + this.processName; }));
            if (!converter.CallingFunctions.Exists(new Predicate<IFunction>(delegate(IFunction func) { return func.Name == "func_" + this.processName; })))
            {
                converter.CallingFunctions.Add(f);
            }
            converter.CurrentFunction.AddToSource(f.Name + " ");
            bool first = true;
            foreach (IParameter p in f.Parameters)
            {
                if (!first) { converter.CurrentFunction.AddToSource(","); } else { first = false; }
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedProc, p.EffectiveParameter, p.IsMutableParameter);
                converter.CurrentFunction.AddToSource(p.EffectiveParameter);
            }
            converter.CurrentFunction.AddToSource(Environment.NewLine);
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            IFunction f = converter.ImplementedFunctions.Find(new Predicate<IFunction>(delegate(IFunction func) { return func.Name == "func_" + this.processName; }));
            if (!converter.CallingFunctions.Exists(new Predicate<IFunction>(delegate(IFunction func) { return func.Name == "func_" + this.processName; })))
            {
                converter.CallingFunctions.Add(f);
            }
            converter.CurrentFunction.AddToSource(f.Name + " ");
            bool first = true;
            foreach (IParameter p in f.Parameters)
            {
                if (!first) { converter.CurrentFunction.AddToSource(" "); } else { first = false; }
                converter.CurrentFunction.AddToSource("-" + p.FormalParameter + " ");
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedProc, p.EffectiveParameter, p.IsMutableParameter);
                if (p.IsMutableParameter)
                {
                    converter.CurrentFunction.AddToSource("([ref]" + "$" + p.EffectiveParameter + ")");
                }
                else { converter.CurrentFunction.AddToSource("$" + p.EffectiveParameter); }
            }
            converter.CurrentFunction.AddToSource(Environment.NewLine);
        }

        #endregion
    }
}
