using Converters;
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace o2Mate
{
    internal class Condition : Compiler
    {
        #region Private Fields
        private string expression;
        private string labelTrue;
        private string labelFalse;
        private int indent;
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Condition"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayCondition", new object[] { this.expression, this.labelTrue, this.labelFalse, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayCondition", new object[] { this.expression, this.labelTrue, this.labelFalse, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.expression = this.GetElementByName(elem, "expression").InnerText;
            this.labelTrue = this.GetElementByName(elem, "labelTrue").InnerText;
            this.labelFalse = this.GetElementByName(elem, "labelFalse").InnerText;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("condition");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("expression");
            writer.WriteString(this.expression);
            writer.WriteEndElement();
            writer.WriteStartElement("iftrue");
            writer.WriteString(this.labelTrue);
            writer.WriteEndElement();
            writer.WriteStartElement("iffalse");
            writer.WriteString(this.labelFalse);
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.expression = node.SelectSingleNode("expression").InnerText;
            this.labelTrue = node.SelectSingleNode("iftrue").InnerText;
            this.labelFalse = node.SelectSingleNode("iffalse").InnerText;
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.expression = node.SelectSingleNode("expression").InnerText;
            this.labelTrue = node.SelectSingleNode("iftrue").InnerText;
            this.labelFalse = node.SelectSingleNode("iffalse").InnerText;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            int nextPosition;
            o2Mate.Expression expr = new o2Mate.Expression();
            IData result = expr.Evaluate(this.expression, proc.CurrentScope);
            if (result != null)
            {
                if (result.ValueInt == 1)
                {
                    nextPosition = proc.GetPositionLabel(this.labelTrue);
                    proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "goto", this.labelTrue));
                }
                else
                {
                    nextPosition = proc.GetPositionLabel(this.labelFalse);
                    proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "goto", this.labelFalse));
                }
                proc.CurrentPositionExecution = nextPosition;
            }
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
            throw new Exception("Impossible de convertir une instruction de branchement. Utilisez plutt le modle '/CodeCommander/Condition'");
        }

        #endregion
    }
}
