using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace o2Mate
{
    public class CreateMOP : Compiler
    {
        #region Private Constants
        private const char ParamIdentifier = '$';
        #endregion

        #region Private Fields
        private string language;
        private string name;
        private List<string> refs;
        private string xmlCode;
        private int indent;
        private LegendeDict legendes;
        #endregion

        #region Default Constructor
        public CreateMOP()
        {
            this.language = "";
            this.name = "";
            this.legendes = new LegendeDict();
            this.xmlCode = "";
            this.refs = new List<string>();
        }
        #endregion

        #region Public Properties
        public string Language
        {
            get
            {
                return this.language;
            }
            set
            {
                this.language = value;
            }
        }

        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                this.name = value;
            }
        }

        public string XmlCode
        {
            get
            {
                return this.xmlCode;
            }
        }

        public LegendeDict Legendes
        {
            get
            {
                return this.legendes;
            }
        }

        public string ReferenceString
        {
            get
            {
                string output = "";
                bool first = true;
                foreach (string key in this.refs)
                {
                    if (!first) output += ", "; else first = false;
                    output += CreateMOP.ParamIdentifier + key;
                }
                return output;
            }
        }

        public List<string> References
        {
            get
            {
                return this.refs;
            }
        }
        #endregion

        #region Public Methods
        public string ReplaceParams(Dictionary<string, string> values)
        {
            string output = "";
            string xmlCode = this.XmlCode;
            for (int strIndex = 0; strIndex < xmlCode.Length; ++strIndex)
            {
                char c = this.XmlCode[strIndex];
                if (c == CreateMOP.ParamIdentifier && (strIndex + 1) < xmlCode.Length)
                {
                    bool found = false;
                    Dictionary<string, string> sortedDict = new Dictionary<string, string>();
                    int maxLength = 0;
                    // rechercher parmi les paramtres celui qui a le nom le plus long
                    foreach (string paramName in this.refs)
                    {
                        if ((strIndex + 1 + paramName.Length) < xmlCode.Length)
                        {
                            if (xmlCode.Substring(strIndex + 1, paramName.Length) == paramName)
                            {
                                if (values.ContainsKey(paramName))
                                {
                                    sortedDict.Add(paramName, values[paramName]);
                                    if (paramName.Length > maxLength)
                                    {
                                        maxLength = paramName.Length;
                                    }
                                }
                            }
                        }
                    }
                    // itrer tous les paramtres qui correspondent et prendre le plus long
                    foreach (string paramName in sortedDict.Keys)
                    {
                        if (paramName.Length == maxLength)
                        {
                            output += sortedDict[paramName];
                            found = true;
                            strIndex += paramName.Length;
                        }
                    }
                    if (!found)
                    {
                        output += CreateMOP.ParamIdentifier;
                    }
                }
                else
                {
                    output += c;
                }
            }
            return output;
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "CreateMOP"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.language = node.Attributes.GetNamedItem("language").Value;
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.legendes.Load(node.SelectSingleNode("legendes"));
            this.refs.Clear();
            foreach (XmlNode paramNode in node.SelectNodes("references/ref"))
            {
                if (!this.refs.Contains(paramNode.InnerText))
                    this.refs.Add(paramNode.InnerText);
            }
            this.xmlCode = node.SelectSingleNode("code").InnerXml;
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayCreateMOP", new object[] { this.language, this.name, this.ReferenceString, this.xmlCode, this.legendes, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayCreateMOP", new object[] { this.language, this.name, this.ReferenceString, this.xmlCode, this.legendes, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.language = this.GetElementByName(elem, "language").InnerText;
            this.name = this.GetElementByName(elem, "name").InnerText;
            string strParams = this.GetElementByName(elem, "params").InnerText;
            this.refs.Clear();
            if (!String.IsNullOrEmpty(strParams))
            {
                string[] tab = strParams.Split(',');
                foreach (string param in tab)
                {
                    string name = param.Trim();
                    if (name.StartsWith(CreateMOP.ParamIdentifier.ToString()))
                        name = name.Substring(1).Trim();
                    if (!String.IsNullOrEmpty(name))
                    {
                        if (!this.refs.Contains(name))
                            this.refs.Add(name);
                    }
                }
            }
            // mise  jour des legendes
            this.legendes.Update(this.GetElementByName(elem, "myLegendes"));
            this.xmlCode = this.GetElementByName(elem, "xml").InnerHtml;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("createmop");
            writer.WriteAttributeString("language", this.language);
            writer.WriteAttributeString("name", this.name);
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("legendes");
            this.legendes.Save(writer);
            writer.WriteEndElement();
            writer.WriteStartElement("references");
            foreach (string param in this.refs)
            {
                writer.WriteStartElement("ref");
                writer.WriteString(param);
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
            writer.WriteStartElement("code");
            writer.WriteRaw(this.xmlCode);
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.language = node.Attributes.GetNamedItem("language").Value;
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.legendes.Load(node.SelectSingleNode("legendes"));
            this.refs.Clear();
            foreach (XmlNode paramNode in node.SelectNodes("references/ref"))
            {
                if (!this.refs.Contains(paramNode.InnerText))
                    this.refs.Add(paramNode.InnerText);
            }
            this.xmlCode = node.SelectSingleNode("code").InnerXml;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
        }

        public void Convert(Converters.ICodeConverter converter, IProcess proc, FinalFile file)
        {
        }

        #endregion
    }
}
