using System;
using System.Collections.Generic;
using System.Text;

namespace o2Mate
{
    public class CreateProject : Compiler
    {
        #region Private Fields
        private string name;
        private int indent;
        #endregion

        #region Public Properties
        public string Name
        {
            get { return this.name; }
            set { this.name = value; }
        }
        #endregion

        #region Private Static Methods
        internal static bool AddIntoArray(o2Mate.Dictionnaire dict, string arrayName, Dictionary<string, string> values)
        {
            bool add = true;
            o2Mate.Array arr = null;
            if (dict.IsArray(arrayName))
            {
                arr = dict.GetArray(arrayName) as o2Mate.Array;
                for (int index = 1; index <= arr.Count; ++index)
                {
                    o2Mate.Fields f = arr.Item(index) as o2Mate.Fields;
                    string pKey = f.GetString("primaryKey");
                    if (pKey == values["primaryKey"])
                    {
                        add = false;
                    }
                }
            }
            else
            {
                arr = new o2Mate.Array();
                dict.AddArray(arrayName, arr);
            }
            if (add)
            {
                o2Mate.Fields f = new o2Mate.Fields();
                foreach (KeyValuePair<string, string> keyVal in values)
                {
                    f.AddString(keyVal.Key, keyVal.Value);
                }
                arr.Add(f);
            }
            // retourne vrai si l'objet a t ajout
            return add;
        }

        internal static void AddIntoProcess(o2Mate.Dictionnaire dict, string processName, int position, Dictionary<string, string> values)
        {
            int firstPos = 0;
            int currentPos = 1;
            o2Mate.Array statements = null;
            o2Mate.Fields process = null;
            if (dict.IsArray("statements"))
            {
                statements = dict.GetArray("statements") as o2Mate.Array;
                currentPos = statements.Count + 1;
            }
            else
            {
                statements = new o2Mate.Array();
                dict.AddArray("statements", statements);
            }
            o2Mate.Array processes = null;
            bool found = false;
            if (dict.IsArray("processes"))
            {
                // je recherche le processus qui a le mme nom
                processes = dict.GetArray("processes") as o2Mate.Array;
                for (int index = 1; index <= processes.Count; ++index)
                {
                    process = processes.Item(index) as o2Mate.Fields;
                    if (process.GetString("processName") == processName)
                    {
                        Int32.TryParse(process.GetString("current"), out currentPos);
                        Int32.TryParse(process.GetString("first"), out firstPos);
                        found = true;
                        break;
                    }
                }
            }
            else
            {
                processes = new o2Mate.Array();
                dict.AddArray("processes", processes);
            }
            if (!found)
            {
                process = new o2Mate.Fields();
                process.AddString("processName", processName);
                process.AddString("first", currentPos.ToString());
                process.AddString("current", currentPos.ToString());
                processes.Add(process);
            }
            else
            {
                found = false;
                for (int index = firstPos; index <= currentPos; ++index)
                {
                    o2Mate.Fields f = statements.Item(index) as o2Mate.Fields;
                    if (f.GetString("position") == position.ToString())
                    {
                        found = true;
                        break;
                    }
                }
                if (!found)
                {
                    o2Mate.Fields previous = statements.Item(currentPos) as o2Mate.Fields;
                    previous.AddString("next", (statements.Count + 1).ToString());
                }
            }
            if (!found)
            {
                o2Mate.Fields statmt = new o2Mate.Fields();
                statmt.AddString("processName", processName);
                statmt.AddString("position", position.ToString());
                statmt.AddString("next", "0");
                foreach (KeyValuePair<string, string> keyVal in values)
                {
                    statmt.AddString(keyVal.Key, keyVal.Value);
                }
                statements.Add(statmt);
                process.AddString("current", statements.Count.ToString());
            }
        }
        #endregion

        #region Public Static Methods
        public static o2Mate.Dictionnaire Compilation(Tree<ProjectItem> tree)
        {
            o2Mate.Dictionnaire dict = new Dictionnaire();
            foreach (Node<ProjectItem> node in tree)
            {
                node.Object.Print(dict, node);
                CreateProject.Compilation(dict, node);
            }
            return dict;
        }

        public static void Compilation(o2Mate.Dictionnaire dict, Node<ProjectItem> node)
        {
            foreach (Node<ProjectItem> child in node)
            {
                child.Object.Print(dict, child);
            }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "CreateProject"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.Attributes.GetNamedItem("name").Value;
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayCreateProject", new object[] { this.name, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayCreateProject", new object[] { this.name, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.name = this.GetElementByName(elem, "name").InnerText;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("createproject");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteAttributeString("name", this.name);
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.Attributes.GetNamedItem("name").Value;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "project", this.name));
        }

        public void Convert(Converters.ICodeConverter converter, IProcess proc, FinalFile file)
        {
        }

        #endregion
    }
}
