using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using Converters;

namespace o2Mate
{
    class CreateWriter : Compiler, ILanguageConverter
    {
        #region Private Fields
        private int indent;
        private string writerName;
        private List<string> filePath;
        private ICompilateur cachedComp;
        private List<string> directories, fileName;
        #endregion

        #region Public Properties
        public string WriterName
        {
            get { return this.writerName; }
        }

        public List<string> FilePath
        {
            get { return this.filePath; }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "CreateWriter"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.writerName = node.Attributes.GetNamedItem("name").Value;
            this.filePath = new List<string>();
            if (node.SelectNodes("file")[0].FirstChild != null)
            {
                foreach (XmlNode paramNode in node.SelectNodes("file/expression"))
                {
                    string paramValue = paramNode.InnerText;
                    this.filePath.Add(paramValue);
                }
            }
            // on en a besoin pour la conversion
            this.cachedComp = comp;
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            string strFile = String.Empty;
            bool first = true;
            foreach (string expression in this.filePath)
            {
                if (!first) { strFile += "/"; } else { first = false; }
                strFile += expression;
            }
            doc.InvokeScript("displayWriter", new object[] { this.writerName, strFile, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            string strFile = String.Empty;
            bool first = true;
            foreach (string expression in this.filePath)
            {
                if (!first) { strFile += "/"; } else { first = false; }
                strFile += expression;
            }
            doc.InvokeScript("displayWriter", new object[] { this.writerName, strFile, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.writerName = this.GetElementByName(elem, "writer").InnerText;
            if (String.IsNullOrEmpty(this.writerName)) this.writerName = "";
            string strFile = this.GetElementByName(elem, "file").InnerText;
            this.filePath = new List<string>();
            string[] splitPath = strFile.Split('/');
            foreach (string directory in splitPath)
            {
                if (!String.IsNullOrEmpty(directory))
                {
                    string value = directory;
                    value = value.Replace("&", "&amp;");
                    value = value.Replace("<", "&lt;");
                    value = value.Replace(">", "&gt;");
                    this.filePath.Add(value);
                }
            }
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("createwriter");
            writer.WriteAttributeString("name", this.writerName);
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("file");
            foreach (string expression in this.filePath)
            {
                writer.WriteStartElement("expression");
                writer.WriteString(expression);
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.writerName = node.Attributes.GetNamedItem("name").Value;
            this.filePath = new List<string>();
            if (node.SelectNodes("file")[0].FirstChild != null)
            {
                foreach (XmlNode paramNode in node.SelectNodes("file/expression"))
                {
                    string paramValue = paramNode.InnerText;
                    this.filePath.Add(paramValue);
                }
            }
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            string fileName = FinalFile.BuildDirectory;
            for(int index = 0; index < this.filePath.Count - 1; ++index)
            {
                string expression = this.filePath[index];
                o2Mate.Expression expr = new o2Mate.Expression();
                o2Mate.IData result = expr.Evaluate(expression, proc.CurrentScope);
                string value = result.ValueString;
                foreach (char c in Path.GetInvalidPathChars())
                {
                    int position = -1;
                    if ((position = value.IndexOf(c)) != -1) value = value.Substring(0, position) + value.Substring(position + 1);
                }
                fileName += value + Path.DirectorySeparatorChar;
                FinalFile.EnsureDirectoryCreated(fileName);
            }
            {
                string expression = this.filePath[this.filePath.Count - 1];
                o2Mate.Expression expr = new o2Mate.Expression();
                // le sparateur . permet d'utiliser des expressions
                string[] split = expression.Split('.');
                bool first = true;
                foreach (string s in split)
                {
                    o2Mate.IData result = expr.Evaluate(s, proc.CurrentScope);
                    string value = result.ValueString;
                    foreach (char c in Path.GetInvalidFileNameChars())
                    {
                        int position = -1;
                        if ((position = value.IndexOf(c)) != -1)
                            value = value.Substring(0, position) + value.Substring(position + 1);
                    }
                    if (!first) fileName += "."; else first = false;
                    fileName += value;
                }
            }
            FinalFile.EraseFile(fileName);
            if (proc.CurrentScope.Exists(this.writerName))
            {
                proc.CurrentScope.GetVariable(this.writerName).Value = "{\"" + fileName + "\",0,true}";
                proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "create writer", this.writerName, fileName));
            }
            else
            {
                proc.CurrentScope.Add(this.writerName, "{\"" + fileName + "\",0,true}", proc.Name);
                proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "create writer", this.writerName, fileName));
            }
        }

        public void Convert(Converters.ICodeConverter converter, IProcess proc, FinalFile file)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            this.directories = new List<string>();
            for (int index = 0; index < this.filePath.Count - 1; ++index)
            {
                string expression = this.filePath[index];
                expr.Convert(converter, expression, proc.CurrentScope);
                foreach (string used in converter.CurrentFunction.UsedVariables)
                {
                    this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, proc, used, false);
                }
                this.directories.Add(converter.CurrentFunction.CacheSource);
            }
            this.fileName = new List<string>();
            {
                string expression = this.filePath[this.filePath.Count - 1];
                string[] split = expression.Split('.');
                foreach (string s in split)
                {
                    expr.Convert(converter, s, proc.CurrentScope);
                    foreach (string used in converter.CurrentFunction.UsedVariables)
                    {
                        this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, proc, used, false);
                    }
                    this.fileName.Add(converter.CurrentFunction.CacheSource);
                }
            }
            converter.Convert(this);
            if (proc.CurrentScope.Exists(this.writerName))
            {
                proc.CurrentScope.GetVariable(this.writerName).Value = "";
            }
            else
            {
                proc.CurrentScope.Add(this.writerName, "", proc.Name);
            }
        }

        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
            string currentDir = "\"" + FinalFile.BuildDirectory + "\"";
            foreach (string dir in this.directories)
            {
                converter.CurrentFunction.AddToSource("CreateDirectoryIfNotExists " + currentDir + " & " + dir + Environment.NewLine);
                currentDir += " & " + dir + " & \"" + Path.DirectorySeparatorChar + "\"";
            }
            string completeFileName = "";
            bool first = true;
            this.fileName.ForEach(new Action<string>(delegate(string s)
            {
                if (!first) completeFileName += " & \".\" & "; else first = false;
                completeFileName += s;
            }));
            converter.CurrentFunction.AddToSource("EraseFile " + currentDir + " & " + completeFileName + Environment.NewLine);
            converter.CurrentFunction.AddToSource(this.writerName + " = Array(" + currentDir + " & " + completeFileName + ", 0, true)" + Environment.NewLine);
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            string currentDir = "\"" + FinalFile.BuildDirectory + "\"";
            foreach (string dir in this.directories)
            {
                converter.CurrentFunction.AddToSource("CreateDirectoryIfNotExists (" + currentDir + " + " + dir + ")" + Environment.NewLine);
                currentDir += " + " + dir + " + \"" + Path.DirectorySeparatorChar + "\"";
            }
            string completeFileName = "";
            bool first = true;
            this.fileName.ForEach(new Action<string>(delegate(string s)
            {
                if (!first) completeFileName += " + \".\" + "; else first = false;
                completeFileName += s;
            }));
            converter.CurrentFunction.AddToSource("EraseFile (" + currentDir + " + " + completeFileName + ")" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("$" + this.writerName + " = @((" + currentDir + " + " + completeFileName + "), 0, \"\", $true)" + Environment.NewLine);
        }

        #endregion
    }
}
