using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Xml;
using System.Collections;

namespace o2Mate
{
    [Guid("8B84DC3B-7229-4b38-B9ED-6F07E01D71D9")]
    [CoClass(typeof(IFields))]
    [InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
    public interface IFields
    {
        bool Exists(string name);
        void Load(XmlNode node);
        XmlNode Save(XmlDocument doc);
        void AddString(string name, string value);
        string GetString(string name);
        IEnumerable Keys { get; }
    }

    [ClassInterface(ClassInterfaceType.None)]
    public class Fields : IFields
    {
        #region Private Fields
        private Dictionary<string, string> table;
        #endregion

        #region Default Constructor
        public Fields()
        {
            this.table = new Dictionary<string, string>();
        }
        #endregion

        #region IFields Membres

        public bool Exists(string name)
        {
            return this.table.ContainsKey(name);
        }

        public void Load(XmlNode node)
        {
            this.table.Clear();
            XmlNodeList nodes = node.SelectNodes("field");
            if (nodes != null)
            {
                foreach (XmlNode field in nodes)
                {
                    string name = field.Attributes.GetNamedItem("name").Value;
                    string value = field.InnerText;
                    this.table.Add(name, value);
                }
            }
        }

        public XmlNode Save(XmlDocument doc)
        {
            XmlNode root = doc.CreateElement("item");
            foreach (string key in this.table.Keys)
            {
                string value = this.table[key];
                XmlNode field = doc.CreateElement("field");
                XmlAttribute fieldName = doc.CreateAttribute("name");
                fieldName.Value = key;
                field.Attributes.SetNamedItem(fieldName);
                field.InnerText = value;
                root.AppendChild(field);
            }
            return root;
        }

        public void AddString(string name, string value)
        {
            if (this.Exists(name))
            {
                this.table[name] = value;
            }
            else
            {
                this.table.Add(name, value);
            }
        }

        public string GetString(string name)
        {
            if (this.Exists(name))
            {
                return this.table[name];
            }
            else
            {
                return "";
            }
        }

        public IEnumerable Keys
        {
            get
            {
                string[] arr = new string[this.table.Count];
                this.table.Keys.CopyTo(arr, 0);
                return arr;
            }
        }

        #endregion
    }
}
