using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.Runtime.CompilerServices;
using Microsoft.VisualBasic.FileIO;

namespace o2Mate
{
    public class FinalFile
    {
        #region Private Constants
        private const string IndentString = "    ";
        #endregion

        #region Private Fields
        private string fileName;
        private List<int> lines;
        private bool precededWithSpaces;
        private bool mustIndent;
        #endregion

        #region Public Properties
        public string FileName
        {
            get { return this.fileName; }
            set { this.fileName = value; }
        }
        #endregion

        #region Public Static Properties
        public static string CodeCommanderDirectory
        {
            get { return SpecialDirectories.MyDocuments + "\\CodeCommander\\"; }
        }

        public static string BuildDirectory
        {
            get { return FinalFile.CodeCommanderDirectory + "files\\"; }
        }
        #endregion

        #region Private Methods
        private static bool ExtractInfos(string infos, out string fileName, out int indent, out bool startLine)
        {
            string data = infos.Substring(1, infos.Length - 2);
            string[] split = data.Split(',');
            fileName = split[0].Substring(1, split[0].Length - 2);
            indent = Int32.Parse(split[1]);
            startLine = Boolean.Parse(split[2]);
            return true;
        }

        private static string Indent(int indent)
        {
            string output = "";
            for(int index = 0; index < indent; ++index)
            {
                output += FinalFile.IndentString;
            }
            return output;
        }

        private int CountSpaces(string text)
        {
            int count = 0;
            foreach (char c in text)
            {
                if (c == ' ')
                {
                    ++count;
                }
                else if (c == '\t')
                {
                }
                else
                {
                    break;
                }
            }
            return count;
        }

        private List<int> ComputeTabAlignment()
        {
            List<int> aligns = new List<int>();
            aligns.Add(0);
            if (this.lines.Count > 0)
            {
                int lastSpaces = this.lines[this.lines.Count - 1];
                for (int index = 0; index < this.lines.Count; ++index)
                {
                    int currentSpaces = this.lines[index];
                    currentSpaces = (currentSpaces >> 2) * 4;
                    if (currentSpaces > 0)
                    {
                        if (currentSpaces > lastSpaces)
                        {
                            aligns.Add(currentSpaces);
                        }
                        else if (currentSpaces < lastSpaces)
                        {
                            if (aligns.Count > 1 && aligns[aligns.Count - 2] != currentSpaces)
                            {
                                // dans le cas o ce n'est pas normal
                                System.Diagnostics.Debugger.Break();
                            }
                            aligns.RemoveAt(aligns.Count - 1);
                        }
                    }
                    else
                    {
                        aligns.RemoveAll(new Predicate<int>(delegate(int i) { return i != 0; }));
                    }
                    lastSpaces = currentSpaces;
                }
            }
            return aligns;
        }

        private void CheckSpaces(int countSpaces, ref int newSpaces)
        {
            List<int> aligns = this.ComputeTabAlignment();
            for (int index = 0; index < aligns.Count; ++index)
            {
                if (aligns[index] == countSpaces)
                {
                    // si c'est le premier (qui vaut 0)
                    if (index == 0)
                    {
                        if (aligns.Count == 2)
                        {
                            // le nombre d'alignements vaut 2
                            newSpaces = aligns[1];
                        }
                        else if (aligns.Count > 2)
                        {
                            // le nombre d'alignements est suprieur  2
                            newSpaces = aligns[aligns.Count - 1];
                        }
                        else
                        {
                            newSpaces = 0;
                        }
                        return;
                    }
                    else if (index == (aligns.Count - 2))
                    {
                        // si c'est l'avant-dernier
                        newSpaces = aligns[aligns.Count - 2];
                        return;
                    }
                    else
                    {
                        newSpaces = aligns[index];
                        return;
                    }
                }
            }
            if (countSpaces > aligns[aligns.Count - 1])
            {
                newSpaces = countSpaces;
            }
            else
            {
                newSpaces = aligns[aligns.Count - 1];
            }
        }

        private static string IndentText(string text, int indent, ref bool startLine)
        {
            string output = "";
            Regex reg = new Regex("((.*" + Environment.NewLine + ")|.*$)");
            MatchCollection matches = reg.Matches(text);
            foreach (Match m in matches)
            {
                if (!String.IsNullOrEmpty(m.Groups[1].Value))
                {
                    if (!startLine)
                    {
                        if (m.Groups[1].Value.EndsWith(Environment.NewLine))
                        {
                            startLine = true;
                        }
                        output += m.Groups[1].Value;
                    }
                    else
                    {
                        if (!m.Groups[1].Value.EndsWith(Environment.NewLine))
                        {
                            startLine = false;
                        }
                        output += FinalFile.Indent(indent) + m.Groups[1].Value;
                    }
                }
            }
            return output;
        }
        #endregion

        #region Public Methods
        public void StartWithFile(string fileName)
        {
            this.FileName = fileName;
            this.Start();
        }

        public void Start()
        {
            this.lines = new List<int>();
            this.mustIndent = true;
            this.precededWithSpaces = false;
            FileStream fs = new FileStream(this.FileName, FileMode.Create);
            fs.Close();
        }

        [MethodImpl(MethodImplOptions.Synchronized)]
        public void WriteToFile(string text)
        {
            FileStream fs = new FileStream(this.FileName, FileMode.Append, FileAccess.Write, FileShare.Write);
            StreamWriter sw = new StreamWriter(fs, Encoding.Unicode);
            sw.Write(text);
            sw.Close();
            sw.Dispose();
            fs.Close();
            fs.Dispose();
        }

        public static void EnsureDirectoryCreated(string fileName)
        {
            try
            {
                DirectoryInfo di = new DirectoryInfo(fileName);
                di.Create();
            }
            catch { }
        }

        public static void EraseFile(string fileName)
        {
            try
            {
                FileInfo fi = new FileInfo(fileName);
                fi.Create().Close();
            }
            catch { }
        }

        public static void Indent(ref string destination)
        {
            try
            {
                string fileName;
                int indent;
                bool startLine;
                if (FinalFile.ExtractInfos(destination, out fileName, out indent, out startLine))
                {
                    destination = "{\"" + fileName + "\"," + (indent + 1).ToString() + "," + startLine.ToString() + "}";
                }
            }
            catch { }
        }

        public static void Unindent(ref string destination)
        {
            try
            {
                string fileName;
                int indent;
                bool startLine;
                if (FinalFile.ExtractInfos(destination, out fileName, out indent, out startLine))
                {
                    if (indent > 0)
                        destination = "{\"" + fileName + "\"," + (indent - 1).ToString() + "," + startLine.ToString() + "}";
                }
            }
            catch { }
        }

        public static void WriteToFile(ref string destination, string text)
        {
            try
            {
                string fileName;
                int indent;
                bool startLine;
                if (FinalFile.ExtractInfos(destination, out fileName, out indent, out startLine))
                {
                    FileStream fs = new FileStream(fileName, FileMode.Append, FileAccess.Write, FileShare.Write);
                    StreamWriter sw = new StreamWriter(fs, Encoding.Unicode);
                    sw.Write(FinalFile.IndentText(text, indent, ref startLine));
                    sw.Close();
                    sw.Dispose();
                    fs.Close();
                    fs.Dispose();
                    destination = "{\"" + fileName + "\"," + indent.ToString() + "," + startLine.ToString() + "}";
                }
            }
            catch { }
        }
        #endregion
    }
}
