using System;
using System.Collections.Generic;
using System.Text;

namespace o2Mate
{
    public class Injector : Compiler
    {
        #region Private Fields
        private string skeleton;
        private string name;
        private int indent;
        private int injector, before, after;
        IProcess injectedProcess;
        private Stack<Process> processStack;
        #endregion

        #region Default Constructor
        public Injector()
        {
            this.skeleton = "";
            this.name = "";
            this.indent = 0;
            this.injector = this.before = this.after = 0;
            this.injectedProcess = null;
            this.processStack = new Stack<Process>();
        }
        #endregion

        #region Public Properties
        public string Name
        {
            get { return this.name; }
            set { this.name = value; }
        }

        public string Skeleton
        {
            get { return this.skeleton; }
        }

        public string LongName
        {
            get { return this.skeleton + this.name; }
        }

        public IProcess InjectedProcess
        {
            get { return this.injectedProcess; }
            set { this.injectedProcess = value; }
        }

        internal Stack<Process> StackProcesses
        {
            get
            {
                return this.processStack;
            }
        }

        public int Before
        {
            get { return this.before; }
            set { this.before = value; }
        }

        public int After
        {
            get { return this.after; }
            set { this.after = value; }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Public Methods
        public void InjectObject(IProcess proc, Compiler obj, string modifier)
        {
            // injecter dans le processus initial mais pas dans les nouveaux processus
            if (this.StackProcesses.Count == 0)
            {
                if (modifier == "after" || modifier == "replace")
                {
                    if (after < proc.Objects.Count)
                    {
                        proc.InsertObject(after, obj);
                        ++after;
                    }
                    else
                    {
                        proc.AddObject(obj);
                    }
                }
                else if (modifier == "before")
                {
                    proc.InsertObject(before, obj);
                    ++before;
                }
            }
            else
            {
                // on n'est plus dans le processus inject mais dans une subroutine
                proc.AddObject(obj);
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Injector"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.injectedProcess = comp.CurrentProcess;
            this.injector = this.injectedProcess.Objects.IndexOf(this);
            this.before = this.after = this.injector;
            // si je suis dans un squelette
            if (this.injectedProcess.Name.StartsWith("Skeleton:"))
            {
                this.skeleton = this.injectedProcess.Name.Substring(9) + "/";
            }
            else
            {
                this.skeleton = "";
            }
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayInjector", new object[] { this.name, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayInjector", new object[] { this.name, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.name = this.GetElementByName(elem, "name").InnerText;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("injector");
            writer.WriteAttributeString("name", this.name);
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.Attributes.GetNamedItem("name").Value;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "injector", this.Name));
        }

        public void Convert(Converters.ICodeConverter converter, IProcess proc, FinalFile file)
        {
        }

        #endregion
    }
}
