using System;
using System.Collections.Generic;
using System.Text;
using Converters;

namespace o2Mate
{
    internal class Label : Compiler
    {
        #region Private Fields
        private string name;
        private int position;
        private int indent;
        #endregion

        #region Default Constructor
        public Label()
        {
            this.name = "";
        }
        #endregion

        #region Public Properties
        public string Name
        {
            get
            {
                return this.name;
            }
        }

        public int LabelPosition
        {
            get
            {
                return this.position;
            }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Label"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayLabel", new object[] { this.name, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayLabel", new object[] { this.name, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.name = this.GetElementByName(elem, "label").InnerText;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("label");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteString(this.name);
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.InnerText;
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.InnerText;
            this.position = comp.CurrentProcess.Count;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "label", this.Name));
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
        }

        #endregion
    }
}
