using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Converters;

namespace o2Mate
{
    public class Parallel : Compiler, ILanguageConverter
    {
        #region Private Fields
        private int jobCount;
        private XmlNode xmlCode;
        private int indent;
        private IProcess cachedProc;
        #endregion

        #region Public Properties
        public XmlNode XmlCode
        {
            get { return this.xmlCode; }
        }

        public int JobCount
        {
            get { return this.jobCount; }
            set { this.jobCount = value; }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Parallel"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.xmlCode = node.SelectSingleNode("code");
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayParallel", new object[] { this.indent.ToString(), false });
            int subIndent = this.indent + 1;
            foreach (Node<Compiler> child in node)
            {
                child.Object.Display(doc, child, true, ref subIndent);
            }
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayParallel", new object[] { this.indent.ToString(), true });
            int subIndent = this.indent + 1;
            foreach (Node<Compiler> child in node)
            {
                child.Object.DisplayReadOnly(doc, child, true, ref subIndent);
            }
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("parallel");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("code");
            // search next sibling child of code
            System.Windows.Forms.HtmlElement subElement = child.NextSibling;
            comp.Save(writer, ref subElement, this.indent + 1);
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = subElement;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            comp.Parse(comp, node.SelectSingleNode("code"));
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
        }

        public void Convert(Converters.ICodeConverter converter, IProcess proc, FinalFile file)
        {
            this.cachedProc = proc;
            converter.Convert(this);
        }

        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            for (int index = 0; index < this.JobCount; ++index)
            {
                converter.CurrentFunction.AddToSource("Invoke-Expression \" Start-Job {" + Environment.NewLine);
                converter.CurrentFunction.IndentSize += 1;
                converter.CurrentFunction.AddToSource(". '$functions'" + Environment.NewLine);
                IFunction f = converter.ImplementedFunctions.Find(new Predicate<IFunction>(delegate(IFunction func) { return func.Name == "job_" + this.cachedProc.Name + index.ToString(); }));
                if (!converter.CallingFunctions.Exists(new Predicate<IFunction>(delegate(IFunction func) { return func.Name == "job_" + this.cachedProc.Name + index.ToString(); })))
                {
                    converter.CallingFunctions.Add(f);
                }
                converter.CurrentFunction.AddToSource(f.Name + " ");
                bool first = true;
                foreach (IParameter p in f.Parameters)
                {
                    if (!first) { converter.CurrentFunction.AddToSource(" "); } else { first = false; }
                    converter.CurrentFunction.AddToSource("-" + p.FormalParameter + " ");
                    if (p.IsMutableParameter)
                    {
                        converter.CurrentFunction.AddToSource("([ref]" + "$" + p.EffectiveParameter + ")");
                    }
                    else { converter.CurrentFunction.AddToSource("$" + p.EffectiveParameter); }
                }
                converter.CurrentFunction.AddToSource(Environment.NewLine);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("} \"" + Environment.NewLine);
            }
        }

        #endregion
    }
}
