using System;
using System.Collections.Generic;
using System.Text;
using Converters;
using System.Threading;

namespace o2Mate
{
    public interface IProcess
    {
        int CurrentPositionExecution { get; set; }
        string Name { get; set; }
        o2Mate.Scope CurrentScope { get; set; }
        Dictionnaire CurrentDictionnaire { get; set; }
        Tree<ProjectItem> CurrentProject { get; set; }
        int Count { get; }
        List<Compiler> Objects { get; }
        void AddObject(Compiler obj);
        void InsertObject(int index, Compiler obj);
        void Remove(int from, int to);
        int GetPositionLabel(string name);
        void Convert(ICodeConverter converter, ICompilateur comp, FinalFile file);
        IProcess Clone();
        string DefaultWriter { get; set; }
    }

    internal class Process : IProcess, Compiler
    {
        #region Internal Class
        internal class ThreadObject
        {
            #region Private Fields
            private Process proc;
            private Compilateur comp;
            private Compiler tache;
            private FinalFile file;
            private Dictionnaire dict;
            #endregion

            #region Default Constructor
            public ThreadObject(Dictionnaire dict, Compilateur comp, Process proc, Compiler tache, FinalFile file)
            {
                this.comp = comp;
                this.proc = proc;
                this.tache = tache;
                this.file = file;
                this.dict = dict;
            }
            #endregion

            #region Public Methods
            public void Execute()
            {
                this.tache.WriteToFile(this.proc, this.file);
                if (this.tache.TypeName == "Call")
                {
                    Call appel = tache as Call;
                    this.proc.CurrentProject.Add(new ProjectItem(this.proc.Name, this.proc.CurrentPositionExecution, "call process", appel.ProcessName));
                    // on push le process
                    this.proc.CurrentProject.Push();
                    this.proc.CurrentScope.Push();
                    Process proc = this.comp.GetInternalProcess(appel.ProcessName);
                    this.proc.CurrentProject.Add(new ProjectItem(proc.Name, 0, "begin process", appel.ProcessName));
                    proc.DefaultWriter = this.proc.DefaultWriter;
                    this.comp.WriteToFile(proc, this.file);
                    this.proc.CurrentScope.Pop();
                }
                else if (tache.TypeName == "CallSkeleton")
                {
                    CallSkeleton appel = tache as CallSkeleton;
                    this.proc.CurrentProject.Add(new ProjectItem(this.proc.Name, this.proc.CurrentPositionExecution, "call skeleton", appel.Name));
                    // on push le process
                    this.proc.CurrentProject.Push();
                    this.proc.CurrentScope.Push();
                    Skeleton skel = this.comp.GetInternalSkeleton(appel.Name);
                    Process proc = skel.Process;
                    this.proc.CurrentProject.Add(new ProjectItem(proc.Name, 0, "begin skeleton", appel.Name));
                    proc.DefaultWriter = this.proc.DefaultWriter;
                    this.comp.WriteToFile(proc, this.file);
                    this.proc.CurrentScope.Pop();
                }
                else if (tache.TypeName == "Parallel")
                {
                    Parallel par = tache as Parallel;
                    Compilateur newComp = new Compilateur();
                    newComp.StartParallel(this.dict, this.comp.Processes.ConvertAll<IProcess>(new Converter<Process, IProcess>(delegate(Process p) { return p as IProcess; })), this.comp.Threads, this.proc.CurrentScope, par.XmlCode, this.file, this.proc);
                }
                ++this.proc.waitAll;
            }
            #endregion
        }
        #endregion

        #region Private Fields
        private string name;
        private List<Compiler> list;
        private int currentPosition;
        private o2Mate.Scope currentScope;
        private Dictionnaire currentDict;
        private Template currentTemplate;
        private Tree<ProjectItem> currentProject;
        private Stack<Template> templateStack;
        private string writer;
        internal int waitAll;
        private Timer tim;
        private ManualResetEvent wait;
        #endregion

        #region Default Constructor
        public Process()
        {
            this.name = "";
            this.list = new List<Compiler>();
            this.currentScope = null;
            this.currentDict  = null;
            this.currentProject = null;
            this.currentPosition = 0;
            this.templateStack = new Stack<Template>();
            this.currentTemplate = null;
        }
        #endregion

        #region Public Properties
        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                this.name = value;
            }
        }

        public Stack<Template> TemplateStack
        {
            get
            {
                return this.templateStack;
            }
        }

        public Template CurrentTemplate
        {
            get
            {
                return this.currentTemplate;
            }
        }

        public o2Mate.Scope CurrentScope
        {
            get
            {
                return this.currentScope;
            }
            set
            {
                this.currentScope = value;
            }
        }

        public Dictionnaire CurrentDictionnaire
        {
            get
            {
                return this.currentDict;
            }
            set
            {
                this.currentDict = value;
            }
        }

        public Tree<ProjectItem> CurrentProject
        {
            get
            {
                return this.currentProject;
            }
            set
            {
                this.currentProject = value;
            }
        }

        public int Count
        {
            get
            {
                return this.list.Count;
            }
        }

        public int CurrentPositionExecution
        {
            get
            {
                return this.currentPosition;
            }
            set
            {
                this.currentPosition = value;
            }
        }

        public string DefaultWriter
        {
            get { return this.writer; }
            set { this.writer = value; }
        }

        public List<Compiler> Objects
        {
            get
            {
                return this.list;
            }
        }
        #endregion

        #region Public Methods
        public IProcess Clone()
        {
            Process p = new Process();
            p.Name = this.Name;
            Compiler[] array = new Compiler[this.list.Count];
            this.list.CopyTo(array);
            foreach (Compiler c in array)
            {
                p.list.Add(c);
            }
            return p;
        }

        public Compiler Item(int index)
        {
            return this.list[index];
        }

        public void AddObject(Compiler obj)
        {
            this.list.Add(obj);
        }

        public void InsertObject(int index, Compiler obj)
        {
            if (this.CurrentPositionExecution > index)
                ++this.CurrentPositionExecution;
            this.list.Insert(index, obj);
        }

        public void Remove(int from, int to)
        {
            this.list.RemoveRange(from, to - from + 1);
        }

        public int GetPositionLabel(string labName)
        {
            foreach (Compiler tache in this.list)
            {
                if (tache.TypeName == "Label")
                {
                    Label lab = tache as Label;
                    if (lab.Name == labName)
                    {
                        return lab.LabelPosition;
                    }
                }
            }
            throw new Exception("Label '" + labName + "' non trouv");
        }

        public void StartTemplate(Template curTemp)
        {
            this.currentTemplate = curTemp;
            this.templateStack.Push(curTemp);
        }

        public void EndTemplate(Template curTemp)
        {
            if (this.templateStack.Count > 0)
            {
                Template lastTemp = this.templateStack.Peek();
                if (lastTemp.Name == curTemp.Name)
                {
                    this.templateStack.Pop();
                    if (this.templateStack.Count > 0)
                    {
                        this.currentTemplate = this.templateStack.Peek();
                    }
                    else
                    {
                        this.currentTemplate = null;
                    }
                }
                else
                    throw new Exception("Le template actif n'a pas le mme nom");
            }
            else
                throw new Exception("Il n'y a pas de template qui soit activ");
        }

        public void ExtractDictionary(Compilateur comp)
        {
            this.CurrentPositionExecution = 0;
            while (this.CurrentPositionExecution < this.list.Count)
            {
                Compiler tache = this.list[this.CurrentPositionExecution++];
                tache.ExtractDictionary(this);
                if (tache.TypeName == "Call")
                {
                    Call appel = tache as Call;
                    this.CurrentScope.Push();
                    Process proc = comp.GetInternalProcess(appel.ProcessName);
                    comp.ExtractDictionary(proc);
                    this.CurrentScope.Pop();
                }
                else if (tache.TypeName == "CallSkeleton")
                {
                    CallSkeleton appel = tache as CallSkeleton;
                    this.currentScope.Push();
                    Skeleton skel = comp.GetInternalSkeleton(appel.Name);
                    comp.InputLegendes(skel.Legendes);
                    comp.ExtractDictionary(skel.Process);
                    this.CurrentScope.Pop();
                }
            }
        }

        public void RunThread(object obj)
        {
            ThreadObject to = obj as ThreadObject;
            to.Execute();
        }

        public void WaitAll(object obj)
        {
            if (waitAll == this.Count)
            {
                this.tim.Dispose();
                this.wait.Set();
            }
        }

        public void ExecuteProcess(Compilateur comp, FinalFile file)
        {
            if (this.Name == "Parallel")
            {
                this.waitAll = 0;
                ThreadPool.SetMaxThreads(this.Count, this.Count);
            }
            this.CurrentPositionExecution = 0;
            while (this.CurrentPositionExecution < this.list.Count)
            {
                if (this.Name == "Parallel")
                {
                    ThreadObject to = new ThreadObject(this.currentDict, comp, this, this.list[this.CurrentPositionExecution++], file);
                    ThreadPool.QueueUserWorkItem(new WaitCallback(RunThread), to);
                }
                else
                {
                    Compiler tache = this.list[this.CurrentPositionExecution++];
                    tache.WriteToFile(this, file);
                    if (tache.TypeName == "Call")
                    {
                        Call appel = tache as Call;
                        this.CurrentProject.Add(new ProjectItem(this.Name, this.CurrentPositionExecution, "call process", appel.ProcessName));
                        // on push le process
                        this.CurrentProject.Push();
                        this.CurrentScope.Push();
                        Process proc = comp.GetInternalProcess(appel.ProcessName);
                        this.CurrentProject.Add(new ProjectItem(proc.Name, 0, "begin process", appel.ProcessName));
                        proc.DefaultWriter = this.DefaultWriter;
                        comp.WriteToFile(proc, file);
                        this.CurrentScope.Pop();
                    }
                    else if (tache.TypeName == "CallSkeleton")
                    {
                        CallSkeleton appel = tache as CallSkeleton;
                        this.CurrentProject.Add(new ProjectItem(this.Name, this.CurrentPositionExecution, "call skeleton", appel.Name));
                        // on push le process
                        this.CurrentProject.Push();
                        this.CurrentScope.Push();
                        Skeleton skel = comp.GetInternalSkeleton(appel.Name);
                        Process proc = skel.Process;
                        this.CurrentProject.Add(new ProjectItem(proc.Name, 0, "begin skeleton", appel.Name));
                        proc.DefaultWriter = this.DefaultWriter;
                        comp.WriteToFile(proc, file);
                        this.CurrentScope.Pop();
                    }
                    else if (tache.TypeName == "Parallel")
                    {
                        Parallel par = tache as Parallel;
                        Compilateur newComp = new Compilateur();
                        newComp.StartParallel(this.currentDict, comp.Processes.ConvertAll<IProcess>(new Converter<Process,IProcess>(delegate(Process p) { return p as IProcess; })), comp.Threads, this.currentScope, par.XmlCode, file, this);
                    }
                }
            }
            if (this.Name == "Parallel")
            {
                this.wait = new ManualResetEvent(false);
                this.tim = new Timer(new TimerCallback(WaitAll), null, 0, 1000);
                this.wait.WaitOne();
                this.wait.Close();
            }
        }

        public void Convert(ICodeConverter converter, ICompilateur comp, FinalFile file)
        {
            this.CurrentPositionExecution = 0;
            while (this.CurrentPositionExecution < this.list.Count)
            {
                if (this.Name.StartsWith("Parallel:"))
                {
                    Compiler tache = this.list[this.CurrentPositionExecution++];
                    if (tache.TypeName != "BeginProcess" && tache.TypeName != "EndProcess")
                    {
                        tache.Convert(converter, this, file);
                    }
                }
                else
                {
                    Compiler tache = this.list[this.CurrentPositionExecution++];
                    if (tache.TypeName == "Call")
                    {
                        Call c = tache as Call;
                        this.CurrentScope.Push();
                        comp.PushProcess(this);
                        IProcess proc = comp.GetProcess(c.ProcessName);
                        proc.DefaultWriter = this.DefaultWriter;
                        Function f = new Function();
                        f.Name = "func_" + c.ProcessName;
                        converter.ImplementedFunctions.Add(f);
                        converter.PushFunction(converter.CurrentFunction);
                        converter.CurrentFunction = f;
                        comp.Convert(converter, proc, file);
                        comp.PopProcess();
                        this.CurrentScope.Pop();
                        converter.CurrentFunction = converter.PopFunction();
                        c.Convert(converter, this, file);
                    }
                    else if (tache.TypeName == "CallSkeleton")
                    {
                        CallSkeleton c = tache as CallSkeleton;
                        this.CurrentScope.Push();
                        comp.PushProcess(this);
                        Skeleton skel = comp.GetSkeleton(c.Name);
                        IProcess proc = skel.Process;
                        proc.DefaultWriter = this.DefaultWriter;
                        Function f = new Function();
                        f.Name = "func_" + CallSkeleton.SpecialChars(skel.Path + "/" + skel.Name);
                        converter.ImplementedFunctions.Add(f);
                        converter.PushFunction(converter.CurrentFunction);
                        converter.CurrentFunction = f;
                        comp.Convert(converter, proc, file);
                        comp.PopProcess();
                        this.CurrentScope.Pop();
                        converter.CurrentFunction = converter.PopFunction();
                        c.Convert(converter, this, file);
                    }
                    else if (tache.TypeName == "BeginProcess")
                    {
                        BeginProcess bp = tache as BeginProcess;
                        bp.Convert(converter, this, file);
                    }
                    else if (tache.TypeName == "BeginSkeleton")
                    {
                        BeginSkeleton bs = tache as BeginSkeleton;
                        bs.Convert(converter, this, file);
                    }
                    else if (tache.TypeName == "EndProcess")
                    {
                        EndProcess ep = tache as EndProcess;
                        ep.Convert(converter, this, file);
                    }
                    else if (tache.TypeName == "EndSkeleton")
                    {
                        EndSkeleton es = tache as EndSkeleton;
                        es.Convert(converter, this, file);
                    }
                    else if (tache.TypeName == "Parallel")
                    {
                        this.CurrentScope.Push();
                        comp.PushProcess(this);
                        IProcess proc = comp.GetParallelProcess(this.Name);
                        Parallel par = tache as Parallel;
                        par.JobCount = proc.Count;
                        for (int index = this.Count - 1; index >= 0; --index)
                        {
                            BeginJob bj = new BeginJob();
                            bj.JobName = "job_" + this.Name + index.ToString();
                            proc.InsertObject(index, bj);
                            EndJob ej = new EndJob();
                            ej.JobName = "job_" + this.Name + index.ToString();
                            proc.InsertObject(index + 2, ej);
                        }
                        proc.DefaultWriter = this.DefaultWriter;
                        comp.Convert(converter, proc, file);
                        comp.PopProcess();
                        this.CurrentScope.Pop();
                        par.Convert(converter, this, file);
                    }
                    else
                    {
                        tache.Convert(converter, this, file);
                    }
                }
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Process"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
        }
        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
        }

        #endregion
    }
}
