using System;
using System.Collections.Generic;
using System.Text;

namespace o2Mate
{
    public class ProjectItem
    {
        #region Private Fields
        private string processName;
        private int position;
        private string type;
        private string[] values;
        #endregion

        #region Public Constructor
        public ProjectItem(string processName, int position, string type, params string[] values)
        {
            this.processName = processName;
            this.position = position;
            this.type = type;
            this.values = values;
        }
        #endregion

        #region Public Properties
        public string Type
        {
            get { return this.type; }
            set { this.type = value; }
        }

        public string ProcessName
        {
            get { return this.processName; }
            set { this.processName = value; }
        }

        public int Position
        {
            get { return this.position; }
            set { this.position = value; }
        }
        #endregion

        #region Public Methods
        public void Print(o2Mate.Dictionnaire dict, Node<ProjectItem> node)
        {
            Dictionary<string, string> tabVal = new Dictionary<string, string>();
            tabVal.Add("processName", this.processName);
            tabVal.Add("position", this.position.ToString());
            tabVal.Add("type", this.type);
            switch (this.type)
            {
                case "affectation":
                    tabVal.Add("primaryKey", this.values[0]);
                    tabVal.Add("varName", this.values[0]);
                    CreateProject.AddIntoArray(dict, "affectation", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "dictionary string":
                    tabVal.Add("primaryKey", this.values[0] + ":" + this.values[1]);
                    tabVal.Add("varName", this.values[0]);
                    tabVal.Add("stringName", this.values[1]);
                    CreateProject.AddIntoArray(dict, "dictionary string", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "dictionary field":
                    tabVal.Add("primaryKey", this.values[0] + ":" + this.values[1] + "." + this.values[2]);
                    tabVal.Add("varName", this.values[0]);
                    tabVal.Add("tabName", this.values[1]);
                    tabVal.Add("fieldName", this.values[2]);
                    CreateProject.AddIntoArray(dict, "dictionary field", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "begin process":
                    tabVal.Add("beginProcessName", this.values[0]);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "begin skeleton":
                    tabVal.Add("beginSkeletonName", this.values[0]);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "call process":
                    tabVal.Add("primaryKey", this.values[0]);
                    tabVal.Add("callProcessName", this.values[0]);
                    CreateProject.AddIntoArray(dict, "call process", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    CreateProject.Compilation(dict, node);
                    break;
                case "call skeleton":
                    tabVal.Add("primaryKey", this.values[0]);
                    tabVal.Add("callSkeletonName", this.values[0]);
                    CreateProject.AddIntoArray(dict, "call skeleton", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    CreateProject.Compilation(dict, node);
                    break;
                case "print field":
                    tabVal.Add("primaryKey", this.values[0] + "." + this.values[1]);
                    tabVal.Add("tabName", this.values[0]);
                    tabVal.Add("fieldName", this.values[1]);
                    CreateProject.AddIntoArray(dict, "print field", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "goto":
                    tabVal.Add("primaryKey", this.values[0]);
                    tabVal.Add("label", this.values[0]);
                    CreateProject.AddIntoArray(dict, "goto", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "create mop":
                    tabVal.Add("primaryKey", this.values[1] + "(" + this.values[0] + ")");
                    tabVal.Add("languageName", this.values[0]);
                    tabVal.Add("createMopName", this.values[1]);
                    CreateProject.AddIntoArray(dict, "create mop", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "create writer":
                    tabVal.Add("primaryKey", this.values[1]);
                    tabVal.Add("writerName", this.values[0]);
                    tabVal.Add("fileName", this.values[1]);
                    CreateProject.AddIntoArray(dict, "create writer", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "default writer":
                    tabVal.Add("varName", this.values[0]);
                    CreateProject.AddIntoArray(dict, "default writer", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "end process":
                    tabVal.Add("endProcessName", this.values[0]);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "end skeleton":
                    tabVal.Add("endSkeletonPath", this.values[0]);
                    tabVal.Add("endSkeletonName", this.values[1]);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "injector":
                    tabVal.Add("primaryKey", this.values[0]);
                    tabVal.Add("injectorName", this.values[0]);
                    CreateProject.AddIntoArray(dict, "injector", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "label":
                    tabVal.Add("primaryKey", this.processName + "." + this.values[0]);
                    tabVal.Add("labelName", this.values[0]);
                    CreateProject.AddIntoArray(dict, "label", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "size":
                    tabVal.Add("primaryKey", this.values[0] + ":" + this.values[1]);
                    tabVal.Add("varName", this.values[0]);
                    tabVal.Add("tabName", this.values[1]);
                    CreateProject.AddIntoArray(dict, "size", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "declare template":
                    tabVal.Add("primaryKey", this.values[0] + "/" + this.values[1]);
                    tabVal.Add("templatePath", this.values[0]);
                    tabVal.Add("templateName", this.values[1]);
                    tabVal.Add("parameters", this.values[2]);
                    CreateProject.AddIntoArray(dict, "declare template", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "print":
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "use mop":
                    tabVal.Add("primaryKey", this.values[1] + "(" + this.values[0] + ")");
                    tabVal.Add("languageName", this.values[0]);
                    tabVal.Add("command", this.values[1]);
                    CreateProject.AddIntoArray(dict, "use mop", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "use template":
                    tabVal.Add("primaryKey", this.values[0]);
                    tabVal.Add("templateName", this.values[0]);
                    CreateProject.AddIntoArray(dict, "use template", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "print variable":
                    tabVal.Add("primaryKey", this.values[0]);
                    tabVal.Add("varName", this.values[0]);
                    CreateProject.AddIntoArray(dict, "print variable", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "print string":
                    tabVal.Add("primaryKey", this.values[0]);
                    tabVal.Add("stringName", this.values[0]);
                    CreateProject.AddIntoArray(dict, "print string", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
                case "project":
                    tabVal.Add("primaryKey", this.values[0]);
                    tabVal.Add("name", this.values[0]);
                    CreateProject.AddIntoArray(dict, "project", tabVal);
                    CreateProject.AddIntoProcess(dict, this.processName, this.position, tabVal);
                    break;
            }
        }
        #endregion
    }
}
