using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Converters;

namespace o2Mate
{
    public class Syntax : Compiler
    {
        #region Private Fields
        private string name;
        private string xmlCode;
        private int indent;
        private LegendeDict legendes;
        #endregion

        #region Default Constructor
        public Syntax()
        {
            this.name = "";
            this.legendes = new LegendeDict();
        }
        #endregion

        #region Public Properties
        public string Name
        {
            get { return this.name; }
            set { this.name = value; }
        }

        public string XmlCode
        {
            get { return this.xmlCode; }
        }

        public LegendeDict Legendes
        {
            get { return this.legendes; }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Syntax"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Inject(Compiler obj, ICompilateur comp, XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.legendes.Load(node.SelectSingleNode("legendes"));
            this.xmlCode = node.SelectSingleNode("code").InnerXml;
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displaySyntax", new object[] { this.name, this.xmlCode, this.legendes, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displaySyntax", new object[] { this.name, this.xmlCode, this.legendes, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            string name = this.GetElementByName(elem, "name").InnerText;
            if (String.IsNullOrEmpty(name)) name = "";
            this.name = name;
            // mise  jour des lgendes
            this.legendes.Update(this.GetElementByName(elem, "myLegendes"));
            this.xmlCode = this.GetElementByName(elem, "xml").InnerHtml;
        }

        public void Save(ICompilateur comp, XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("syntax");
            writer.WriteAttributeString("name", this.name);
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("legendes");
            this.legendes.Save(writer);
            writer.WriteEndElement();
            writer.WriteStartElement("code");
            writer.WriteRaw(this.xmlCode);
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.legendes.Load(node.SelectSingleNode("legendes"));
            this.xmlCode = node.SelectSingleNode("code").InnerXml;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
        }

        #endregion
    }
}
