using System;
using System.Collections.Generic;
using System.Text;
using Converters;

namespace o2Mate
{
    internal class Taille : Compiler, ILanguageConverter
    {
        #region Private Fields
        private string varName;
        private string tabName;
        private int indent;
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Taille"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displaySize", new object[] { this.varName, this.tabName, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displaySize", new object[] { this.varName, this.tabName, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.varName = this.GetElementByName(elem, "varName").InnerText;
            this.tabName = this.GetElementByName(elem, "size").InnerText;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("size");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("terme");
            writer.WriteString(this.varName);
            writer.WriteEndElement();
            writer.WriteStartElement("tableau");
            writer.WriteString(this.tabName);
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.varName = node.SelectSingleNode("terme").InnerText;
            this.tabName = node.SelectSingleNode("tableau").InnerText;
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.varName = node.SelectSingleNode("terme").InnerText;
            this.tabName = node.SelectSingleNode("tableau").InnerText;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            if (proc.CurrentDictionnaire.IsArray(this.tabName))
            {
                Object obj = proc.CurrentDictionnaire.GetArray(this.tabName);
                Array arr = obj as Array;
                int size = arr.Count;
                if (proc.CurrentScope.Exists(this.varName))
                {
                    proc.CurrentScope.GetVariable(this.varName).Value = size.ToString();
                    proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "size", this.varName, this.tabName));
                }
                else
                {
                    proc.CurrentScope.Add(this.varName, size.ToString(), proc.Name);
                    proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "size", this.varName, this.tabName));
                }
            }
            else
            {
                if (proc.CurrentScope.Exists(this.varName))
                {
                    proc.CurrentScope.GetVariable(this.varName).Value = "0";
                }
                else
                {
                    proc.CurrentScope.Add(this.varName, "0", proc.Name);
                }
            }
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
            if (proc.CurrentScope.Exists(this.varName))
            {
                if (proc.CurrentScope.GetVariable(this.varName).BelongsTo != proc.Name)
                {
                    IParameter p = converter.CurrentFunction.Parameters.Find(new Predicate<IParameter>(delegate(IParameter param) { return param.FormalParameter == this.varName; }));
                    if (p != null)
                    {
                        p.IsMutableParameter = true;
                    }
                    else
                    {
                        Parameter newParam = new Parameter();
                        newParam.EffectiveParameter = this.varName;
                        newParam.IsMutableParameter = true;
                        converter.CurrentFunction.Parameters.Add(newParam);
                    }
                }
                else
                {
                    if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == this.varName; })))
                    {
                        converter.CurrentFunction.LocalVariables.Add(this.varName);
                    }
                }
            }
            else
            {
                converter.CurrentFunction.LocalVariables.Add(this.varName);
            }
            // pas de test sur l'existence du tableau dans le dictionnaire
            converter.Convert(this);
            if (!proc.CurrentScope.Exists(this.varName))
            {
                proc.CurrentScope.Add(this.varName, "0", proc.Name);
            }
        }

        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource(this.varName + " = theDict.GetArray(\"" + this.tabName + "\").Count" + Environment.NewLine);
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource("$" + this.varName + " = $theDict.GetArray(\"" + this.tabName + "\").Count" + Environment.NewLine);
        }

        #endregion
    }
}
