using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Converters;

namespace o2Mate
{
    public class Template : Compiler
    {
        #region Private Constants
        private const char ParamIdentifier = '$';
        #endregion

        #region Private Fields
        private string path;
        private string name;
        private Dictionary<string, string> pars;
        private string xmlCode;
        private int indent;
        private LegendeDict legendes;
        #endregion

        #region Default Constructor
        public Template()
        {
            this.path = "";
            this.name = "";
            this.legendes = new LegendeDict();
        }
        #endregion

        #region Public Properties
        public string Path
        {
            get
            {
                return this.path;
            }
            set
            {
                this.path = value;
            }
        }

        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                this.name = value;
            }
        }

        public string XmlCode
        {
            get
            {
                return this.xmlCode;
            }
        }

        public LegendeDict Legendes
        {
            get { return this.legendes; }
        }

        public string Parameters
        {
            get
            {
                string output = "";
                if (this.pars != null)
                {
                    foreach (KeyValuePair<string, string> kv in this.pars)
                    {
                        output += kv.Key + " ";
                    }
                }
                return output;
            }
        }
        #endregion

        #region Public Methods
        public string ReplaceParams(Dictionary<string, string> values)
        {
            string output = "";
            string xmlCode = this.XmlCode;
            for (int strIndex = 0; strIndex < xmlCode.Length; ++strIndex)
            {
                char c = this.XmlCode[strIndex];
                if (c == Template.ParamIdentifier && (strIndex + 1) < xmlCode.Length)
                {
                    bool found = false;
                    Dictionary<string, string> sortedDict = new Dictionary<string, string>();
                    int maxLength = 0;
                    // rechercher parmi les paramtres celui qui a le nom le plus long
                    foreach (string paramName in this.pars.Keys)
                    {
                        if ((strIndex + 1 + paramName.Length) < xmlCode.Length)
                        {
                            if (xmlCode.Substring(strIndex + 1, paramName.Length) == paramName)
                            {
                                if (values.ContainsKey(paramName))
                                {
                                    sortedDict.Add(paramName, values[paramName]);
                                    if (paramName.Length > maxLength)
                                    {
                                        maxLength = paramName.Length;
                                    }
                                }
                            }
                        }
                    }
                    // itrer tous les paramtres qui correspondent et prendre le plus long
                    foreach (string paramName in sortedDict.Keys)
                    {
                        if (paramName.Length == maxLength)
                        {
                            output += sortedDict[paramName];
                            found = true;
                            strIndex += paramName.Length;
                        }
                    }
                    if (!found)
                    {
                        output += Template.ParamIdentifier;
                    }
                }
                else
                {
                    output += c;
                }
            }
            return output;
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Template"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            string strParams = String.Empty;
            bool first = true;
            foreach (string param in this.pars.Keys)
            {
                if (!first) strParams += ", "; else first = false; 
                strParams += param;
            }
            doc.InvokeScript("displayTemplate", new object[] { this.path + "/" + this.name, strParams, this.xmlCode, this.legendes, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            string strParams = String.Empty;
            bool first = true;
            foreach (string param in this.pars.Keys)
            {
                if (!first) strParams += ", "; else first = false;
                strParams += param;
            }
            doc.InvokeScript("displayTemplate", new object[] { this.path + "/" + this.name, strParams, this.xmlCode, this.legendes, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            string path = this.GetElementByName(elem, "path").InnerText;
            if (String.IsNullOrEmpty(path)) path = "";
            int pos = path.LastIndexOf("/");
            if (pos != -1)
            {
                this.path = path.Substring(0, pos);
                this.name = path.Substring(pos + 1);
            }
            else
            {
                this.name = path;
                this.path = "";
            }
            string strParams = this.GetElementByName(elem, "params").InnerText;
            this.pars = new Dictionary<string, string>();
            if (!String.IsNullOrEmpty(strParams))
            {
                string[] tab = strParams.Split(',');
                foreach (string param in tab)
                {
                    string name = String.Empty;
                    if (param.StartsWith(Template.ParamIdentifier.ToString()))
                    {
                        name = param.Substring(1).Trim();
                    }
                    else
                    {
                        name = param.Trim();
                    }
                    if (!this.pars.ContainsKey(name))
                        this.pars.Add(name, "");
                }
            }
            // mise  jour des legendes
            this.legendes.Update(this.GetElementByName(elem, "myLegendes"));
            this.xmlCode = this.GetElementByName(elem, "xml").InnerHtml;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("template");
            writer.WriteAttributeString("path", this.path);
            writer.WriteAttributeString("name", this.name);
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("legendes");
            this.legendes.Save(writer);
            writer.WriteEndElement();
            writer.WriteStartElement("params");
            foreach (string param in this.pars.Keys)
            {
                writer.WriteStartElement("param");
                writer.WriteString(param);
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
            writer.WriteStartElement("code");
            writer.WriteRaw(this.xmlCode);
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.path = node.Attributes.GetNamedItem("path").Value;
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.legendes.Load(node.SelectSingleNode("legendes"));
            this.pars = new Dictionary<string, string>();
            foreach (XmlNode paramNode in node.SelectNodes("params/param"))
            {
                if (!this.pars.ContainsKey(paramNode.InnerText))
                    this.pars.Add(paramNode.InnerText, "");
            }
            this.xmlCode = node.SelectSingleNode("code").InnerXml;
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.path = node.Attributes.GetNamedItem("path").Value;
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.legendes.Load(node.SelectSingleNode("legendes"));
            this.pars = new Dictionary<string, string>();
            foreach (XmlNode paramNode in node.SelectNodes("params/param"))
            {
                if (!this.pars.ContainsKey(paramNode.InnerText))
                    this.pars.Add(paramNode.InnerText, "");
            }
            this.xmlCode = node.SelectSingleNode("code").InnerXml;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "declare template", this.Path, this.Name, this.Parameters));
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
        }

        #endregion
    }
}
