using System;
using System.Collections.Generic;
using System.Text;
using Converters;

namespace o2Mate
{
    internal class Texte : Compiler, ILanguageConverter
    {
        #region Private Fields
        private string text;
        private int indent;
        private string defaultWriter;
        #endregion

        #region Public Properties
        public string Text
        {
            get
            {
                return this.text;
            }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Texte"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayText", new object[] { this.text, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayText", new object[] { this.text, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.text = this.GetElementByName(elem, "text").InnerText;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("texte");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteString(this.text);
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.text = node.InnerText;
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.text = node.InnerText;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            if (proc.CurrentScope.Exists(proc.DefaultWriter))
            {
                string writer = proc.CurrentScope.GetVariable(proc.DefaultWriter).ValueString;
                FinalFile.WriteToFile(ref writer, this.text.Replace(Environment.NewLine, "").Replace('', ' ').Replace('', '\t').Replace("", Environment.NewLine));
                proc.CurrentScope.GetVariable(proc.DefaultWriter).Value = writer;
                proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "print"));
            }
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
            if (proc.CurrentScope.Exists(proc.DefaultWriter))
            {
                this.defaultWriter = proc.DefaultWriter;
                converter.Convert(this);
            }
        }

        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource("WriteToFile " + this.defaultWriter + ", \"" + this.text.Replace("\"", "\"\"").Replace("\r\n", "").Replace('', ' ').Replace('', '\t').Replace("", "\" & vbCrLf & \"") + "\"" + Environment.NewLine); 
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource("WriteToFile $" + this.defaultWriter + " (\"" + this.text.Replace("\"", "\"\"").Replace("\r\n", "").Replace('', ' ').Replace('', '\t').Replace("", "\" + $crlf + \"") + "\")" + Environment.NewLine);
        }

        #endregion
    }
}
