using System;
using System.Collections.Generic;
using System.Text;
using Converters;

namespace o2Mate
{
    public class Unindent : Compiler, ILanguageConverter
    {
        #region Private Fields
        private int indent;
        private string defaultWriter;
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Unindent"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayUnindent", new object[] { this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayUnindent", new object[] { this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("unindent");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            if (proc.CurrentScope.Exists(proc.DefaultWriter))
            {
                string writer = proc.CurrentScope.GetVariable(proc.DefaultWriter).ValueString;
                FinalFile.Unindent(ref writer);
                proc.CurrentScope.GetVariable(proc.DefaultWriter).Value = writer;
            }
        }

        public void Convert(Converters.ICodeConverter converter, IProcess proc, FinalFile file)
        {
            if (proc.CurrentScope.Exists(proc.DefaultWriter))
            {
                this.defaultWriter = proc.DefaultWriter;
                converter.Convert(this);
            }
        }

        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource("Unindent " + this.defaultWriter + Environment.NewLine);
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            converter.CurrentFunction.AddToSource("Unindent $" + this.defaultWriter + Environment.NewLine);
        }

        #endregion
    }
}
