using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;

namespace o2Mate
{
    public class UseMOP : Compiler
    {
        #region Private Fields
        private string language;
        private string name;
        private List<string> skeletonPath;
        private string modifier;
        private string expression;
        private int indent;
        private ICompilateur cachedComp;
        #endregion

        #region Default Constructor
        public UseMOP()
        {
            this.language = "";
            this.name = "";
            this.skeletonPath = new List<string>();
            this.modifier = "after";
            this.expression = "";
        }
        #endregion

        #region Public Properties
        public string Language
        {
            get
            {
                return this.language;
            }
            set
            {
                this.language = value;
            }
        }

        public string Name
        {
            get { return this.name; }
            set { this.name = value; }
        }

        public List<string> Path
        {
            get { return this.skeletonPath; }
            set { this.skeletonPath = value; }
        }

        public string Injector
        {
            get
            {
                string insert = String.Empty;
                bool first = true;
                foreach (string path in this.skeletonPath)
                {
                    if (!first) insert += "/"; else first = false;
                    insert += path;
                }
                return insert;
            }
        }

        public string Modifier
        {
            get { return this.modifier; }
            set { this.modifier = value; }
        }

        public string Command
        {
            get
            {
                if (!this.HasInsertor)
                    return this.name;
                else
                {
                    string command = this.name + "@";
                    bool first = true;
                    foreach (string path in this.skeletonPath)
                    {
                        if (!first) command += "/"; else first = false;
                        command += path;
                    }
                    command += "{" + this.modifier + "}";
                    return command;
                }
            }
            set
            {
                Regex reg = new Regex(@"^([^@]+)([@](/?(([^/]+)/)*([^/]+))\{(after|before|replace)\})?$", RegexOptions.IgnoreCase);
                Match m = reg.Match(value);
                if (m.Success)
                {
                    this.name = m.Groups[1].Value;
                    string file = m.Groups[3].Value;
                    this.skeletonPath.Clear();
                    string[] tab = file.Split('/');
                    foreach (string path in tab)
                    {
                        if (!String.IsNullOrEmpty(path))
                            this.skeletonPath.Add(path);
                    }
                    this.modifier = m.Groups[7].Value.ToLower();
                }
                else
                {
                    throw new FormatException("Format de la commande '" + value + "' inattendue");
                }
            }
        }

        public string Expression
        {
            get { return this.expression; }
        }


        public bool HasInsertor
        {
            get
            {
                return !(this.skeletonPath.Count == 0 || String.IsNullOrEmpty(this.modifier));
            }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "UseMOP"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
            // pas de recherche de mop, car c'est dynamique (en fonction de la valeur des paramtres)
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.language = node.Attributes.GetNamedItem("language").Value;
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.modifier = node.Attributes.GetNamedItem("modifier").Value;
            this.expression = node.Attributes.GetNamedItem("expression").Value;
            this.skeletonPath.Clear();
            foreach (XmlNode fileNode in node.SelectNodes("file/expression"))
            {
                this.skeletonPath.Add(fileNode.InnerText);
            }
            this.cachedComp = comp;
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayUseMOP", new object[] { this.language, this.Command, this.expression, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayUseMOP", new object[] { this.language, this.Command, this.expression, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.language = this.GetElementByName(elem, "language").InnerText;
            if (String.IsNullOrEmpty(this.language)) this.language = "";
            try
            {
                this.Command = this.GetElementByName(elem, "command").InnerText;
            }
            catch { }
            this.expression = this.GetElementByName(elem, "expression").InnerText;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("usemop");
            writer.WriteAttributeString("language", this.language);
            writer.WriteAttributeString("name", this.name);
            writer.WriteAttributeString("modifier", this.modifier);
            writer.WriteAttributeString("expression", this.expression);
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("file");
            foreach (string expression in this.skeletonPath)
            {
                writer.WriteStartElement("expression");
                writer.WriteString(expression);
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.language = node.Attributes.GetNamedItem("language").Value;
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.modifier = node.Attributes.GetNamedItem("modifier").Value;
            this.expression = node.Attributes.GetNamedItem("expression").Value;
            this.skeletonPath.Clear();
            foreach (XmlNode fileNode in node.SelectNodes("file/expression"))
            {
                this.skeletonPath.Add(fileNode.InnerText);
            }
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "use mop", this.Language, this.Command));
            // rechercher le MOP source
            CreateMOP mopSrc = this.cachedComp.GetMOP(this.language, this.name);
            // valuation de l'expression (liste)
            string[] tab;
            if (!String.IsNullOrEmpty(this.expression))
            {
                o2Mate.Expression expr = new Expression();
                IData result = expr.Evaluate("array(" + this.expression + ")", proc.CurrentScope);
                tab = result.ValueString.Split(',');
            }
            else
            {
                tab = new string[0];
            }
            // remplacement des valeurs
            int index = 0;
            Dictionary<string, string> values = new Dictionary<string, string>();
            while (index < mopSrc.References.Count)
            {
                if (index < tab.Length)
                {
                    values.Add(mopSrc.References[index], tab[index]);
                }
                ++index;
            }
            if (index < tab.Length && mopSrc.References.Count > 0)
            {
                values[mopSrc.References[mopSrc.References.Count - 1]] += String.Join(",", tab, index, tab.Length - index + 1);
            }
            string code = mopSrc.ReplaceParams(values);
            // charge dans le compilateur l'extrait de code du mop
            XmlDocument doc = new XmlDocument();
            doc.LoadXml("<code>" + code + "</code>");
            if (!this.HasInsertor)
            {
                Injector j = new Injector();
                j.Before = j.After = proc.Objects.IndexOf(this) + 1;
                j.InjectedProcess = proc;
                this.cachedComp.Injection(j, doc.DocumentElement, "after");
            }
            else
            {
                Injector j = this.cachedComp.GetInjector(this.Injector);
                this.cachedComp.Injection(j, doc.DocumentElement, this.modifier);
            }
        }

        public void Convert(Converters.ICodeConverter converter, IProcess proc, FinalFile file)
        {
        }

        #endregion
    }
}
