using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Text.RegularExpressions;
using Converters;

namespace o2Mate
{
    public class UseTemplate : Compiler, ILanguageConverter
    {
        private delegate void Conversion(ICodeConverter converter);
        struct NameConverter
        {
            public string name;
            public Conversion del;

            public NameConverter(string name, Conversion conv)
            {
                this.name = name;
                this.del = conv;
            }
        }

        #region Private Fields
        private string name;
        private Dictionary<string, string> defParams;
        private List<Coding> codings;
        private Template template;
        private int indent;
        private ICompilateur cachedComp;
        private FinalFile cachedFile;
        private NameConverter[] namePowerShellConverters, nameVBScriptConverters;
        #endregion

        #region Default Constructor
        public UseTemplate()
        {
            this.name = "";
            this.codings = new List<Coding>();
            this.namePowerShellConverters = new NameConverter[] {
                new NameConverter("/CodeCommander/Condition", new Conversion(this.ConditionToPowerShell)),
                new NameConverter("/CodeCommander/WithCondition", new Conversion(this.WithConditionToPowerShell)),
                new NameConverter("/CodeCommander/LoopWithWhile", new Conversion(this.LoopWithWhileToPowerShell)),
                new NameConverter("/CodeCommander/LoopWithConditionalWhile", new Conversion(this.LoopWithConditionalWhileToPowerShell)),
                new NameConverter("/CodeCommander/LoopWhile", new Conversion(this.LoopWhileToPowerShell)),
                new NameConverter("/CodeCommander/LoopConditionalWhile", new Conversion(this.LoopConditionalWhileToPowerShell)),
                new NameConverter("/CodeCommander/LoopWithUntil", new Conversion(this.LoopWithUntilToPowerShell)),
                new NameConverter("/CodeCommander/LoopWithConditionalUntil", new Conversion(this.LoopWithConditionalUntilToPowerShell)),
                new NameConverter("/CodeCommander/LoopUntil", new Conversion(this.LoopUntilToPowerShell)),
                new NameConverter("/CodeCommander/LoopConditionalUntil", new Conversion(this.LoopConditionalUntilToPowerShell)),
                new NameConverter("/CodeCommander/ForEach", new Conversion(this.LoopForEachToPowerShell)),
                new NameConverter("/CodeCommander/ForEachConditional", new Conversion(this.LoopCondtionalForEachToPowerShell)),
                new NameConverter("/CodeCommander/Loop", new Conversion(this.LoopToPowerShell)),
                new NameConverter("/CodeCommander/LoopInverse", new Conversion(this.LoopInverseToPowerShell)),
                new NameConverter("/String/length", new Conversion(this.StringLengthToPowerShell)),
                new NameConverter("/String/substring", new Conversion(this.StringSubstringToPowerShell)),
                new NameConverter("/String/replace", new Conversion(this.StringReplaceToPowerShell))
            };

            this.nameVBScriptConverters = new NameConverter[] {
                new NameConverter("/CodeCommander/Condition", new Conversion(this.ConditionToVBScript)),
                new NameConverter("/CodeCommander/WithCondition", new Conversion(this.WithConditionToVBScript)),
                new NameConverter("/CodeCommander/LoopWithWhile", new Conversion(this.LoopWithWhileToVBScript)),
                new NameConverter("/CodeCommander/LoopWithConditionalWhile", new Conversion(this.LoopWithConditionalWhileToVBScript)),
                new NameConverter("/CodeCommander/LoopWhile", new Conversion(this.LoopWhileToVBScript)),
                new NameConverter("/CodeCommander/LoopConditionalWhile", new Conversion(this.LoopConditionalWhileToVBScript)),
                new NameConverter("/CodeCommander/LoopWithUntil", new Conversion(this.LoopWithUntilToVBScript)),
                new NameConverter("/CodeCommander/LoopWithConditionalUntil", new Conversion(this.LoopWithConditionalUntilToVBScript)),
                new NameConverter("/CodeCommander/LoopUntil", new Conversion(this.LoopUntilToVBScript)),
                new NameConverter("/CodeCommander/LoopConditionalUntil", new Conversion(this.LoopConditionalUntilToVBScript)),
                new NameConverter("/CodeCommander/ForEach", new Conversion(this.LoopForEachToVBScript)),
                new NameConverter("/CodeCommander/ForEachConditional", new Conversion(this.LoopConditionalWhileToVBScript)),
                new NameConverter("/CodeCommander/Loop", new Conversion(this.LoopToVBScript)),
                new NameConverter("/CodeCommander/LoopInverse", new Conversion(this.LoopInverseToVBScript)),
                new NameConverter("/String/length", new Conversion(this.StringLengthToVBScript)),
                new NameConverter("/String/substring", new Conversion(this.StringSubstringToVBScript)),
                new NameConverter("/String/replace", new Conversion(this.StringReplaceToVBScript))
            };
        }
        #endregion

        #region Public Properties
        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                this.name = value;
            }
        }

        public Dictionary<string, string> Parameters
        {
            get { return this.defParams; }
        }
        #endregion

        #region Public Methods
        public Coding GetCoding(string name)
        {
            foreach (Coding c in this.codings)
            {
                if (c.CodingName == name)
                {
                    return c;
                }
            }
            return null;
        }

        public void AddCoding(Coding code)
        {
            this.codings.Add(code);
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }

        private bool IsConvertible(ICodeConverter converter, string name)
        {
            bool isConvertible = true;
            switch (converter.LanguageName)
            {
                case "PowerShell":
                    foreach (NameConverter conv in this.namePowerShellConverters)
                    {
                        if (conv.name == name)
                        {
                            isConvertible = false;
                        }
                    }
                    break;
                case "VBScript":
                    foreach (NameConverter conv in this.nameVBScriptConverters)
                    {
                        if (conv.name == name)
                        {
                            isConvertible = false;
                        }
                    }
                    break;
            }
            return isConvertible;
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "UseTemplate"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
            proc.CurrentDictionnaire.Legendes.CopyFrom(this.template.Legendes);
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            string strParams = String.Empty;
            bool first = true;
            foreach (KeyValuePair<string,string> kv in this.defParams)
            {
                if (!first) { strParams += " "; } else { first = false; }
                strParams += "@" + kv.Key + "=" + kv.Value;
            }
            doc.InvokeScript("displayUseTemplate", new object[] { this.name, strParams, this.indent.ToString(), false });
            int subIndent = this.indent + 1;
            foreach (Node<Compiler> child in node)
            {
                child.Object.Display(doc, child, true, ref subIndent);
            }
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            string strParams = String.Empty;
            bool first = true;
            foreach (KeyValuePair<string, string> kv in this.defParams)
            {
                if (!first) { strParams += " "; } else { first = false; }
                strParams += "@" + kv.Key + "=" + kv.Value;
            }
            doc.InvokeScript("displayUseTemplate", new object[] { this.name, strParams, this.indent.ToString(), true });
            int subIndent = this.indent + 1;
            foreach (Node<Compiler> child in node)
            {
                child.Object.DisplayReadOnly(doc, child, true, ref subIndent);
            }
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.name = this.GetElementByName(elem, "path").InnerText;
            if (String.IsNullOrEmpty(this.name)) this.name = "";
            string strParams = this.GetElementByName(elem, "params").InnerText + " ";
            this.defParams = new Dictionary<string, string>();
            // on peut ajouter des parentheses pour utiliser le caractre =
            Regex reg = new Regex(@"@([a-zA-Z_0-9]+)=(?<sentence>(?<word>\w|[^@]|\s)+)");
            if (!String.IsNullOrEmpty(strParams))
            {
                MatchCollection mColl = reg.Matches(strParams);
                foreach (Match m in mColl)
                {
                    if (m.Success)
                    {
                        string value = m.Groups[2].Value.Trim();
                        value = value.Replace("&", "&amp;");
                        value = value.Replace("<", "&lt;");
                        value = value.Replace(">", "&gt;");
                        if (!this.defParams.ContainsKey(m.Groups[1].Value))
                            this.defParams.Add(m.Groups[1].Value, value);
                    }
                }
            }
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("usetemplate");
            writer.WriteAttributeString("name", this.name);
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteStartElement("params");
            foreach (KeyValuePair<string, string> keyValue in this.defParams)
            {
                writer.WriteStartElement("param");
                writer.WriteAttributeString("name", keyValue.Key);
                writer.WriteString(keyValue.Value);
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
            writer.WriteStartElement("codings");
            System.Windows.Forms.HtmlElement subElement = child.NextSibling;
            comp.Save(writer, ref subElement, this.indent + 1);
            writer.WriteEndElement();
            writer.WriteEndElement();
            child = subElement;
        }

        public void Parse(ICompilateur comp, XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.defParams = new Dictionary<string, string>();
            foreach (XmlNode paramNode in node.SelectNodes("params/param"))
            {
                string paramName = paramNode.Attributes.GetNamedItem("name").Value;
                string paramValue = paramNode.InnerText;
                if (!this.defParams.ContainsKey(paramName))
                    this.defParams.Add(paramName, paramValue);
            }
            XmlNode nodeCodings = node.SelectSingleNode("codings");
            comp.Parse(comp, nodeCodings);
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            Injector injector = obj as Injector;
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.defParams = new Dictionary<string, string>();
            if (node.SelectNodes("params")[0].FirstChild != null)
            {
                foreach (XmlNode paramNode in node.SelectNodes("params/param"))
                {
                    string paramName = paramNode.Attributes.GetNamedItem("name").Value;
                    string paramValue = paramNode.InnerText;
                    if (!this.defParams.ContainsKey(paramName))
                        this.defParams.Add(paramName, paramValue);
                }
            }
            // dmarre un using template
            comp.StartUsingTemplate(this);
            // itre les extraits de code
            XmlNode nodeCodings = node.SelectSingleNode("codings");
            comp.Injection(injector, nodeCodings, modifier);
            // on ne charge pas les donnes si le template est dj dvelopp
            if (!comp.UnderConversion || this.IsConvertible(comp.ConvertedLanguage, this.Name))
            {
                this.template = comp.GetTemplate(this.name);
                string code = this.template.ReplaceParams(this.defParams);
                XmlDocument doc = new XmlDocument();
                // charge dans le compilateur l'extrait de code du template
                doc.LoadXml("<code>" + code + "</code>");
                comp.Injection(injector, doc.DocumentElement, modifier);
            }
            else
            {
                // on a besoin de rcuprer le compilateur pour plus tard
                this.cachedComp = comp;
            }
            // termine un using template
            comp.EndUsingTemplate(this);
        }

        public void Load(ICompilateur comp, XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.defParams = new Dictionary<string,string>();
            if (node.SelectNodes("params")[0].FirstChild != null)
            {
                foreach (XmlNode paramNode in node.SelectNodes("params/param"))
                {
                    string paramName = paramNode.Attributes.GetNamedItem("name").Value;
                    string paramValue = paramNode.InnerText;
                    if (!this.defParams.ContainsKey(paramName))
                        this.defParams.Add(paramName, paramValue);
                }
            }
            // dmarre un using template
            comp.StartUsingTemplate(this);
            // itre les extraits de code
            XmlNode nodeCodings = node.SelectSingleNode("codings");
            comp.Load(comp, nodeCodings);
            // on ne charge pas les donnes si le template est dj dvelopp
            if (!comp.UnderConversion || this.IsConvertible(comp.ConvertedLanguage, this.Name))
            {
                this.template = comp.GetTemplate(this.name);
                string code = this.template.ReplaceParams(this.defParams);
                XmlDocument doc = new XmlDocument();
                // charge dans le compilateur l'extrait de code du template
                doc.LoadXml("<code>" + code + "</code>");
                comp.Load(comp, doc.DocumentElement);
            }
            else
            {
                // on a besoin de rcuprer le compilateur pour plus tard
                this.cachedComp = comp;
            }
            // termine un using template
            comp.EndUsingTemplate(this);
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "use template", this.Name));
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
            // nous avons besoin de stocker ces donnes pour les rutiliser
            // en dehors de la fonction (notamment quand on effectue la conversion)
            this.cachedFile = file;
            converter.Convert(this);
        }

        #endregion

        #region ILanguageConverter Membres

        private string ReplaceStringParamater(string parameterName)
        {
            string value = this.Parameters[parameterName];
            value = value.Replace("&lt;", "<");
            value = value.Replace("&gt;", ">");
            value = value.Replace("&amp;", "&");
            return value;
        }

        private void LoopToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string value = "0";
            if (this.Parameters.ContainsKey("value")) value = this.ReplaceStringParamater("value");
            expr.Convert(converter, value, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource("For " + counter + " = 1 To CLng(" + converter.CurrentFunction.CacheSource + ")" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("Next" + Environment.NewLine);
        }

        private void LoopToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string value = "0";
            if (this.Parameters.ContainsKey("value")) value = this.ReplaceStringParamater("value");
            expr.Convert(converter, value, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource("for($" + counter + " = 1;$" + counter + " -le " + converter.CurrentFunction.CacheSource + ";++$" + counter + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
        }

        private void LoopInverseToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            // il nous faut cette variable counter dans le processus courant pour que counter puisse tre exprim
            // dans value comme cela est possible dans le code
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string value = "0";
            if (this.Parameters.ContainsKey("value")) value = this.ReplaceStringParamater("value");
            expr.Convert(converter, value, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource("For " + counter + " = CLng(" + converter.CurrentFunction.CacheSource + ") To 1 Step -1" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("Next" + Environment.NewLine);
        }

        private void LoopInverseToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            // il nous faut cette variable counter dans le processus courant pour que counter puisse tre exprim
            // dans value comme cela est possible dans le code
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string value = "0";
            if (this.Parameters.ContainsKey("value")) value = this.ReplaceStringParamater("value");
            expr.Convert(converter, value, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource("for($" + counter + " = " + converter.CurrentFunction.CacheSource + ";$" + counter + " -ge 1;--$" + counter + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
        }

        private void LoopWhileToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            // il nous faut cette variable counter dans le processus courant pour que counter puisse tre exprim
            // dans init comme cela est possible dans le code
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "0";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            expr.Convert(converter, limit, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            limit = converter.CurrentFunction.CacheSource;
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource(counter + " = " + init + Environment.NewLine);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = True" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("While CBool(" + limit + ")" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If Not " + name + "First Then" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // on ne peut pas ranger la variable nameFirst dans le mme processus que la sous-procdure
                // alors on le met dans le processus en cours (dfaut)
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais on met nameFirst en local dans la fonction courante afin que l'on n'utilise pas cette variable en rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            }
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource(counter + " = " + counter + " + 1" + Environment.NewLine);
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = False" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("Wend" + Environment.NewLine);
        }

        private void LoopWhileToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            // on ne peut pas ajouter le compteur dans le sous-processus du coding alors on le met dans le processus courant (c'est pas grave)
            // parce que le processus coding ne cre pas de fonction
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            // on ajoute le compteur dans les variables locales  la fonction en cours pour ne pas qu'elle soit considre en rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "0";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            expr.Convert(converter, limit, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            limit = converter.CurrentFunction.CacheSource;
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource("$" + counter + " = " + init + Environment.NewLine);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("while (" + limit + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If (!$" + name + "First)" + Environment.NewLine + "{" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // on ne peut pas ranger la variable nameFirst dans le mme processus que la sous-procdure
                // alors on le met dans le processus en cours (dfaut)
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais nameFirst est dans une variable locale pour ne pas qu'elle soit considre comme une rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            }
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("++$" + counter + Environment.NewLine);
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $false" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
        }

        private void LoopWithWhileToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            // il nous faut cette variable counter dans le processus courant pour que counter puisse tre exprim
            // dans init comme cela est possible dans le code
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "limit";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(limit))
                this.cachedComp.CurrentProcess.CurrentScope.Add(limit, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == limit; })))
                converter.CurrentFunction.LocalVariables.Add(limit);
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource(counter + " = " + init + Environment.NewLine);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = True" + Environment.NewLine);
            converter.CurrentFunction.AddToSource(limit + " = true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("While CBool(" + limit + ")" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingLimit = this.GetCoding("limit");
            if (codingLimit != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingLimit.CodingName);
                this.cachedComp.RemoveCodingProcess(codingLimit.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("If CBool(" + limit + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If Not " + name + "First Then" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // on ne peut pas ranger la variable nameFirst dans le mme processus que la sous-procdure
                // alors on le met dans le processus en cours (dfaut)
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais on met nameFirst en local dans la fonction courante afin que l'on n'utilise pas cette variable en rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            }
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            Coding codingIncrement = this.GetCoding("increment");
            if (codingIncrement != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingIncrement.CodingName);
                this.cachedComp.RemoveCodingProcess(codingIncrement.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = False" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("Wend" + Environment.NewLine);
        }

        private void LoopWithWhileToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            // on ne peut pas ajouter le compteur dans le sous-processus du coding alors on le met dans le processus courant (c'est pas grave)
            // parce que le processus coding ne cre pas de fonction
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            // on ajoute le compteur dans les variables locales  la fonction en cours pour ne pas qu'elle soit considre en rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "limit";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(limit))
                this.cachedComp.CurrentProcess.CurrentScope.Add(limit, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == limit; })))
                converter.CurrentFunction.LocalVariables.Add(limit);
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource("$" + counter + " = " + init + Environment.NewLine);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("$" + limit + " = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("while ($" + limit + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingLimit = this.GetCoding("limit");
            if (codingLimit != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingLimit.CodingName);
                this.cachedComp.RemoveCodingProcess(codingLimit.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("if ($" + limit + ") {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If (!$" + name + "First)" + Environment.NewLine + "{" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // on ne peut pas ranger la variable nameFirst dans le mme processus que la sous-procdure
                // alors on le met dans le processus en cours (dfaut)
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais nameFirst est dans une variable locale pour ne pas qu'elle soit considre comme une rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            }
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            Coding codingIncrement = this.GetCoding("increment");
            if (codingIncrement != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingIncrement.CodingName);
                this.cachedComp.RemoveCodingProcess(codingIncrement.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $false" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
        }

        private void LoopUntilToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            // il nous faut cette variable counter dans le processus courant pour que counter puisse tre exprim
            // dans init comme cela est possible dans le code
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "0";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            expr.Convert(converter, limit, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            limit = converter.CurrentFunction.CacheSource;
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource(counter + " = " + init + Environment.NewLine);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = True" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("Do" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If Not " + name + "First Then" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // on ne peut pas ranger la variable nameFirst dans le mme processus que la sous-procdure
                // alors on le met dans le processus en cours (dfaut)
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais on met nameFirst en local dans la fonction courante afin que l'on n'utilise pas cette variable en rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            }
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                converter.CurrentFunction.LocalVariables.Add(counter);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource(counter + " = " + counter + " + 1" + Environment.NewLine);
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = False" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("While CBool(" + limit + ")" + Environment.NewLine);
        }

        private void LoopUntilToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            // on ne peut pas ajouter le compteur dans le sous-processus du coding alors on le met dans le processus courant (c'est pas grave)
            // parce que le processus coding ne cre pas de fonction
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            // on ajoute le compteur dans les variables locales  la fonction en cours pour ne pas qu'elle soit considre en rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "0";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            expr.Convert(converter, limit, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            limit = converter.CurrentFunction.CacheSource;
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource("$" + counter + " = " + init + Environment.NewLine);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("do {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If (!$" + name + "First)" + Environment.NewLine + "{" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // on ne peut pas ranger la variable nameFirst dans le mme processus que la sous-procdure
                // alors on le met dans le processus en cours (dfaut)
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais nameFirst est dans une variable locale pour ne pas qu'elle soit considre comme une rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            }
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("++$" + counter + Environment.NewLine);
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $false" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("} while (" + limit + ")" + Environment.NewLine);
        }

        private void LoopWithUntilToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            // il nous faut cette variable counter dans le processus courant pour que counter puisse tre exprim
            // dans init comme cela est possible dans le code
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "limit";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(limit))
                this.cachedComp.CurrentProcess.CurrentScope.Add(limit, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == limit; })))
                converter.CurrentFunction.LocalVariables.Add(limit);
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource(counter + " = " + init + Environment.NewLine);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = True" + Environment.NewLine);
            converter.CurrentFunction.AddToSource(limit + " = true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("Do" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingLimit = this.GetCoding("limit");
            if (codingLimit != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingLimit.CodingName);
                this.cachedComp.RemoveCodingProcess(codingLimit.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("If CBool(" + limit + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If Not " + name + "First Then" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // on ne peut pas ranger la variable nameFirst dans le mme processus que la sous-procdure
                // alors on le met dans le processus en cours (dfaut)
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais on met nameFirst en local dans la fonction courante afin que l'on n'utilise pas cette variable en rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            }
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                converter.CurrentFunction.LocalVariables.Add(counter);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            Coding codingIncrement = this.GetCoding("increment");
            if (codingIncrement != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingIncrement.CodingName);
                this.cachedComp.RemoveCodingProcess(codingIncrement.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = False" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("While CBool(" + limit + ")" + Environment.NewLine);
        }

        private void LoopWithUntilToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            // on ne peut pas ajouter le compteur dans le sous-processus du coding alors on le met dans le processus courant (c'est pas grave)
            // parce que le processus coding ne cre pas de fonction
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            // on ajoute le compteur dans les variables locales  la fonction en cours pour ne pas qu'elle soit considre en rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "limit";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(limit))
                this.cachedComp.CurrentProcess.CurrentScope.Add(limit, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == limit; })))
                converter.CurrentFunction.LocalVariables.Add(limit);
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource("$" + counter + " = " + init + Environment.NewLine);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("$" + limit + " = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("do {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingLimit = this.GetCoding("limit");
            if (codingLimit != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingLimit.CodingName);
                this.cachedComp.RemoveCodingProcess(codingLimit.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("if ($" + limit + ") {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If (!$" + name + "First)" + Environment.NewLine + "{" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // on ne peut pas ranger la variable nameFirst dans le mme processus que la sous-procdure
                // alors on le met dans le processus en cours (dfaut)
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais nameFirst est dans une variable locale pour ne pas qu'elle soit considre comme une rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            }
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            Coding codingIncrement = this.GetCoding("increment");
            if (codingIncrement != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingIncrement.CodingName);
                this.cachedComp.RemoveCodingProcess(codingIncrement.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $false" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("} while (" + limit + ")" + Environment.NewLine);
        }

        private void LoopForEachToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string tabName = "tableau";
            if (this.Parameters.ContainsKey("tabName")) tabName = this.ReplaceStringParamater("tabName");
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = True" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("For " + counter + " = (" + init + ") To (theDict.GetArray(\"" + tabName + "\").Count)" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If Not " + name + "First Then" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // on ne peut pas ranger la variable nameFirst dans le mme processus que la sous-procdure
                // alors on le met dans le processus en cours (dfaut)
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais on met nameFirst en local dans la fonction courante afin que l'on n'utilise pas cette variable en rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            }
            Coding codingBody = this.GetCoding("body");
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = False" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("Next" + Environment.NewLine);
        }

        private void LoopForEachToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "0";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            expr.Convert(converter, limit, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            limit = converter.CurrentFunction.CacheSource;
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $true" + Environment.NewLine);
            string tabName = "tableau";
            if (this.Parameters.ContainsKey("tabName")) tabName = this.ReplaceStringParamater("tabName");
            converter.CurrentFunction.AddToSource("for($" + counter + " = (" + init + ");$" + counter + " -le ($theDict.GetArray(\"" + tabName + "\").Count);++$" + counter + ") {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If (!$" + name + "First)" + Environment.NewLine + "{" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // on ne peut pas ranger la variable nameFirst dans le mme processus que la sous-procdure
                // alors on le met dans le processus en cours (dfaut)
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais nameFirst est dans une variable locale pour ne pas qu'elle soit considre comme une rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            }
            Coding codingBody = this.GetCoding("body");
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $false" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
        }

        private void LoopConditionalForEachToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            // on ne peut pas ranger la variable condition dans le mme processus que la sous-procdure
            // alors on le met dans le processus en cours (dfaut)
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "0", this.cachedComp.CurrentProcess.Name);
            string tabName = "tableau";
            if (this.Parameters.ContainsKey("tabName")) tabName = this.ReplaceStringParamater("tabName");
            init = converter.CurrentFunction.CacheSource;
            Coding codingBody = this.GetCoding("body");
            converter.CurrentFunction.AddToSource("For " + counter + " = (" + init + ") To (theDict.GetArray(\"" + tabName + "\").Count)" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            converter.CurrentFunction.AddToSource(condition + " = trueValue" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("If CBool(" + condition + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("Next" + Environment.NewLine);
        }

        private void LoopCondtionalForEachToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "0";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            expr.Convert(converter, limit, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            limit = converter.CurrentFunction.CacheSource;
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            Coding codingBody = this.GetCoding("body");
            string tabName = "tableau";
            if (this.Parameters.ContainsKey("tabName")) tabName = this.ReplaceStringParamater("tabName");
            converter.CurrentFunction.AddToSource("for($" + counter + " = (" + init + ");$" + counter + " < ($theDict.GetArray(\"" + tabName + "\").Count;++$" + counter + ") {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            converter.CurrentFunction.AddToSource("$" + condition + " = $true" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("if ($" + condition + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("++$" + counter + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
        }

        private void LoopConditionalWhileToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "0";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            expr.Convert(converter, limit, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            limit = converter.CurrentFunction.CacheSource;
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            // on ne peut pas ranger la variable condition dans le mme processus que la sous-procdure
            // alors on le met dans le processus en cours (dfaut)
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "0", this.cachedComp.CurrentProcess.Name);
            converter.CurrentFunction.AddToSource(counter + " = CLng(" + init + ")" + Environment.NewLine);
            // on ajoute condition dans les variables locales pour qu'elle ne soit pas considre comme une rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = True" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("While CBool(" + limit + ")" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            converter.CurrentFunction.AddToSource(condition + " = trueValue" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("If CBool(" + condition + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If Not " + name + "First Then" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // nameFirst sera dans le processus en cours parce qu'on n'arrive pas  l'ajouter dans le sous-processus du coding
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais on le considre comme une variable locale et non pas une variable en rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate (string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            }
            Coding codingBody = this.GetCoding("body");
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            converter.CurrentFunction.AddToSource(counter + " = " + counter + " + 1" + Environment.NewLine);
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = False" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("Wend" + Environment.NewLine);
        }

        private void LoopConditionalWhileToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "0";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            expr.Convert(converter, limit, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            limit = converter.CurrentFunction.CacheSource;
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            // la variable condition n'est pas le processus du coding mais tant pis
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "0", this.cachedComp.CurrentProcess.Name);
            converter.CurrentFunction.AddToSource("$" + counter + " = " + init + Environment.NewLine);
            // en revanche, la variable condition est bien dans une variable locale, ce qui fait qu'elle n'est pas vue comme une rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate (string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("while (" + limit + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            converter.CurrentFunction.AddToSource("$" + condition + " = $true" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("if ($" + condition + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If (!$" + name + "First)" + Environment.NewLine + "{" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // nameFirst n'est pas le processus du coding car on ne peut pas le faire
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // en revanche, elle est dans les variables locales donc elle n'est pas considre comme une rfrence
                // si on utilise la variable condition dans le coding, elle est susceptible de devenir un paramtre
                // mais elle restera galement dans les variables locales et donc elle n'est pas en rfrence
                // de plus, un processus coding ne fait pas une fonction dans PowerShell
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            }
            Coding codingBody = this.GetCoding("body");
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("++$" + counter + Environment.NewLine);
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $false" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
        }

        private void LoopWithConditionalWhileToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "limit";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(limit))
                this.cachedComp.CurrentProcess.CurrentScope.Add(limit, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == limit; })))
                converter.CurrentFunction.LocalVariables.Add(limit);
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            // on ne peut pas ranger la variable condition dans le mme processus que la sous-procdure
            // alors on le met dans le processus en cours (dfaut)
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "0", this.cachedComp.CurrentProcess.Name);
            converter.CurrentFunction.AddToSource(counter + " = CLng(" + init + ")" + Environment.NewLine);
            // on ajoute condition dans les variables locales pour qu'elle ne soit pas considre comme une rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = True" + Environment.NewLine);
            converter.CurrentFunction.AddToSource(limit + " = True" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("While CBool(" + limit + ")" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingLimit = this.GetCoding("limit");
            if (codingLimit != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingLimit.CodingName);
                this.cachedComp.RemoveCodingProcess(codingLimit.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("If CBool(" + limit + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            converter.CurrentFunction.AddToSource(condition + " = trueValue" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("If CBool(" + condition + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If Not " + name + "First Then" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // nameFirst sera dans le processus en cours parce qu'on n'arrive pas  l'ajouter dans le sous-processus du coding
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais on le considre comme une variable locale et non pas une variable en rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            }
            Coding codingBody = this.GetCoding("body");
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            Coding codingIncrement = this.GetCoding("increment");
            if (codingIncrement != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingIncrement.CodingName);
                this.cachedComp.RemoveCodingProcess(codingIncrement.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = False" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("Wend" + Environment.NewLine);
        }

        private void LoopWithConditionalWhileToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "limit";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(limit))
                this.cachedComp.CurrentProcess.CurrentScope.Add(limit, "0", this.cachedComp.CurrentProcess.Name);
            // en revanche, la variable limit est bien dans une variable locale, ce qui fait qu'elle n'est pas vue comme une rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == limit; })))
                converter.CurrentFunction.LocalVariables.Add(limit);
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            // la variable condition n'est pas le processus du coding mais tant pis
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "0", this.cachedComp.CurrentProcess.Name);
            converter.CurrentFunction.AddToSource("$" + counter + " = " + init + Environment.NewLine);
            // en revanche, la variable condition est bien dans une variable locale, ce qui fait qu'elle n'est pas vue comme une rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("$" + limit + " = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("while ($" + limit + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingLimit = this.GetCoding("limit");
            if (codingLimit != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingLimit.CodingName);
                this.cachedComp.RemoveCodingProcess(codingLimit.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("if ($" + limit + ") {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            converter.CurrentFunction.AddToSource("$" + condition + " = $true" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("if ($" + condition + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If (!$" + name + "First)" + Environment.NewLine + "{" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // nameFirst n'est pas le processus du coding car on ne peut pas le faire
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // en revanche, elle est dans les variables locales donc elle n'est pas considre comme une rfrence
                // si on utilise la variable condition dans le coding, elle est susceptible de devenir un paramtre
                // mais elle restera galement dans les variables locales et donc elle n'est pas en rfrence
                // de plus, un processus coding ne fait pas une fonction dans PowerShell
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            }
            Coding codingBody = this.GetCoding("body");
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            Coding codingIncrement = this.GetCoding("increment");
            if (codingIncrement != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingIncrement.CodingName);
                this.cachedComp.RemoveCodingProcess(codingIncrement.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $false" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
        }

        private void LoopConditionalUntilToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "0";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            expr.Convert(converter, limit, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            limit = converter.CurrentFunction.CacheSource;
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            // on ne peut pas ranger la variable condition dans le mme processus que la sous-procdure
            // alors on le met dans le processus en cours (dfaut)
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "0", this.cachedComp.CurrentProcess.Name);
            converter.CurrentFunction.AddToSource(counter + " = CLng(" + init + ")" + Environment.NewLine);
            // on ajoute condition dans les variables locales pour qu'elle ne soit pas considre comme une rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = True" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("Do" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            converter.CurrentFunction.AddToSource(condition + " = trueValue" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("If CBool(" + condition + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If Not " + name + "First Then" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // nameFirst sera dans le processus en cours parce qu'on n'arrive pas  l'ajouter dans le sous-processus du coding
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais on le considre comme une variable locale et non pas une variable en rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            }
            Coding codingBody = this.GetCoding("body");
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            converter.CurrentFunction.AddToSource(counter + " = " + counter + " + 1" + Environment.NewLine);
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = False" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("While CBool(" + limit + ")" + Environment.NewLine);
        }

        private void LoopConditionalUntilToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "0";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            expr.Convert(converter, limit, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            limit = converter.CurrentFunction.CacheSource;
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            // la variable condition n'est pas le processus du coding mais tant pis
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "0", this.cachedComp.CurrentProcess.Name);
            converter.CurrentFunction.AddToSource("$" + counter + " = " + init + Environment.NewLine);
            // en revanche, la variable condition est bien dans une variable locale, ce qui fait qu'elle n'est pas vue comme une rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("do {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            converter.CurrentFunction.AddToSource("$" + condition + " = $true" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("if ($" + condition + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If (!$" + name + "First)" + Environment.NewLine + "{" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // nameFirst n'est pas le processus du coding car on ne peut pas le faire
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // en revanche, elle est dans les variables locales donc elle n'est pas considre comme une rfrence
                // si on utilise la variable condition dans le coding, elle est susceptible de devenir un paramtre
                // mais elle restera galement dans les variables locales et donc elle n'est pas en rfrence
                // de plus, un processus coding ne fait pas une fonction dans PowerShell
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            }
            Coding codingBody = this.GetCoding("body");
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("++$" + counter + Environment.NewLine);
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $false" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("} while (" + limit + ")" + Environment.NewLine);
        }

        private void LoopWithConditionalUntilToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "limit";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(limit))
                this.cachedComp.CurrentProcess.CurrentScope.Add(limit, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == limit; })))
                converter.CurrentFunction.LocalVariables.Add(limit);
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            // on ne peut pas ranger la variable condition dans le mme processus que la sous-procdure
            // alors on le met dans le processus en cours (dfaut)
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "0", this.cachedComp.CurrentProcess.Name);
            converter.CurrentFunction.AddToSource(counter + " = CLng(" + init + ")" + Environment.NewLine);
            // on ajoute condition dans les variables locales pour qu'elle ne soit pas considre comme une rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            converter.CurrentFunction.AddToSource(limit + " = True" + Environment.NewLine);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = True" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("Do" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingLimit = this.GetCoding("limit");
            if (codingLimit != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingLimit.CodingName);
                this.cachedComp.RemoveCodingProcess(codingLimit.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("If CBool(" + limit + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            converter.CurrentFunction.AddToSource(condition + " = trueValue" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("If CBool(" + condition + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If Not " + name + "First Then" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // nameFirst sera dans le processus en cours parce qu'on n'arrive pas  l'ajouter dans le sous-processus du coding
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // mais on le considre comme une variable locale et non pas une variable en rfrence
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            }
            Coding codingBody = this.GetCoding("body");
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            Coding codingIncrement = this.GetCoding("increment");
            if (codingIncrement != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingIncrement.CodingName);
                this.cachedComp.RemoveCodingProcess(codingIncrement.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            if (codingNext != null)
                converter.CurrentFunction.AddToSource(name + "First = False" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("While CBool(" + limit + ")" + Environment.NewLine);
        }

        private void LoopWithConditionalUntilToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string name = "name";
            if (this.Parameters.ContainsKey("name")) name = this.ReplaceStringParamater("name");
            string counter = "counter";
            if (this.Parameters.ContainsKey("counter")) counter = this.ReplaceStringParamater("counter");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(counter))
                this.cachedComp.CurrentProcess.CurrentScope.Add(counter, "0", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == counter; })))
                converter.CurrentFunction.LocalVariables.Add(counter);
            string init = "0";
            if (this.Parameters.ContainsKey("init")) init = this.ReplaceStringParamater("init");
            expr.Convert(converter, init, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            init = converter.CurrentFunction.CacheSource;
            string limit = "limit";
            if (this.Parameters.ContainsKey("limit")) limit = this.ReplaceStringParamater("limit");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(limit))
                this.cachedComp.CurrentProcess.CurrentScope.Add(limit, "0", this.cachedComp.CurrentProcess.Name);
            // en revanche, la variable limit est bien dans une variable locale, ce qui fait qu'elle n'est pas vue comme une rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == limit; })))
                converter.CurrentFunction.LocalVariables.Add(limit);
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            // la variable condition n'est pas le processus du coding mais tant pis
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "0", this.cachedComp.CurrentProcess.Name);
            converter.CurrentFunction.AddToSource("$" + counter + " = " + init + Environment.NewLine);
            // en revanche, la variable condition est bien dans une variable locale, ce qui fait qu'elle n'est pas vue comme une rfrence
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            Coding codingNext = this.GetCoding("next");
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("$" + limit + " = $true" + Environment.NewLine);
            converter.CurrentFunction.AddToSource("do {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingLimit = this.GetCoding("limit");
            if (codingLimit != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingLimit.CodingName);
                this.cachedComp.RemoveCodingProcess(codingLimit.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("if ($" + limit + ") {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            converter.CurrentFunction.AddToSource("$" + condition + " = $true" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.AddToSource("if ($" + condition + ")" + Environment.NewLine + "{" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingNext != null)
            {
                converter.CurrentFunction.AddToSource("If (!$" + name + "First)" + Environment.NewLine + "{" + Environment.NewLine);
                IProcess proc = this.cachedComp.GetCodingProcess(codingNext.CodingName);
                // nameFirst n'est pas le processus du coding car on ne peut pas le faire
                if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(name + "First"))
                    this.cachedComp.CurrentProcess.CurrentScope.Add(name + "First", "0", this.cachedComp.CurrentProcess.Name);
                // en revanche, elle est dans les variables locales donc elle n'est pas considre comme une rfrence
                // si on utilise la variable condition dans le coding, elle est susceptible de devenir un paramtre
                // mais elle restera galement dans les variables locales et donc elle n'est pas en rfrence
                // de plus, un processus coding ne fait pas une fonction dans PowerShell
                if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == name + "First"; })))
                    converter.CurrentFunction.LocalVariables.Add(name + "First");
                converter.CurrentFunction.IndentSize += 1;
                this.cachedComp.RemoveCodingProcess(codingNext.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
                converter.CurrentFunction.IndentSize -= 1;
                converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            }
            Coding codingBody = this.GetCoding("body");
            if (codingBody != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingBody.CodingName);
                this.cachedComp.RemoveCodingProcess(codingBody.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            Coding codingIncrement = this.GetCoding("increment");
            if (codingIncrement != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingIncrement.CodingName);
                this.cachedComp.RemoveCodingProcess(codingIncrement.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            if (codingNext != null)
                converter.CurrentFunction.AddToSource("$" + name + "First = $false" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("} while (" + limit + ")" + Environment.NewLine);
        }

        private void ConditionToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string condition = "";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            expr.Convert(converter, condition, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            Coding codingTrue = this.GetCoding("true");
            converter.CurrentFunction.AddToSource("If CBool(" + converter.CurrentFunction.CacheSource + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingTrue != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingTrue.CodingName);
                this.cachedComp.RemoveCodingProcess(codingTrue.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("Else" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingFalse = this.GetCoding("false");
            if (codingFalse != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingFalse.CodingName);
                this.cachedComp.RemoveCodingProcess(codingFalse.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
        }

        private void ConditionToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string condition = "";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            expr.Convert(converter, condition , this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            Coding codingTrue = this.GetCoding("true");
            converter.CurrentFunction.AddToSource("if (" + converter.CurrentFunction.CacheSource + ") {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingTrue != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingTrue.CodingName);
                this.cachedComp.RemoveCodingProcess(codingTrue.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("} else {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingFalse = this.GetCoding("false");
            if (codingFalse != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingFalse.CodingName);
                this.cachedComp.RemoveCodingProcess(codingFalse.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
        }

        private void WithConditionToVBScript(ICodeConverter converter)
        {
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            converter.CurrentFunction.AddToSource(condition + " = True" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            Coding codingTrue = this.GetCoding("true");
            converter.CurrentFunction.AddToSource("If CBool(" + condition + ") Then" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingTrue != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingTrue.CodingName);
                this.cachedComp.RemoveCodingProcess(codingTrue.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("Else" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingFalse = this.GetCoding("false");
            if (codingFalse != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingFalse.CodingName);
                this.cachedComp.RemoveCodingProcess(codingFalse.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("End If" + Environment.NewLine);
        }

        private void WithConditionToPowerShell(ICodeConverter converter)
        {
            string condition = "condition";
            if (this.Parameters.ContainsKey("condition")) condition = this.ReplaceStringParamater("condition");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(condition))
                this.cachedComp.CurrentProcess.CurrentScope.Add(condition, "", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == condition; })))
                converter.CurrentFunction.LocalVariables.Add(condition);
            converter.CurrentFunction.AddToSource("$" + condition + " = $true" + Environment.NewLine);
            Coding codingCondition = this.GetCoding("condition");
            if (codingCondition != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingCondition.CodingName);
                this.cachedComp.RemoveCodingProcess(codingCondition.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            Coding codingTrue = this.GetCoding("true");
            converter.CurrentFunction.AddToSource("if ($" + condition + ") {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            if (codingTrue != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingTrue.CodingName);
                this.cachedComp.RemoveCodingProcess(codingTrue.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("} else {" + Environment.NewLine);
            converter.CurrentFunction.IndentSize += 1;
            Coding codingFalse = this.GetCoding("false");
            if (codingFalse != null)
            {
                IProcess proc = this.cachedComp.GetCodingProcess(codingFalse.CodingName);
                this.cachedComp.RemoveCodingProcess(codingFalse.CodingName);
                this.cachedComp.Convert(converter, proc, this.cachedFile);
            }
            converter.CurrentFunction.IndentSize -= 1;
            converter.CurrentFunction.AddToSource("}" + Environment.NewLine);
        }

        private void StringLengthToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string value = "";
            if (this.Parameters.ContainsKey("value")) value = this.ReplaceStringParamater("value");
            expr.Convert(converter, value, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            string result = "result";
            if (this.Parameters.ContainsKey("out")) result = this.ReplaceStringParamater("out");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(result))
                this.cachedComp.CurrentProcess.CurrentScope.Add(result, "", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == result; })))
                converter.CurrentFunction.LocalVariables.Add(result);
            converter.CurrentFunction.AddToSource(result + " = Len(" + converter.CurrentFunction.CacheSource + ")" + Environment.NewLine);
        }

        private void StringLengthToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string value = "";
            if (this.Parameters.ContainsKey("value")) value = this.ReplaceStringParamater("value");
            expr.Convert(converter, value, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            string result = "result";
            if (this.Parameters.ContainsKey("out")) result = this.ReplaceStringParamater("out");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(result))
                this.cachedComp.CurrentProcess.CurrentScope.Add(result, "", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == result; })))
                converter.CurrentFunction.LocalVariables.Add(result);
            converter.CurrentFunction.AddToSource("$" + result + " = (" + converter.CurrentFunction.CacheSource + ").Length" + Environment.NewLine);
        }

        private void StringSubstringToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string value = "";
            if (this.Parameters.ContainsKey("value")) value = this.ReplaceStringParamater("value");
            expr.Convert(converter, value, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            value = converter.CurrentFunction.CacheSource;
            string start = "0";
            if (this.Parameters.ContainsKey("start")) start = this.ReplaceStringParamater("start");
            expr.Convert(converter, start, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            start = converter.CurrentFunction.CacheSource;
            string length = "0";
            if (this.Parameters.ContainsKey("length")) length = this.ReplaceStringParamater("length");
            expr.Convert(converter, length, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            length = converter.CurrentFunction.CacheSource;
            string result = "result";
            if (this.Parameters.ContainsKey("out")) result = this.ReplaceStringParamater("out");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(result))
                this.cachedComp.CurrentProcess.CurrentScope.Add(result, "", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == result; })))
                converter.CurrentFunction.LocalVariables.Add(result);
            converter.CurrentFunction.AddToSource(result + " = Mid(" + value + ", CLng(" + start + ") + 1, CLng(" + length + "))" + Environment.NewLine);
        }

        private void StringSubstringToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string value = "";
            if (this.Parameters.ContainsKey("value")) value = this.ReplaceStringParamater("value");
            expr.Convert(converter, value, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            value = converter.CurrentFunction.CacheSource;
            string start = "0";
            if (this.Parameters.ContainsKey("start")) start = this.ReplaceStringParamater("start");
            expr.Convert(converter, start, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            start = converter.CurrentFunction.CacheSource;
            string length = "0";
            if (this.Parameters.ContainsKey("length")) length = this.ReplaceStringParamater("length");
            expr.Convert(converter, length, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            length = converter.CurrentFunction.CacheSource;
            string result = "result";
            if (this.Parameters.ContainsKey("out")) result = this.ReplaceStringParamater("out");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(result))
                this.cachedComp.CurrentProcess.CurrentScope.Add(result, "", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == result; })))
                converter.CurrentFunction.LocalVariables.Add(result);
            converter.CurrentFunction.AddToSource(result + " = (" + value + ").SubString(" + start + ", " + length + ")" + Environment.NewLine);
        }

        private void StringReplaceToVBScript(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string value = "";
            if (this.Parameters.ContainsKey("value")) value = this.ReplaceStringParamater("value");
            expr.Convert(converter, value, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            value = converter.CurrentFunction.CacheSource;
            string search = "";
            if (this.Parameters.ContainsKey("search")) search = this.ReplaceStringParamater("search");
            expr.Convert(converter, search, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            search = converter.CurrentFunction.CacheSource;
            string replacement = "0";
            if (this.Parameters.ContainsKey("replacement")) replacement = this.ReplaceStringParamater("replacement");
            expr.Convert(converter, replacement, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            replacement = converter.CurrentFunction.CacheSource;
            string result = "result";
            if (this.Parameters.ContainsKey("out")) result = this.ReplaceStringParamater("out");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(result))
                this.cachedComp.CurrentProcess.CurrentScope.Add(result, "", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == result; })))
                converter.CurrentFunction.LocalVariables.Add(result);
            converter.CurrentFunction.AddToSource(result + " = Replace(" + value + ", CStr(" + search + "), CStr(" + replacement + "))" + Environment.NewLine);
        }

        private void StringReplaceToPowerShell(ICodeConverter converter)
        {
            o2Mate.Expression expr = new o2Mate.Expression();
            string value = "";
            if (this.Parameters.ContainsKey("value")) value = this.ReplaceStringParamater("value");
            expr.Convert(converter, value, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            value = converter.CurrentFunction.CacheSource;
            string search = "";
            if (this.Parameters.ContainsKey("search")) search = this.ReplaceStringParamater("search");
            expr.Convert(converter, search, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            search = converter.CurrentFunction.CacheSource;
            string replacement = "0";
            if (this.Parameters.ContainsKey("replacement")) replacement = this.ReplaceStringParamater("replacement");
            expr.Convert(converter, replacement, this.cachedComp.CurrentProcess.CurrentScope);
            foreach (string used in converter.CurrentFunction.UsedVariables)
            {
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, this.cachedComp.CurrentProcess, used, false);
            }
            replacement = converter.CurrentFunction.CacheSource;
            string result = "result";
            if (this.Parameters.ContainsKey("out")) result = this.ReplaceStringParamater("out");
            if (!this.cachedComp.CurrentProcess.CurrentScope.Exists(result))
                this.cachedComp.CurrentProcess.CurrentScope.Add(result, "", this.cachedComp.CurrentProcess.Name);
            if (!converter.CurrentFunction.LocalVariables.Exists(new Predicate<string>(delegate(string s) { return s == result; })))
                converter.CurrentFunction.LocalVariables.Add(result);
            converter.CurrentFunction.AddToSource(result + " = (" + value + ") -replace " + search + ", " + replacement + Environment.NewLine);
        }

        public void WriteInVBScript(ICodeConverter converter)
        {
            foreach (NameConverter conv in this.nameVBScriptConverters)
            {
                if (conv.name == this.Name)
                {
                    conv.del(converter);
                }
            }
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            foreach (NameConverter conv in this.namePowerShellConverters)
            {
                if (conv.name == this.Name)
                {
                    conv.del(converter);
                }
            }
        }

        #endregion
    }
}
