using System;
using System.Collections.Generic;
using System.Text;
using Converters;

namespace o2Mate
{
    internal class Variable : Compiler, ILanguageConverter
    {
        #region Private Fields
        private string varName;
        private string realName;
        private bool isFromDict;
        private int indent;
        private ICompilateur cachedComp;
        private string defaultWriter;
        #endregion

        #region Public Properties
        public string VariableName
        {
            get { return this.varName; }
        }
        #endregion

        #region Private Methods
        private System.Windows.Forms.HtmlElement GetElementByName(System.Windows.Forms.HtmlElement from, string name)
        {
            if (from.Name == name)
            {
                return from;
            }
            else
            {
                foreach (System.Windows.Forms.HtmlElement child in from.Children)
                {
                    try
                    {
                        return GetElementByName(child, name);
                    }
                    catch { }
                }
                throw new Exception("this sub-element does not contain that name element");
            }
        }
        #endregion

        #region Compiler Membres

        public string TypeName
        {
            get { return "Variable"; }
        }

        public void ExtractDictionary(IProcess proc)
        {
            if (!proc.CurrentScope.Exists(this.varName))
            {
                proc.CurrentDictionnaire.AddString(this.varName, "");
            }
        }

        public void Display(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayVariable", new object[] { this.varName, this.indent.ToString(), false });
        }

        public void DisplayReadOnly(System.Windows.Forms.HtmlDocument doc, Node<Compiler> node, bool forcedIndent, ref int indent)
        {
            if (forcedIndent)
            {
                this.indent = indent;
            }
            doc.InvokeScript("displayVariable", new object[] { this.varName, this.indent.ToString(), true });
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            Int32.TryParse(elem.GetAttribute("indent"), out this.indent);
            this.varName = this.GetElementByName(elem, "variable").InnerText;
        }

        public void Save(ICompilateur comp, System.Xml.XmlWriter writer, ref System.Windows.Forms.HtmlElement child)
        {
            writer.WriteStartElement("variable");
            if (this.indent > 0)
                writer.WriteAttributeString("indent", this.indent.ToString());
            writer.WriteString(this.varName);
            writer.WriteEndElement();
            child = child.NextSibling;
        }

        public void Parse(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.varName = node.InnerText;
        }

        public void Inject(Compiler obj, ICompilateur comp, System.Xml.XmlNode node, string modifier)
        {
            this.Load(comp, node);
        }

        public void Load(ICompilateur comp, System.Xml.XmlNode node)
        {
            if (node.Attributes.GetNamedItem("indent") != null)
                Int32.TryParse(node.Attributes.GetNamedItem("indent").Value, out this.indent);
            this.varName = node.InnerText;
            this.cachedComp = comp;
        }

        public void WriteToFile(IProcess proc, FinalFile file)
        {
            if (proc.CurrentScope.Exists(this.varName))
            {
                if (proc.CurrentScope.Exists(proc.DefaultWriter))
                {
                    string writer = proc.CurrentScope.GetVariable(proc.DefaultWriter).ValueString;
                    FinalFile.WriteToFile(ref writer, proc.CurrentScope.GetVariable(this.varName).ValueString);
                    proc.CurrentScope.GetVariable(proc.DefaultWriter).Value = writer;
                    proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "print variable", this.varName));
                }
            }
            else if (proc.CurrentDictionnaire.Exists(this.varName))
            {
                if (proc.CurrentDictionnaire.IsString(this.varName))
                {
                    if (proc.CurrentScope.Exists(proc.DefaultWriter))
                    {
                        string writer = proc.CurrentScope.GetVariable(proc.DefaultWriter).ValueString;
                        FinalFile.WriteToFile(ref writer, proc.CurrentDictionnaire.GetString(this.varName));
                        proc.CurrentScope.GetVariable(proc.DefaultWriter).Value = writer;
                        proc.CurrentProject.Add(new ProjectItem(proc.Name, proc.CurrentPositionExecution, "print string", this.varName));
                    }
                }
            }
        }

        public void Convert(ICodeConverter converter, IProcess proc, FinalFile file)
        {
            if (proc.CurrentScope.Exists(this.varName))
            {
                this.isFromDict = false;
                this.realName = proc.CurrentScope.GetVariable(this.varName).RealName;
                this.cachedComp.BacktrackRunProcessesAndUpdateParameters(converter, proc, this.realName, false);
                if (proc.CurrentScope.Exists(proc.DefaultWriter))
                {
                    this.defaultWriter = proc.DefaultWriter;
                    converter.Convert(this);
                }
            }
            else if (proc.CurrentDictionnaire.IsString(this.varName))
            {
                this.isFromDict = true;
                if (proc.CurrentScope.Exists(proc.DefaultWriter))
                {
                    this.defaultWriter = proc.DefaultWriter;
                    converter.Convert(this);
                }
            }
        }

        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
            if (this.isFromDict)
            {
                converter.CurrentFunction.AddToSource("WriteToFile " + this.defaultWriter + ", theDict.GetString(\"" + this.varName + "\")" + Environment.NewLine);
            }
            else
            {
                converter.CurrentFunction.AddToSource("WriteToFile " + this.defaultWriter + ", " + this.realName + Environment.NewLine);
            }
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            if (this.isFromDict)
            {
                converter.CurrentFunction.AddToSource("WriteToFile $" + this.defaultWriter + " $theDict.GetString(\"" + this.varName + "\")" + Environment.NewLine);
            }
            else
            {
                string local = converter.CurrentFunction.LocalVariables.Find(new Predicate<string>(delegate(string name) { return name == this.realName; }));
                if (local != null)
                {
                    converter.CurrentFunction.AddToSource("WriteToFile $" + this.defaultWriter + " $" + this.realName + "" + Environment.NewLine);
                }
                else
                {
                    converter.CurrentFunction.AddToSource("WriteToFile getValue($" + this.realName + ")" + Environment.NewLine);
                }
            }
        }

        #endregion
    }
}
