using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Converters;
using System.Drawing;

namespace o2Mate
{
    [Guid("06708EC5-7257-4f84-A20D-C130090DE43B")]
    [CoClass(typeof(IExpression))]
    [InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
    public interface IExpression
    {
        [DispId(1)]
        IData Evaluate(string expr, Object scope);
    }

    internal class Compute : ILanguageConverter
    {
        #region Constants
        private readonly string[] errors = new string[] { null, "Erreur parenthses", "Expression {0:G} incorrecte", "Terme inconnu" };
        private readonly char[] opers = new char[] { ',', '<', '>', '=', '+', '-', '*', '/', '.', '?', ':' };
        #endregion

        #region Private Fields
        int index;
        IScope scope;
        ICodeConverter converter;
        List<uint> stock = new List<uint>();
        List<string> words = new List<string>();
        Stack<string> stack = new Stack<string>();
        List<Variable> vars = new List<Variable>();
        Form canvas;
        #endregion

        #region Private Methods

        private void Enqueue(string s)
        {
            this.stack.Push(s);
        }

        private string Dequeue()
        {
            return this.stack.Pop();
        }

        private bool function(string expr, ref int i)
        {
            bool result = false;
            if (expr.EndsWith("$"))
            {
                this.words.Add(expr.TrimEnd('$'));
                this.stock.Add('');
                this.stock.Add((uint)(this.words.Count - 1));
                result = true;
            }
            i = expr.Length;
            return result;
        }

        private bool variable(string expr, ref int i)
        {
            string trimedExpr = expr.Trim();
            if (String.IsNullOrEmpty(trimedExpr))
                trimedExpr = " ";
            IData data = this.scope.GetVariable(trimedExpr);
            if (data != null)
            {
                i = trimedExpr.Length;
                int c = 0;
                try
                {
                    this.words.Add(trimedExpr);
                    this.stock.Add('');
                    this.stock.Add((uint)(this.words.Count - 1));
                }
                catch
                {
                    // ce n'est pas une constante
                    noQuoteString(data.ValueString, ref c);
                }
                return true;
            }
            else
            {
                // 1- search this char @ for variable
                // 2- return the string
                int c = 0;
                noQuoteString(trimedExpr, ref c);
                return true;
            }
        }

        private void noQuoteString(string expr, ref int i)
        {
            i = expr.Length;
            this.words.Add(expr);
            this.stock.Add('!');
            this.stock.Add((uint)(this.words.Count - 1));
        }

        private void constante(string expr, ref int i)
        {
            int right;
            right = 0;
            string trimedExpr = expr.TrimStart();
            while (right < trimedExpr.Length && trimedExpr[right] >= '0' && trimedExpr[right] <= '9') ++right;
            i = right;
            string subexpr = trimedExpr.Substring(0, right);
            this.stock.Add('@');
            this.stock.Add(System.Convert.ToUInt32(subexpr));
        }

        private void monome(string expr)
        {
            int i, l;
            string trimedExpr = expr.TrimStart();
            l = trimedExpr.Length;
            if (l == 0)
            {
                System.Diagnostics.Debugger.Break();
            }
            switch (trimedExpr[0])
            {
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                case '8':
                case '9':
                    i = 0;
                    this.constante(trimedExpr, ref i);
                    if (i == l) return;
                    break;
                case '$':
                    {
                        trimedExpr = trimedExpr.TrimEnd();
                        l = trimedExpr.Length;
                        if (l == 1)
                        {
                            string subexpr = this.Dequeue();
                            this.parentheses(subexpr);
                        }
                        break;
                    }
                default:
                    i = 0;
                    if (this.function(trimedExpr, ref i))
                    {
                        string subexpr = this.Dequeue();
                        if (!String.IsNullOrEmpty(subexpr))
                            this.parentheses(subexpr);
                    }
                    else
                    {
                        i = 0;
                        if (this.variable(trimedExpr, ref i))
                        {
                            if (i == l) return;
                        }
                    }
                    break;
            }
        }

        private void operateurs(string expr, int oper)
        {
            int i, j;
            string subexpr;

            j = i = expr.Length;
            if (i == 0) throw new Exception(String.Format(this.errors[2], expr));
            if (expr[--i] != this.opers[oper])
            {
                while (--i >= 0)
                {
                    if (expr[i] == this.opers[oper]) break;
                }
            }
            if (i == -1)
            {
                if (oper + 1 < this.opers.Length)
                    this.operateurs(expr, oper + 1);
                else
                    this.monome(expr);
                return;
            }
            if (i == 0)
            {
                if (this.opers[oper] != '+' && this.opers[oper] != '-')
                    throw new Exception(String.Format(this.errors[2], expr));
                this.stock.Add(this.opers[oper]);
                monome("0");
                subexpr = expr.Substring(i + 1, j - i - 1);
                operateurs(subexpr, oper);
                return;
            }

            this.stock.Add(this.opers[oper]);
            subexpr = expr.Substring(0, i);
            operateurs(subexpr, oper);
            subexpr = expr.Substring(i + 1, j - i - 1);
            operateurs(subexpr, oper);

        }

        private void parentheses(string expr)
        {
            int i;
            int leftpar = -1, rightpar = -1;
            int profpar = 0;
            string subexpr;

            i = expr.Length;
            while (--i >= 0)
            {
                if (expr[i] == ')')
                    if (rightpar == -1) rightpar = i; else ++profpar;
                else if (expr[i] == '(')
                    if (profpar == 0) leftpar = i; else --profpar;

            }
            if (leftpar < 0 && rightpar < 0)
            {
                // il n'y a pas de parentheses
                this.operateurs(expr, 0);
                return;
            }
            if (leftpar < 0 || rightpar < leftpar)
            {
                throw new Exception(this.errors[1]);
            }

            subexpr = expr.Substring(leftpar + 1, rightpar - leftpar - 1);
            this.Enqueue(subexpr);
            string subexpr2 = expr.Substring(0, leftpar) + "$" + expr.Substring(rightpar + 1);
            this.Enqueue(subexpr2);
            parentheses(this.Dequeue());
        }

        private Variable calculate()
        {
            Variable a, b;
            Variable res = new Variable("");
            switch (this.stock[this.index])
            {
                case '>':
                    ++this.index;
                    a = this.calculate();
                    b = this.calculate();
                    res = new Variable("");
                    res.Value = ((a.ValueInt > b.ValueInt) ? 1 : 0).ToString();
                    break;
                case '<':
                    ++this.index;
                    a = this.calculate();
                    b = this.calculate();
                    res = new Variable("");
                    res.Value = ((a.ValueInt < b.ValueInt) ? 1 : 0).ToString();
                    break;
                case '=':
                    ++this.index;
                    a = this.calculate();
                    b = this.calculate();
                    res = new Variable("");
                    res.Value = ((a.ValueInt == b.ValueInt) ? 1 : 0).ToString();
                    break;
                case '+':
                    ++this.index;
                    a = this.calculate();
                    b = this.calculate();
                    res = new Variable("");
                    res.Value = (a.ValueInt + b.ValueInt).ToString();
                    break;
                case '-':
                    ++this.index;
                    a = this.calculate();
                    b = this.calculate();
                    res = new Variable("");
                    res.Value = (a.ValueInt - b.ValueInt).ToString();
                    break;
                case '*':
                    ++this.index;
                    a = this.calculate();
                    b = this.calculate();
                    res = new Variable("");
                    res.Value = (a.ValueInt * b.ValueInt).ToString();
                    break;
                case '/':
                    ++this.index;
                    a = this.calculate();
                    b = this.calculate();
                    res = new Variable("");
                    if (b.ValueInt != 0)
                        res.Value = (a.ValueInt / b.ValueInt).ToString();
                    else
                        res.Value = "0";
                    break;
                case '?':
                    ++this.index;
                    a = this.calculate();
                    b = this.calculate();
                    res = new Variable("");
                    res.Value = ((a.ValueString == b.ValueString) ? 1 : 0).ToString();
                    break;
                case '.':
                    ++this.index;
                    a = this.calculate();
                    b = this.calculate();
                    res = new Variable("");
                    res.Value = a.ValueString + b.ValueString;
                    break;
                case ':':
                    ++this.index;
                    a = this.calculate();
                    b = this.calculate();
                    res = new Variable("");
                    if (b.ValueInt > 0 && b.ValueInt <= a.ValueString.Length)
                    {
                        res.Value = a.ValueString[b.ValueInt - 1].ToString();
                    }
                    else
                    {
                        res.Value = "";
                    }
                    break;
                case ',':
                    ++this.index;
                    while (this.stock[this.index] == ',')
                    {
                        ++this.index;
                        this.vars.Add(this.calculate());
                    }
                    this.vars.Add(this.calculate());
                    res = this.calculate();
                    break;
                case '@':
                    ++this.index;
                    res = new Variable("");
                    res.Value = ((int)this.stock[this.index]).ToString();
                    ++this.index;
                    break;
                case '!':
                    ++this.index;
                    res = new Variable("");
                    res.Value = this.words[(int)this.stock[this.index]];
                    ++this.index;
                    break;
                case '':
                    ++this.index;
                    string varName = this.words[(int)this.stock[this.index]];
                    res = this.scope.GetVariable(varName) as Variable;
                    ++this.index;
                    break;
                case '':
                    ++this.index;
                    string functionName = this.words[(int)this.stock[this.index]];
                    res = new Variable("");
                    ++this.index;
                    switch (functionName)
                    {
                        case "startCanvas":
                            {
                                this.canvas = new Form();
                                Panel p = new Panel();
                                p.Dock = DockStyle.Fill;
                                p.BackColor = Color.White;
                                p.ForeColor = Color.Black;
                                this.vars.Clear();
                                a = this.calculate();
                                this.canvas.Width = this.vars[0].ValueInt;
                                this.canvas.Height = this.vars[1].ValueInt;
                                this.canvas.Controls.Add(p);
                                this.canvas.Show();
                                break;
                            }
                        case "ellipse":
                            {
                                Graphics g = this.canvas.Controls[0].CreateGraphics();
                                g.DrawEllipse(Pens.Black, new Rectangle(200,200,100,100));
                                this.vars.Clear();
                                a = this.calculate();
                                break;
                            }
                        case "left":
                            {
                                res = this.calculate();
                                break;
                            }
                        case "top":
                            {
                                res = this.calculate();
                                break;
                            }
                        case "color":
                            {
                                res = this.calculate();
                                break;
                            }
                        case "array":
                            {
                                this.vars.Clear();
                                a = this.calculate();
                                string list = String.Empty;
                                foreach (Variable v in this.vars)
                                {
                                    list += v.ValueString + ",";
                                }
                                list += a.ValueString;
                                res = new Variable("");
                                res.Value = list;
                                break;
                            }
                        case "upper":
                            {
                                a = this.calculate();
                                res = new Variable("");
                                res.Value = a.ValueString.ToUpper();
                                break;
                            }
                        default:
                            break;
                    }
                    break;
            }
            return res;
        }

        private string ConvertExpressionToVBScript()
        {
            string a = null, b = null;
            string result = "";
            switch (this.stock[this.index])
            {
                case '>':
                    ++this.index;
                    a = ConvertExpressionToVBScript();
                    b = ConvertExpressionToVBScript();
                    result = "CLng(" + a + ") > CLng(" + b + ")";
                    break;
                case '<':
                    ++this.index;
                    a = ConvertExpressionToVBScript();
                    b = ConvertExpressionToVBScript();
                    result = "CLng(" + a + ") < CLng(" + b + ")";
                    break;
                case '=':
                    ++this.index;
                    a = ConvertExpressionToVBScript();
                    b = ConvertExpressionToVBScript();
                    result = "CLng(" + a + ") = CLng(" + b + ")";
                    break;
                case '+':
                    ++this.index;
                    a = ConvertExpressionToVBScript();
                    b = ConvertExpressionToVBScript();
                    result = "CLng(" + a + ") + CLng(" + b + ")";
                    break;
                case '-':
                    ++this.index;
                    a = ConvertExpressionToVBScript();
                    b = ConvertExpressionToVBScript();
                    result = "CLng(" + a + ") - CLng(" + b + ")";
                    break;
                case '*':
                    ++this.index;
                    a = ConvertExpressionToVBScript();
                    b = ConvertExpressionToVBScript();
                    result = "CLng(" + a + ") * CLng(" + b + ")";
                    break;
                case '/':
                    ++this.index;
                    a = ConvertExpressionToVBScript();
                    b = ConvertExpressionToVBScript();
                    result = "CLng(" + a + ") / CLng(" + b + ")";
                    break;
                case '?':
                    ++this.index;
                    a = ConvertExpressionToVBScript();
                    b = ConvertExpressionToVBScript();
                    result = "CStr(" + a + ") = CStr(" + b + ")";
                    break;
                case '.':
                    ++this.index;
                    a = ConvertExpressionToVBScript();
                    b = ConvertExpressionToVBScript();
                    result = "CStr(" + a + ") & CStr(" + b + ")";
                    break;
                case ':':
                    ++this.index;
                    a = ConvertExpressionToVBScript();
                    b = ConvertExpressionToVBScript();
                    result = "Mid(CStr(" + a + "),CLng(" + b + "), 1)";
                    break;
                case '@':
                    ++this.index;
                    result = this.stock[this.index].ToString();
                    ++this.index;
                    break;
                case '!':
                    ++this.index;
                    result = "\"" + this.words[(int)this.stock[this.index]] + "\"";
                    ++this.index;
                    break;
                case '':
                    ++this.index;
                    string varName = this.words[(int)this.stock[this.index]];
                    string realName = this.scope.GetVariable(varName).RealName;
                    this.converter.CurrentFunction.UsedVariables.Add(realName);
                    result = realName;
                    ++this.index;
                    break;
                case ',':
                    {
                        string str = String.Empty;
                        ++this.index;
                        while (this.stock[this.index] == ',')
                        {
                            ++this.index;
                            str += ConvertExpressionToVBScript() + "& \",\" & ";
                        }
                        str += ConvertExpressionToVBScript() + "& \",\" & ";
                        result = str + ConvertExpressionToVBScript();
                        break;
                    }
                case '':
                    ++this.index;
                    string functionName = this.words[(int)this.stock[this.index]];
                    ++this.index;
                    switch (functionName)
                    {
                        case "array":
                            {
                                result = ConvertExpressionToVBScript();
                                break;
                            }
                        case "upper":
                            {
                                result = "UCase(" + ConvertExpressionToVBScript() + ")";
                                break;
                            }
                        default:
                            break;
                    }
                    break;
            }
            return result;
        }

        private string ConvertExpressionToPowerShell()
        {
            string a = null, b = null;
            string result = "";
            switch (this.stock[this.index])
            {
                case '>':
                    ++this.index;
                    a = ConvertExpressionToPowerShell();
                    b = ConvertExpressionToPowerShell();
                    result = "(" + a + " -gt " + b + ")";
                    break;
                case '<':
                    ++this.index;
                    a = ConvertExpressionToPowerShell();
                    b = ConvertExpressionToPowerShell();
                    result = "(" + a + " -lt " + b + ")";
                    break;
                case '=':
                    ++this.index;
                    a = ConvertExpressionToPowerShell();
                    b = ConvertExpressionToPowerShell();
                    result = "(" + a + " -eq " + b + ")";
                    break;
                case '+':
                    ++this.index;
                    a = ConvertExpressionToPowerShell();
                    b = ConvertExpressionToPowerShell();
                    result = "(" + a + " + " + b + ")";
                    break;
                case '-':
                    ++this.index;
                    a = ConvertExpressionToPowerShell();
                    b = ConvertExpressionToPowerShell();
                    result = "(" + a + " - " + b + ")";
                    break;
                case '*':
                    ++this.index;
                    a = ConvertExpressionToPowerShell();
                    b = ConvertExpressionToPowerShell();
                    result = "(" + a + " * " + b + ")";
                    break;
                case '/':
                    ++this.index;
                    a = ConvertExpressionToPowerShell();
                    b = ConvertExpressionToPowerShell();
                    result = "(" + a + " / " + b + ")";
                    break;
                case '?':
                    ++this.index;
                    a = ConvertExpressionToPowerShell();
                    b = ConvertExpressionToPowerShell();
                    result = "(" + a + " -eq " + b + ")";
                    break;
                case '.':
                    ++this.index;
                    a = ConvertExpressionToPowerShell();
                    b = ConvertExpressionToPowerShell();
                    result = "(" + a + " + " + b + ")";
                    break;
                case ':':
                    ++this.index;
                    a = ConvertExpressionToPowerShell();
                    b = ConvertExpressionToPowerShell();
                    result = "((" + a + ")[(" + b + ")-1])";
                    break;
                case '@':
                    ++this.index;
                    result = this.stock[this.index].ToString();
                    ++this.index;
                    break;
                case '!':
                    ++this.index;
                    result = "\"" + this.words[(int)this.stock[this.index]].Replace("\"", "\"\"") + "\"";
                    ++this.index;
                    break;
                case '':
                    ++this.index;
                    string varName = this.words[(int)this.stock[this.index]];
                    string realName = this.scope.GetVariable(varName).RealName;
                    this.converter.CurrentFunction.UsedVariables.Add(realName);
                    string local = this.converter.CurrentFunction.LocalVariables.Find(new Predicate<string>(delegate(string name) { return name == realName; }));
                    if (local != null)
                    {
                        result = "$" + realName;
                    }
                    else
                    {
                        result = "$(getValue($" + realName + "))";
                    }
                    ++this.index;
                    break;
                case ',':
                    {
                        string str = String.Empty;
                        ++this.index;
                        while (this.stock[this.index] == ',')
                        {
                            ++this.index;
                            str += ConvertExpressionToPowerShell() + "+ \",\" + ";
                        }
                        str += ConvertExpressionToPowerShell() + "+ \",\" + ";
                        result = str + ConvertExpressionToPowerShell();
                        break;
                    }
                case '':
                    ++this.index;
                    string functionName = this.words[(int)this.stock[this.index]];
                    ++this.index;
                    switch (functionName)
                    {
                        case "array":
                            {
                                result = ConvertExpressionToPowerShell();
                                break;
                            }
                        case "upper":
                            {
                                result = "(" + ConvertExpressionToPowerShell() + ").ToUpper()";
                                break;
                            }
                        default:
                            break;
                    }
                    break;
            }
            return result;
        }

        #endregion

        #region Public Methods
        public string run(string expr, IScope scope)
        {
            this.scope = scope;
            this.parentheses(expr);
            this.index = 0;
            return calculate().ValueString;
        }

        public void Convert(ICodeConverter converter, string expr, IScope scope)
        {
            this.scope = scope;
            this.parentheses(expr);
            this.index = 0;
            // on efface les utilisations prcdentes
            converter.CurrentFunction.UsedVariables.Clear();
            converter.Convert(this);
        }
        #endregion

        #region ILanguageConverter Membres

        public void WriteInVBScript(ICodeConverter converter)
        {
            this.converter = converter;
            converter.CurrentFunction.CacheSource = this.ConvertExpressionToVBScript();
        }

        public void WriteInPowerShell(ICodeConverter converter)
        {
            this.converter = converter;
            converter.CurrentFunction.CacheSource = this.ConvertExpressionToPowerShell();
        }

        #endregion
    }

    [ClassInterface(ClassInterfaceType.None)]
    public class Expression : IExpression
    {
        #region Public Methods
        public void Convert(ICodeConverter converter, string expr, Object obj)
        {
            Compute c = new Compute();
            IScope scope = obj as IScope;
            if (!String.IsNullOrEmpty(expr))
            {
                c.Convert(converter, expr, scope);
            }
            else
            {
                converter.CurrentFunction.CacheSource = "\"\"";
            }
        }
        #endregion

        #region IExpression Membres

        public IData Evaluate(string expr, Object obj)
        {
            Variable var = new Variable("");
            var.Name = "Result";
            Compute c = new Compute();
            IScope scope = obj as IScope;
            if (!String.IsNullOrEmpty(expr))
                var.Value = c.run(expr, scope);
            else
                var.Value = "";
            return var;
        }

        #endregion
    }
}
