using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Xml;

namespace o2Mate
{
    public struct primaryKey
    {
        public primaryKey(string field)
        {
            this.field = field;
            this.valid = true;
        }
        public bool valid;
        public string field;
    }

    public struct inconnection
    {
        public inconnection(string context, string name, string field)
        {
            this.context = context;
            this.name = name;
            this.field = field;
            this.valid = true;
        }
        public bool valid;
        public string context;
        public string name;
        public string field;
    }

    public struct depends
    {
        public depends(string context, string name, string value)
        {
            this.context = context;
            this.name = name;
            this.value = value;
            this.valid = true;
        }
        public bool valid;
        public string context;
        public string name;
        public string value;
    }

    [Guid("4B20CE6C-19BD-4d5e-A07B-A72C6F8319A6")]
    [InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
    public interface ILegende
    {
        string Context { get; set; }
        string Name { get; set; }
        string Description { get; set; }
        string Commentaire { get; set; }
        string Type { get; set; }
        string Expression { get; set; }
        bool Free { get; set; }
        string Observe { get; set; }
        inconnection InConnection { get; set; }
        depends DependsOn { get; set; }
        primaryKey PrimaryKey { get; set; }

        bool IsArray { get; }
        bool IsNumber { get; }
        bool IsDate { get; }
        bool IsString { get; }
        bool IsEnumeration { get; }
        bool IsTemplate { get; }
        bool IsUnknown { get; }

        void Load(XmlNode node);
        void Save(XmlWriter writer);
        string Translate(string value);
    }

    [ClassInterface(ClassInterfaceType.None)]
    public class Legende : ILegende
    {
        #region Public Static Constants
        public static readonly string TypeArray = "Array";
        public static readonly string TypeNumber = "Number";
        public static readonly string TypeDate = "Date";
        public static readonly string TypeString = "String";
        public static readonly string TypeEnumeration = "Enumeration";
        public static readonly string TypeUnknown = "Unknown";
        public static readonly string TypeTemplate = "Template";
        #endregion

        #region Private Fields
        private string context;
        private string name;
        private string description;
        private string commentaire;
        private string type;
        private string expression;
        private string observe;
        private inconnection relatedTo;
        private depends dependsOn;
        private primaryKey primaryKey;
        private bool free;
        #endregion

        #region Public Constructor
        public Legende()
        {
            this.relatedTo = new inconnection();
            this.dependsOn = new depends();
            this.primaryKey = new primaryKey();
            this.free = true;
        }
        #endregion

        #region ILegende Membres

        public string Context
        {
            get
            {
                return this.context;
            }
            set
            {
                this.context = value;
            }
        }

        public string Name
        {
            get
            {
                return this.name;
            }
            set
            {
                this.name = value;
            }
        }

        public string Commentaire
        {
            get { return this.commentaire.Replace("\\", "\\\\").Replace(Environment.NewLine, "\\r\\n"); }
            set
            {
                o2Mate.LocaleGroup group = new LocaleGroup();
                string groupName, localeName;
                if (group.ExtractGroupAndName(value, out groupName, out localeName))
                {
                    if (!group.Exists(groupName))
                    {
                        group.Create(groupName);
                    }
                    o2Mate.ILocaleSet set = group.Get(groupName) as ILocaleSet;
                    if (!set.ExistsOne(localeName, "fr-FR"))
                        set.Add(localeName, "fr-FR", "");
                    if (!set.ExistsOne(localeName, "en-US"))
                        set.Add(localeName, "en-US", "");
                }
                this.commentaire = value;
            }
        }

        public string Description
        {
            get
            {
                return this.description.Replace("\\", "\\\\").Replace(Environment.NewLine, "\\r\\n");
            }
            set
            {
                o2Mate.LocaleGroup group = new LocaleGroup();
                string groupName, localeName;
                if (group.ExtractGroupAndName(value, out groupName, out localeName))
                {
                    if (!group.Exists(groupName))
                    {
                        group.Create(groupName);
                    }
                    o2Mate.ILocaleSet set = group.Get(groupName) as ILocaleSet;
                    if (!set.ExistsOne(localeName, "fr-FR"))
                        set.Add(localeName, "fr-FR", "");
                    if (!set.ExistsOne(localeName, "en-US"))
                        set.Add(localeName, "en-US", "");
                }
                this.description = value;
            }
        }

        public string Type
        {
            get
            {
                return this.type;
            }
            set
            {
                this.type = value;
            }
        }

        public string Expression
        {
            get
            {
                return this.expression;
            }
            set
            {
                this.expression = value;
            }
        }

        public bool Free
        {
            get
            {
                return this.free;
            }
            set
            {
                this.free = value;
            }
        }

        public string Observe
        {
            get
            {
                return this.observe;
            }
            set
            {
                this.observe = value;
            }
        }

        public inconnection InConnection
        {
            get
            {
                return this.relatedTo;
            }
            set
            {
                this.relatedTo = value;
            }
        }

        public depends DependsOn
        {
            get
            {
                return this.dependsOn;
            }
            set
            {
                this.dependsOn = value;
            }
        }

        public primaryKey PrimaryKey
        {
            get
            {
                return this.primaryKey;
            }
            set
            {
                this.primaryKey = value;
            }
        }

        public bool IsArray
        {
            get
            {
                return String.Compare(this.type, Legende.TypeArray, true) == 0;
            }
        }

        public bool IsNumber
        {
            get { return String.Compare(this.type, Legende.TypeNumber, true) == 0; }
        }

        public bool IsDate
        {
            get { return String.Compare(this.type, Legende.TypeDate, true) == 0; }
        }

        public bool IsString
        {
            get { return String.Compare(this.type, Legende.TypeString, true) == 0; }
        }

        public bool IsEnumeration
        {
            get { return String.Compare(this.type, Legende.TypeEnumeration, true) == 0; }
        }

        public bool IsTemplate
        {
            get { return String.Compare(this.type, Legende.TypeTemplate, true) == 0; }
        }

        public bool IsUnknown
        {
            get { return (String.Compare(this.type, Legende.TypeArray, true) != 0 &&
                          String.Compare(this.type, Legende.TypeNumber, true) != 0 &&
                          String.Compare(this.type, Legende.TypeString, true) != 0 &&
                          String.Compare(this.type, Legende.TypeDate, true) != 0 &&
                          String.Compare(this.type, Legende.TypeEnumeration, true) != 0 &&
                          String.Compare(this.type, Legende.TypeTemplate, true) != 0);
        }
        }

        public void Load(XmlNode node)
        {
            this.context = node.Attributes.GetNamedItem("context").Value;
            this.name = node.Attributes.GetNamedItem("name").Value;
            this.description = node.InnerText;
            this.type = node.Attributes.GetNamedItem("type").Value;
            this.expression = node.Attributes.GetNamedItem("expression").Value;
            this.commentaire = node.Attributes.GetNamedItem("commentaire").Value;
            if (node.Attributes.GetNamedItem("free") != null)
            {
                this.free = Boolean.Parse(node.Attributes.GetNamedItem("free").Value);
            }
            else
            {
                this.free = true;
            }
        }

        public void Save(XmlWriter writer)
        {
            writer.WriteStartElement("item");
            writer.WriteAttributeString("context", this.context);
            writer.WriteAttributeString("name", this.name);
            writer.WriteAttributeString("type", this.type);
            writer.WriteAttributeString("expression", this.expression);
            writer.WriteAttributeString("commentaire", this.commentaire);
            if (!this.free)
            {
                writer.WriteAttributeString("free", this.free.ToString());
            }
            writer.WriteString(this.description);
            writer.WriteEndElement();
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            this.context = elem.GetAttribute("context");
            this.name = elem.GetAttribute("name");
            this.description = elem.GetAttribute("description");
            this.type = elem.GetAttribute("typeLegende");
            this.expression = elem.GetAttribute("expression");
            this.commentaire = elem.GetAttribute("commentaire");
            this.free = Boolean.Parse(elem.GetAttribute("free"));
            o2Mate.LocaleGroup group = new LocaleGroup();
            string groupName, localeName;
            if (group.ExtractGroupAndName(this.description, out groupName, out localeName))
            {
                if (!group.Exists(groupName))
                {
                    group.Create(groupName);
                }
                o2Mate.ILocaleSet set = group.Get(groupName) as ILocaleSet;
                if (!set.ExistsOne(localeName, "fr-FR"))
                    set.Add(localeName, "fr-FR", "");
                if (!set.ExistsOne(localeName, "en-US"))
                    set.Add(localeName, "en-US", "");
            }
            if (group.ExtractGroupAndName(this.commentaire, out groupName, out localeName))
            {
                if (!group.Exists(groupName))
                {
                    group.Create(groupName);
                }
                o2Mate.ILocaleSet set = group.Get(groupName) as ILocaleSet;
                if (!set.ExistsOne(localeName, "fr-FR"))
                    set.Add(localeName, "fr-FR", "");
                if (!set.ExistsOne(localeName, "en-US"))
                    set.Add(localeName, "en-US", "");
            }
        }

        public string Translate(string value)
        {
            o2Mate.LocaleGroup group = new LocaleGroup();
            string groupName, localeName;
            if (group.ExtractGroupAndName(value, out groupName, out localeName))
            {
                if (group.Exists(groupName))
                {
                    o2Mate.ILocaleSet set = group.Get(groupName) as o2Mate.ILocaleSet;
                    if (set.ExistsOne(localeName, System.Threading.Thread.CurrentThread.CurrentUICulture.Name))
                    {
                        return set.Get(localeName, System.Threading.Thread.CurrentThread.CurrentUICulture.Name);
                    }
                    else
                    {
                        return value;
                    }
                }
                else
                {
                    return value;
                }
            }
            else
            {
                return value;
            }
        }

        #endregion
    }
}
