using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Xml;

namespace o2Mate
{
    public interface ILegendeDict
    {
        void Load(XmlNode dict);
        void Save(XmlWriter writer);
        void Update(System.Windows.Forms.HtmlElement elem);
        void Clear();
        void Add(string context, string name, string desc, string type, string expr);
        void CopyFrom(LegendeDict dict);
        int Count { get; }
        ILegende GetLegendeByIndex(int index);
        ILegende GetLegendeByName(string name);
        ILegende GetLegendeByName(string name, string context);
    }

    public class LegendeDict : ILegendeDict, IEnumerable<ILegende>
    {
        #region Private Fields
        private List<ILegende> legendes;
        #endregion

        #region Default Constructor
        public LegendeDict()
        {
            this.legendes = new List<ILegende>();
        }
        #endregion

        #region ILegendeDict Membres

        public void Clear()
        {
            this.legendes.Clear();
        }

        public void Load(XmlNode dict)
        {
            foreach (XmlNode child in dict)
            {
                Legende leg = new Legende();
                leg.Load(child);
                this.legendes.Add(leg);
            }
        }

        public void Save(XmlWriter writer)
        {
            foreach (ILegende leg in this.legendes)
            {
                leg.Save(writer);
            }
        }

        public void Update(System.Windows.Forms.HtmlElement elem)
        {
            // un update des legendes => on efface les anciennes legendes
            this.Clear();
            foreach (System.Windows.Forms.HtmlElement child in elem.Children)
            {
                Legende leg = new Legende();
                leg.Update(child);
                this.legendes.Add(leg);
            }
        }

        public int Count
        {
            get { return this.legendes.Count; }
        }

        public ILegende GetLegendeByIndex(int index)
        {
            return this.legendes[index];
        }

        public ILegende GetLegendeByName(string name)
        {
            if (!String.IsNullOrEmpty(name))
            {
                ILegende foundLegende = this.legendes.Find(new Predicate<ILegende>(delegate(ILegende a) { return a.Name == name; }));
                if (foundLegende != null)
                {
                    return foundLegende;
                }
                else
                {
                    return null;
                }
            }
            else
            {
                return null;
            }
        }

        public ILegende GetLegendeByName(string name, string context)
        {
            if (!String.IsNullOrEmpty(name) && !String.IsNullOrEmpty(context))
            {
                ILegende foundLegende = this.legendes.Find(new Predicate<ILegende>(delegate(ILegende a) { return a.Context == context && a.Name == name; }));
                if (foundLegende != null)
                {
                    return foundLegende;
                }
                else
                {
                    return null;
                }
            }
            else
            {
                return null;
            }
        }

        public void Add(string context, string name, string desc, string type, string expr)
        {
            if (!String.IsNullOrEmpty(name) && context != null)
            {
                ILegende foundLegende = this.legendes.Find(new Predicate<ILegende>(delegate(ILegende a) { return a.Context == context && a.Name == name; }));
                if (foundLegende != null)
                {
                    foundLegende.Description = desc;
                    foundLegende.Type = type;
                    foundLegende.Expression = expr;
                }
                else
                {
                    ILegende legende = new Legende();
                    legende.Context = context;
                    legende.Name = name;
                    legende.Description = desc;
                    legende.Type = type;
                    legende.Expression = expr;
                    this.legendes.Add(legende);
                }
            }
        }

        public void CopyFrom(LegendeDict dict)
        {
            this.legendes.AddRange(dict);
        }

        #endregion

        #region IEnumerable<ILegende> Membres

        public IEnumerator<ILegende> GetEnumerator()
        {
            return this.legendes.GetEnumerator();
        }

        #endregion

        #region IEnumerable Membres

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return (System.Collections.IEnumerator)this.legendes.GetEnumerator();
        }

        #endregion
    }
}
