using System;
using System.Collections.Generic;
using System.Text;
using System.Globalization;
using System.Text.RegularExpressions;
using System.IO;
using System.Runtime.InteropServices;

namespace o2Mate
{
    [Guid("2C5E0F1E-D2DA-47ce-98A1-4045073E3CC7")]
    [InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
    [ComVisible(true)]
    public interface ILocale
    {
        string Name { get; }
        string Language { get; }
        string Value { get; }
    }

    [ClassInterface(ClassInterfaceType.AutoDispatch)]
    public class LocaleElement : ILocale
    {
        #region Private Fields
        private string name;
        private string language;
        private string value;
        #endregion

        #region Private Static Methods
        private static string IdentifyWord(string s, ref int position)
        {
            string output = "";
            for (int index = position; index < s.Length; ++index)
            {
                char c = s[index];
                if (c == '\"')
                {
                    position = index;
                    break;
                }
                else
                {
                    output += c;
                }
            }
            return output;
        }

        #endregion

        #region Public Static Methods
        public static bool ValidateName(string name)
        {
            if (name.IndexOf('\"') != -1)
            {
                throw new ArgumentException("Le nom d'un objet Locale ne peut pas avoir de caractre \"");
            }
            else if (name.IndexOf(' ') != -1 || name.IndexOf('\t') != -1 || name.IndexOf(Environment.NewLine) != -1)
            {
                throw new ArgumentException("Le nom d'un objet Locale ne peut avoir d'espaces, de tabulation ou de retour chariot");
            }
            return true;
        }

        public static bool ValidateLanguage(string name)
        {
            if (name.IndexOf('\"') != -1)
            {
                throw new ArgumentException("Le langage d'un objet Locale ne peut avoir de caractre \"");
            }
            return true;
        }

        public static string FormatValue(string value)
        {
            string output = "";
            if (!String.IsNullOrEmpty(value))
            {
                string ret = "";
                ret += ((char)13);
                ret += ((char)10);
                output = value.Replace(ret, "\\r\\n");
            }
            return output;
        }

        public static LocaleElement Parse(string line)
        {
            int position = 1;
            string name = LocaleElement.IdentifyWord(line, ref position);
            position += 3;
            string language = LocaleElement.IdentifyWord(line, ref position);
            position += 2;
            string value = line.Substring(position);
            return new LocaleElement(name, language, value);
        }
        #endregion

        #region Public Constructor
        public LocaleElement(string name, string language, string value)
        {
            if (LocaleElement.ValidateName(name) && LocaleElement.ValidateLanguage(language))
            {
                this.Name = name;
                this.Language = language;
                this.Value = value;
            }
        }
        #endregion

        #region Public Properties
        public string Name
        {
            get { return this.name; }
            set { if (LocaleElement.ValidateName(value)) { this.name = value; } }
        }

        public string Language
        {
            get { return this.language; }
            set { if (LocaleElement.ValidateLanguage(value)) { this.language = value; } }
        }

        public string Value
        {
            get
            {
                string ret = "";
                ret += ((char)13);
                ret += ((char)10);
                return this.value.Replace("\\r\\n", ret);
            }
            set { this.value = LocaleElement.FormatValue(value); }
        }
        #endregion

        #region Public Methods
        public void WriteToFile(StreamWriter sw)
        {
            sw.WriteLine("\"" + this.Name + "\"" + "\t\"" + this.Language + "\"\t" + this.value);
        }
        #endregion
    }
}
