using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace o2Mate
{
    [Guid("87D2A7F1-D48E-4dfd-B3FA-58B4304E41A7")]
    [InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
    [CoClass(typeof(ILocaleSet))]
    [ComVisible(true)]
    public interface ILocaleSet
    {
        string GroupName { get; set; }
        bool ExistsOne(string name, string language);
        bool Exists(string name);
        void Add(string name, string language, string value);
        void Modify(string name, string language, string value);
        void DeleteOne(string name, string language);
        void Delete(string name);
        void RenameOne(string oldName, string name, string language);
        void Rename(string oldName, string name);
        string Get(string name, string language);
        ITab Names { get; }
        ITab GetValues(string name);
    }

    [ClassInterface(ClassInterfaceType.None)]
    public class LocaleSet : ILocaleSet
    {
        #region Private Fields
        private string groupName;
        #endregion

        #region Default Constructor
        public LocaleSet(string groupName)
        {
            this.groupName = groupName;
        }
        #endregion

        #region ILocaleSet Membres

        public string GroupName
        {
            get
            {
                return this.groupName;
            }
            set
            {
                LocaleSystem.RenameGroup(this.groupName, value);
                this.groupName = value;
            }
        }

        public bool ExistsOne(string name, string language)
        {
            return LocaleSystem.ExistLocale(this.groupName, name, language);
        }

        public bool Exists(string name)
        {
            return LocaleSystem.ExistLocale(this.groupName, name);
        }

        public void Add(string name, string language, string value)
        {
            LocaleSystem.InsertLocale(this.groupName, name, language, value);
        }

        public void Modify(string name, string language, string value)
        {
            LocaleSystem.ModifyLocale(this.groupName, name, language, value);
        }

        public void DeleteOne(string name, string language)
        {
            LocaleSystem.RemoveLocale(this.groupName, name, language);
        }

        public void Delete(string name)
        {
            LocaleSystem.RemoveLocale(this.groupName, name);
        }

        public void RenameOne(string oldName, string name, string language)
        {
            LocaleSystem.RenameLocale(oldName, this.groupName, name, language);
        }

        public void Rename(string oldName, string name)
        {
            LocaleSystem.RenameLocale(oldName, this.groupName, name);
        }

        public string Get(string name, string language)
        {
            LocaleElement elem = LocaleSystem.GetLocale(this.groupName, name, language);
            if (elem != null)
            {
                return elem.Value;
            }
            else
            {
                throw new Exception("L'objet locale '" + name + "' pour le language '" + language + "' n'existe pas");
            }
        }

        public ITab Names
        {
            get
            {
                List<LocaleElement> list = LocaleSystem.GetNames(this.groupName);
                return Tab.FromArray(list.ToArray());
            }
        }

        public ITab GetValues(string name)
        {
            List<LocaleElement> list = LocaleSystem.GetLocale(this.groupName, name);
            return Tab.FromArray(list.ToArray());
        }

        #endregion
    }
}
