using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.VisualBasic.FileIO;

namespace o2Mate
{
    class LocaleSystem
    {
        #region Public Static Constants
        public static readonly string LocaleDirectory = SpecialDirectories.MyDocuments + "\\CodeCommander\\locales\\";
        #endregion

        #region Public Static Methods
        public static bool ValidateGroupName(string name)
        {
            char[] chars = Path.GetInvalidFileNameChars();
            foreach (char c in chars)
            {
                if (name.IndexOf(c) != -1)
                {
                    throw new ArgumentException("Le nom du groupe contient des caractres invalides");
                }
            }
            if (name.IndexOf(' ') != -1)
            {
                throw new ArgumentException("Le nom du groupe ne doit pas contenir d'espaces");
            }
            return true;
        }

        public static void CreateGroup(string name)
        {
            if (LocaleSystem.ValidateGroupName(name))
            {
                DirectoryInfo dir = new DirectoryInfo(LocaleSystem.LocaleDirectory);
                if (!dir.Exists)
                {
                    dir.Create();
                }
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + name + ".tsv");
                if (!fi.Exists)
                {
                    fi.Create().Close();
                }
            }
        }

        public static void RenameGroup(string name, string newName)
        {
            if (LocaleSystem.ValidateGroupName(name) && LocaleSystem.ValidateGroupName(newName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + name + ".tsv");
                if (fi.Exists)
                {
                    fi.MoveTo(LocaleSystem.LocaleDirectory + newName + ".tsv");
                }
            }
        }

        public static void DeleteGroup(string name)
        {
            if (LocaleSystem.ValidateGroupName(name))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + name + ".tsv");
                if (fi.Exists)
                {
                    fi.Delete();
                }
            }
        }

        public static bool ExistsGroup(string name)
        {
            if (LocaleSystem.ValidateGroupName(name))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + name + ".tsv");
                return fi.Exists;
            }
            return false;
        }

        public static ILocaleSet GetSet(string name)
        {
            if (!ExistsGroup(name))
            {
                CreateGroup(name);
            }
            return new LocaleSet(name);
        }

        public static List<string> GetGroups()
        {
            List<string> list = new List<string>();
            DirectoryInfo dir = new DirectoryInfo(LocaleSystem.LocaleDirectory);
            if (dir.Exists)
            {
                foreach (FileInfo fi in dir.GetFiles("*.tsv"))
                {
                    list.Add(Path.GetFileNameWithoutExtension(fi.Name));
                }
            }
            return list;
        }

        public static void InsertLocale(string groupName, string name, string language, string value)
        {
            if (!LocaleSystem.ExistsGroup(groupName))
            {
                LocaleSystem.CreateGroup(groupName);
            }
            LocaleElement newLe = new LocaleElement(name, language, value);
            bool done = false;
            FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
            List<LocaleElement> list = LocaleSystem.GetAll(groupName);
            StreamWriter sw = new StreamWriter(fi.Open(FileMode.Truncate, FileAccess.Write, FileShare.Write));
            foreach (LocaleElement le in list)
            {
                if (le.Name == name && le.Language == language)
                {
                    done = true;
                }
                le.WriteToFile(sw);
            }
            if (!done)
            {
                newLe.WriteToFile(sw);
            }
            sw.Close();
            sw.Dispose();
        }

        public static void ModifyLocale(string groupName, string name, string language, string value)
        {
            if (!LocaleSystem.ExistsGroup(groupName))
            {
                LocaleSystem.CreateGroup(groupName);
            }
            LocaleElement newLe = new LocaleElement(name, language, value);
            FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
            List<LocaleElement> list = LocaleSystem.GetAll(groupName);
            StreamWriter sw = new StreamWriter(fi.Open(FileMode.Truncate, FileAccess.Write, FileShare.Write));
            foreach (LocaleElement le in list)
            {
                if (le.Name == name && le.Language == language)
                {
                    le.Value = value;
                }
                le.WriteToFile(sw);
            }
            sw.Close();
            sw.Dispose();
        }

        public static void RenameLocale(string oldName, string groupName, string name, string language)
        {
            if (LocaleSystem.ExistsGroup(groupName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
                List<LocaleElement> list = LocaleSystem.GetAll(groupName);
                StreamWriter sw = new StreamWriter(fi.Open(FileMode.Truncate, FileAccess.Write, FileShare.Write));
                foreach (LocaleElement le in list)
                {
                    if (le.Name == oldName && le.Language == language)
                    {
                        le.Name = name;
                    }
                    le.WriteToFile(sw);
                }
                sw.Close();
                sw.Dispose();
            }
        }

        public static void RenameLocale(string oldName, string groupName, string name)
        {
            if (LocaleSystem.ExistsGroup(groupName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
                List<LocaleElement> list = LocaleSystem.GetAll(groupName);
                StreamWriter sw = new StreamWriter(fi.Open(FileMode.Truncate, FileAccess.Write, FileShare.Write));
                foreach (LocaleElement le in list)
                {
                    if (le.Name == oldName)
                    {
                        le.Name = name;
                    }
                    le.WriteToFile(sw);
                }
                sw.Close();
                sw.Dispose();
            }
        }

        public static void RemoveLocale(string groupName, string name, string language)
        {
            if (LocaleSystem.ExistsGroup(groupName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
                List<LocaleElement> list = LocaleSystem.GetAll(groupName);
                StreamWriter sw = new StreamWriter(fi.Open(FileMode.Truncate, FileAccess.Write, FileShare.Write));
                foreach (LocaleElement le in list)
                {
                    if (!(le.Name == name && le.Language == language))
                    {
                        le.WriteToFile(sw);
                    }
                }
                sw.Close();
                sw.Dispose();
            }
        }

        public static void RemoveLocale(string groupName, string name)
        {
            if (LocaleSystem.ExistsGroup(groupName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
                List<LocaleElement> list = LocaleSystem.GetAll(groupName);
                StreamWriter sw = new StreamWriter(fi.Open(FileMode.Truncate, FileAccess.Write, FileShare.Write));
                foreach(LocaleElement le in list)
                {
                    if (!(le.Name == name))
                    {
                        le.WriteToFile(sw);
                    }
                }
                sw.Close();
                sw.Dispose();
            }
        }

        public static bool ExistLocale(string groupName, string name, string language)
        {
            if (LocaleSystem.ExistsGroup(groupName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
                StreamReader sr = new StreamReader(fi.Open(FileMode.Open, FileAccess.Read, FileShare.Read));
                while (!sr.EndOfStream)
                {
                    LocaleElement le = LocaleElement.Parse(sr.ReadLine());
                    if (le.Name == name && le.Language == language)
                    {
                        sr.Close();
                        sr.Dispose();
                        return true;
                    }
                }
                sr.Close();
                sr.Dispose();
            }
            return false;
        }

        public static bool ExistLocale(string groupName, string name)
        {
            if (LocaleSystem.ExistsGroup(groupName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
                StreamReader sr = new StreamReader(fi.Open(FileMode.Open, FileAccess.Read, FileShare.Read));
                while (!sr.EndOfStream)
                {
                    LocaleElement le = LocaleElement.Parse(sr.ReadLine());
                    if (le.Name == name)
                    {
                        sr.Close();
                        sr.Dispose();
                        return true;
                    }
                }
                sr.Close();
                sr.Dispose();
            }
            return false;
        }

        public static LocaleElement GetLocale(string groupName, string name, string language)
        {
            if (LocaleSystem.ExistsGroup(groupName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
                StreamReader sr = new StreamReader(fi.Open(FileMode.Open, FileAccess.Read, FileShare.Read));
                while (!sr.EndOfStream)
                {
                    LocaleElement le = LocaleElement.Parse(sr.ReadLine());
                    if (le.Name == name && le.Language == language)
                    {
                        sr.Close();
                        sr.Dispose();
                        return le;
                    }
                }
                sr.Close();
                sr.Dispose();
            }
            return null;
        }

        public static List<LocaleElement> GetLocale(string groupName, string name)
        {
            List<LocaleElement> list = new List<LocaleElement>();
            if (LocaleSystem.ExistsGroup(groupName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
                StreamReader sr = new StreamReader(fi.Open(FileMode.Open, FileAccess.Read, FileShare.Read));
                while (!sr.EndOfStream)
                {
                    LocaleElement le = LocaleElement.Parse(sr.ReadLine());
                    if (le.Name == name)
                    {
                        list.Add(le);
                    }
                }
                sr.Close();
                sr.Dispose();
            }
            return list;
        }

        public static List<LocaleElement> GetAll(string groupName)
        {
            List<LocaleElement> list = new List<LocaleElement>();
            if (LocaleSystem.ExistsGroup(groupName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
                StreamReader sr = new StreamReader(fi.Open(FileMode.Open, FileAccess.Read, FileShare.Read));
                while (!sr.EndOfStream)
                {
                    LocaleElement le = LocaleElement.Parse(sr.ReadLine());
                    list.Add(le);
                }
                sr.Close();
                sr.Dispose();
            }
            return list;
        }

        public static List<LocaleElement> GetNames(string groupName)
        {
            List<LocaleElement> list = new List<LocaleElement>();
            if (LocaleSystem.ExistsGroup(groupName))
            {
                FileInfo fi = new FileInfo(LocaleSystem.LocaleDirectory + groupName + ".tsv");
                StreamReader sr = new StreamReader(fi.Open(FileMode.Open, FileAccess.Read, FileShare.Read));
                while (!sr.EndOfStream)
                {
                    LocaleElement le = LocaleElement.Parse(sr.ReadLine());
                    if (!list.Exists(new Predicate<LocaleElement>(delegate(LocaleElement loc) { return loc.Name == le.Name; })))
                    {
                        list.Add(le);
                    }
                }
                sr.Close();
                sr.Dispose();
            }
            return list;
        }
        #endregion
    }
}
