using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;

namespace o2Mate
{
    [Guid("D68D058B-43F5-47e1-8055-C7F6CA809E78")]
    [CoClass(typeof(IScope))]
    [InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
    public interface IScope
    {
        void Add(string name, string value, string belongsTo);
        IData Item(int index);
        IEnumerator GetEnumerator();
        bool Exists(string name);
        IData GetVariable(string name);
        void Remove(string name);
        void Push();
        void Pop();
        IScope Parent { get; set; }
    }

    [ClassInterface(ClassInterfaceType.None)]
    public class Scope : IScope, IEnumerable
    {
        #region Private Fields
        private List<IData> scope;
        private IScope parent;
        #endregion

        #region Default Constructor
        public Scope()
        {
            this.parent = null;
            this.scope = new List<IData>();
            this.Add("true", "1", "principal");
            this.scope[0].RealName = "trueValue";
            this.Add("false", "0", "principal");
            this.scope[1].RealName = "falseValue";
            this.Add("SpaceChar", " ", "principal");
            this.Add("PlusChar", "+", "principal");
            this.Add("MinusChar", "-", "principal");
            this.Add("StarChar", "*", "principal");
            this.Add("SlashChar", "/", "principal");
            this.Add("ColonChar", ":", "principal");
            this.Add("CommaChar", ",", "principal");
            this.Add("DotChar", ".", "principal");
            this.Add("ExChar", "!", "principal");
            this.Add("IntChar", "?", "principal");
            this.Add("EmptyChar", "", "principal");
            this.Add("InfChar", "<", "principal");
            this.Add("SupChar", ">", "principal");
            this.Add("CrLf", Environment.NewLine, "principal");
        }
        #endregion

        #region IScope Membres

        public IScope Parent
        {
            get { return this.parent; }
            set { this.parent = value; }
        }

        public void AddReference(string name, string reference, string belongsTo)
        {
            Reference r = new Reference(belongsTo);
            r.Name = name;
            r.Value = reference;
            this.scope.Add(r);
        }

        public void Add(string name, string value, string belongsTo)
        {
            Variable var = new Variable(belongsTo);
            var.Name = name;
            var.Value = value;
            this.scope.Add(var);
        }

        public IData Item(int index)
        {
            return this.scope[index];
        }

        public bool Exists(string name)
        {
            for (int index = this.scope.Count - 1; index >= 0; --index)
            {
                IData data = this.scope[index];
                if (!data.IsSeparator)
                {
                    if (data.Name == name)
                    {
                        return true;
                    }
                }
            }
            if (this.parent != null)
            {
                return this.parent.Exists(name);
            }
            return false;
        }

        public IData GetVariable(string name)
        {
            for (int index = this.scope.Count - 1; index >= 0; --index)
            {
                IData data = this.scope[index];
                if (!data.IsSeparator)
                {
                    if (data.Name == name)
                    {
                        if (data is Reference)
                        {
                            // on change le nom de recherche et on continue la recherche en remontant
                            name = data.ValueString;
                        }
                        else
                        {
                            return data;
                        }
                    }
                }
            }
            if (this.parent != null)
            {
                return this.parent.GetVariable(name);
            }
            return null;
        }

        public void Remove(string name)
        {
            for (int index = this.scope.Count - 1; index >= 0; --index)
            {
                IData data = this.scope[index];
                if (!data.IsSeparator)
                {
                    if (data.Name == name)
                    {
                        this.scope.Remove(data);
                        return;
                    }
                }
            }
            if (this.parent != null)
            {
                this.parent.Remove(name);
            }
        }

        public void Push()
        {
            Separator sep = new Separator();
            this.scope.Add(sep);
        }

        public void Pop()
        {
            for (int index = this.scope.Count - 1; index >= 0; --index)
            {
                IData data = this.scope[index];
                this.scope.RemoveAt(index);
                if (data.IsSeparator)
                {
                    return;
                }
            }
        }

        #endregion

        #region IEnumerable Membres

        public IEnumerator GetEnumerator()
        {
            return this.scope.GetEnumerator();
        }

        #endregion
    }
}
